@@ -14,8 +14,8 @@
 ;; along with this program; if not, contact:
 ;;
 ;; Free Software Foundation           Voice:  +1-617-542-5942
-;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
-;; Boston, MA  02111-1307,  USA       gnu@gnu.org
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 
 (define (gnc:make-option
          ;; The category of this option
@@ -121,7 +121,7 @@
 (define (gnc:option-value-validator option)  
   (vector-ref option 11))
 (define (gnc:option-data option)
-    (vector-ref option 12))
+  (vector-ref option 12))
 (define (gnc:option-data-fns option)
   (vector-ref option 13))
 
@@ -282,7 +282,7 @@
     (if (string? currency)
         (gnc:commodity-table-lookup
          (gnc:book-get-commodity-table (gnc:get-current-book))
-         GNC_COMMODITY_NS_ISO currency)
+         GNC_COMMODITY_NS_CURRENCY currency)
         currency))
 
    (let* ((value (currency->scm default-value))
@@ -301,6 +301,55 @@
       (lambda (x) (list #t x))
       #f #f #f #f)))
 
+;; budget option
+;; TODO: need to double-check this proc
+(define (gnc:make-budget-option
+	 section
+	 name
+	 sort-tag
+         documentation-string)
+
+  (define (budget->guid budget)
+    (if (string? budget)
+        budget
+        (gnc:budget-get-guid budget)))
+
+  (define (guid->budget budget)
+    (if (string? budget)
+        (gnc:budget-lookup budget (gnc:get-current-book))
+        budget))
+
+  (let* ((default-value (gnc:budget-get-default (gnc:get-current-book)))
+         (value (budget->guid default-value))
+         (option-set #f)
+         (value->string (lambda ()
+                          (string-append
+                           "'" (gnc:value->string (if option-set option #f)))))
+
+         )
+    (gnc:make-option
+     section name sort-tag 'budget documentation-string
+     ;; the getter should always return a budget pointer
+     (lambda () (guid->budget  ;; getter
+                 (if option-set
+                     value
+                     default-value))
+             )
+
+     (lambda (x) (set! value (budget->guid x))
+             (set! option-set #t)) ;; setter
+     (lambda ()
+       (guid->budget
+        (gnc:budget-get-default (gnc:get-current-book)))) ;; default-getter
+     (gnc:restore-form-generator value->string) ;; ??
+     (lambda (f p) (gnc:kvp-frame-set-slot-path f value p))
+     (lambda (f p)
+       (let ((v (gnc:kvp-frame-get-slot-path f p)))
+         (if (and v (string? v))
+             (set! value v))))
+     (lambda (x) (list #t x)) ;; value-validator
+     #f #f #f #f)))
+
 ;; commodity options use a specialized widget for entering commodities
 ;; in the GUI implementation.
 (define (gnc:make-commodity-option
@@ -313,7 +362,7 @@
   (define (commodity->scm commodity)
     (if (string? commodity)
         (list 'commodity-scm
-              GNC_COMMODITY_NS_ISO
+              GNC_COMMODITY_NS_CURRENCY
               commodity)
         (list 'commodity-scm
               (gnc:commodity-get-namespace commodity)
@@ -427,7 +476,9 @@
              (list #f "pixmap-option: not a string"))))
      #f #f #f #f)))
 
-;; subtype should be on of 'relative 'absolute or 'both
+;; show-time is boolean
+;; subtype should be one of 'relative 'absolute or 'both
+;; if subtype is 'absolute then relative-date-list should be #f
 ;; relative-date-list should be the list of relative dates permitted
 ;; gnc:all-relative-dates contains a list of all relative dates.
 
@@ -1108,13 +1159,17 @@
     (inexact->exact
      (min 255.0
           (truncate (* (/ 255.0 range) value)))))
+  (define (number->hex-string number)
+    (let ((ret (number->string number 16)))
+      (cond ((< (string-length ret) 2) (string-append "0" ret))
+            (else ret))))
   (let ((red (car color))
         (green (cadr color))
         (blue (caddr color)))
     (string-append
-     (number->string (html-value red) 16)
-     (number->string (html-value green) 16)
-     (number->string (html-value blue) 16))))
+     (number->hex-string (html-value red))
+     (number->hex-string (html-value green))
+     (number->hex-string (html-value blue)))))
 
 (define (gnc:color->html color range)
     (string-append "#"
@@ -1130,6 +1185,65 @@
         (range (car (gnc:option-data color-option))))
     (gnc:color->hex-string color range)))
 
+;;
+;; dateformat option
+;;
+(define (gnc:make-dateformat-option
+	 section
+	 name
+	 sort-tag
+	 documentation-string
+	 default-value)
+
+  (define (def-value)
+    (if (list? default-value)
+	default-value
+	'(locale number #t "")))
+
+  (let* ((value (def-value))
+         (value->string (lambda () 
+                          (string-append "'" (gnc:value->string value)))))
+    (gnc:make-option
+     section name sort-tag 'dateformat documentation-string
+     (lambda () value)
+     (lambda (x) (set! value x))
+     (lambda () (def-value))
+     (gnc:restore-form-generator value->string)
+     (lambda (f p)
+       (gnc:kvp-frame-set-slot-path
+	f (symbol->string (car value)) (append p '("fmt")))
+       (gnc:kvp-frame-set-slot-path
+	f (symbol->string (cadr value)) (append p '("month")))
+       (gnc:kvp-frame-set-slot-path
+	f (if (caddr value) 1 0) (append p '("years")))
+       (gnc:kvp-frame-set-slot-path f (cadddr value) (append p '("custom"))))
+     (lambda (f p)
+       (let ((fmt (gnc:kvp-frame-get-slot-path f (append p '("fmt"))))
+	     (month (gnc:kvp-frame-get-slot-path f (append p '("month"))))
+	     (years (gnc:kvp-frame-get-slot-path f (append p '("years"))))
+	     (custom (gnc:kvp-frame-get-slot-path f (append p '("custom")))))
+	 (if (and
+	      fmt (string? fmt)
+	      month (string? month)
+	      years (number? years)
+	      custom (string? custom))
+	     (set! value (list (string->symbol fmt) (string->symbol month)
+			       (if (= years 0) #f #t) custom)))))
+     (lambda (x)
+       (cond ((not (list? x)) (list #f "dateformat-option: not a list"))
+	     ((not (= (length x) 4))
+	      (list #f "dateformat-option: wrong list length" (length x)))
+	     ((not (symbol? (car x)))
+	      (list #f "dateformat-option: no format symbol"))
+	     ((not (symbol? (cadr x)))
+	      (list #f "dateformat-option: no months symbol"))
+	     ((not (string? (cadddr x)))
+	      (list #f "dateformat-option: no custom string"))
+	     (else (list #t x))))
+     #f #f #f #f)))
+
+(define (gnc:dateformat-get-format v)
+  (cadddr v))
 
 ;; Create a new options database
 (define (gnc:new-options)
@@ -1423,9 +1537,8 @@
       end-current-quarter 
       end-prev-quarter
       end-cal-year 
-      end-prev-year 
-      end-cur-fin-year
-      end-prev-fin-year
+      end-prev-year
+      end-accounting-period
       ))))
 
 (define (gnc:options-make-date-interval! options pagename name-from info-from
@@ -1445,8 +1558,7 @@
       start-prev-quarter
       start-cal-year 
       start-prev-year
-      start-cur-fin-year 
-      start-prev-fin-year
+      start-accounting-period
       )))
   (gnc:options-make-end-date! options pagename name-to
 			      (string-append sort-tag "b") info-to))
