@@ -16,8 +16,8 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
  *******************************************************************/
 
@@ -28,9 +28,9 @@
 
 #include "Group.h"
 #include "SchedXactionP.h"
+#include "SX-book.h"
 
 #include "gnc-xml-helper.h"
-#include "gnc-engine-util.h"
 
 #include "sixtp.h"
 #include "sixtp-utils.h"
@@ -46,7 +46,7 @@
 
 #include "sixtp-dom-parsers.h"
 
-static short module = MOD_SX;
+static QofLogModule log_module = GNC_MOD_SX;
 
 /**
  * The XML output should look something like:
@@ -154,20 +154,20 @@ gnc_schedXaction_dom_tree_create(SchedXaction *sx)
     templ_acc_guid = xaccAccountGetGUID(sx->template_acct);
 
     /* FIXME: this should be the same as the def in io-gncxml-v2.c */
-    ret = xmlNewNode( NULL, GNC_SCHEDXACTION_TAG );
+    ret = xmlNewNode( NULL, BAD_CAST GNC_SCHEDXACTION_TAG );
 
-    xmlSetProp( ret, "version", schedxaction_version_string );
+    xmlSetProp( ret, BAD_CAST "version", BAD_CAST schedxaction_version_string );
 
     xmlAddChild( ret,
                  guid_to_dom_tree(SX_ID,
                                   xaccSchedXactionGetGUID(sx)) );
 
-    xmlNewTextChild( ret, NULL, SX_NAME, xaccSchedXactionGetName(sx) );
+    xmlNewTextChild( ret, NULL, BAD_CAST SX_NAME, BAD_CAST xaccSchedXactionGetName(sx) );
 
-    xmlNewTextChild( ret, NULL, SX_AUTOCREATE,
-                     ( sx->autoCreateOption ? "y" : "n" ) );
-    xmlNewTextChild( ret, NULL, SX_AUTOCREATE_NOTIFY,
-                     ( sx->autoCreateNotify ? "y" : "n" ) );
+    xmlNewTextChild( ret, NULL, BAD_CAST SX_AUTOCREATE,
+                     BAD_CAST ( sx->autoCreateOption ? "y" : "n" ) );
+    xmlNewTextChild( ret, NULL, BAD_CAST SX_AUTOCREATE_NOTIFY,
+                     BAD_CAST ( sx->autoCreateNotify ? "y" : "n" ) );
     xmlAddChild(ret, int_to_dom_tree(SX_ADVANCE_CREATE_DAYS,
                                      sx->advanceCreateDays));
     xmlAddChild(ret, int_to_dom_tree(SX_ADVANCE_REMIND_DAYS,
@@ -205,7 +205,7 @@ gnc_schedXaction_dom_tree_create(SchedXaction *sx)
 				  templ_acc_guid));
 				  
     /* output freq spec */
-    fsNode = xmlNewNode(NULL, SX_FREQSPEC);
+    fsNode = xmlNewNode(NULL, BAD_CAST SX_FREQSPEC);
     xmlAddChild( fsNode,
                  gnc_freqSpec_dom_tree_create(
                          xaccSchedXactionGetFreqSpec(sx)) );
@@ -220,7 +220,7 @@ gnc_schedXaction_dom_tree_create(SchedXaction *sx)
             for ( l = gnc_sx_get_defer_instances( sx ); l; l = l->next ) {
                     tsd = (temporalStateData*)l->data;
 
-                    instNode = xmlNewNode( NULL, SX_DEFER_INSTANCE );
+                    instNode = xmlNewNode( NULL, BAD_CAST SX_DEFER_INSTANCE );
                     if ( g_date_valid( &tsd->last_date ) )
                     {
                       xmlAddChild( instNode, gdate_to_dom_tree( SX_LAST,
@@ -251,7 +251,7 @@ gnc_schedXaction_dom_tree_create(SchedXaction *sx)
 struct sx_pdata
 {
   SchedXaction *sx;
-  GNCBook *book;
+  QofBook *book;
 };
 
 static
@@ -263,7 +263,7 @@ sx_id_handler( xmlNodePtr node, gpointer sx_pdata )
     GUID        *tmp = dom_tree_to_guid( node );
 
     g_return_val_if_fail( tmp, FALSE );
-    xaccSchedXactionSetGUID(sx, *tmp);
+    xaccSchedXactionSetGUID(sx, tmp);
     g_free( tmp );
 
     return TRUE;
@@ -634,10 +634,10 @@ gnc_schedXaction_end_handler(gpointer data_for_children,
     if ( sx->template_acct == NULL )
     {
             AccountGroup *ag = NULL;
-            char *id = NULL;
+            const char *id = NULL;
             Account *acct = NULL;
             sixtp_gdv2 *sixdata = gdata->parsedata;
-            GNCBook *book;
+            QofBook *book;
 
             book = sixdata->book;
 
@@ -645,6 +645,7 @@ gnc_schedXaction_end_handler(gpointer data_for_children,
                change re: storing template accounts. */
             /* Fix: get account with name of our GUID from the template
                accounts group.  Make that our template_acct pointer. */
+            /* THREAD-UNSAFE */
             id = guid_to_string( xaccSchedXactionGetGUID( sx ) );
             ag = gnc_book_get_template_group(book);
             if ( ag == NULL )
@@ -667,15 +668,13 @@ gnc_schedXaction_end_handler(gpointer data_for_children,
                    xaccAccountGetName( acct ), id );
 
             /* FIXME: free existing template account. 
-	     *  HUH????? We only execute this if there isn't
-	     * currently an existing template account, don't we?
-	     * <rgmerk>
-	     */
+             *  HUH????? We only execute this if there isn't
+             * currently an existing template account, don't we?
+             * <rgmerk>
+             */
 
             sx->template_acct = acct;
     }
-
-    xaccSchedXactionSetDirtyness(sx, FALSE);
     xmlFreeNode( tree );
 
     return successful;
@@ -713,7 +712,8 @@ tt_act_handler( xmlNodePtr node, gpointer data )
                            away at some point, but the same concern still
                            applies for
                            SchedXaction.c:xaccSchedXactionInit... */
-                        com = gnc_commodity_new( "template", "template",
+                        com = gnc_commodity_new( txd->book,
+						 "template", "template",
                                                  "template", "template",
                                                  1 );
                         xaccAccountSetCommodity( acc, com );
@@ -737,21 +737,6 @@ tt_trn_handler( xmlNodePtr node, gpointer data )
         if ( trn == NULL ) {
                 return FALSE;
         } else {
-                /* Handle old data saved without a 'Posted' date... new data
-                 * is created correctly. */
-                Timespec ts;
-                time_t tt;
-                xaccTransGetDatePostedTS( trn, &ts );
-                tt = timespecToTime_t( ts );
-                if ( tt == 0 )
-                {
-                        // re-use 'tt'.
-                        xaccTransBeginEdit( trn );
-                        xaccTransGetDateEnteredTS( trn, &ts );
-                        tt = timespecToTime_t( ts );
-                        xaccTransSetDatePostedSecs( trn, tt );
-                        xaccTransCommitEdit( trn );
-                }
                 txd->transactions = g_list_append( txd->transactions, trn );
         }
 
@@ -776,7 +761,7 @@ gnc_template_transaction_end_handler(gpointer data_for_children,
         gboolean   successful = FALSE;
         xmlNodePtr tree = data_for_children;
         gxpf_data  *gdata = global_data;
-        GNCBook    *book = gdata->bookdata;
+        QofBook    *book = gdata->bookdata;
         GList      *n;
         gnc_template_xaction_data txd;
 
