@@ -16,13 +16,10 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
-
-#define _GNU_SOURCE
-
 #include "config.h"
 
 #include <glib.h>
@@ -37,9 +34,8 @@
 #include "Group.h"
 #include "GroupP.h"
 #include "gnc-commodity.h"
-#include "gnc-engine-util.h"
-#include "gnc-event.h"
-#include "guid.h"
+#include "gnc-engine.h"
+#include "gnc-pricedb.h"
 #include "Transaction.h"
 #include "TransactionP.h"
  
@@ -53,7 +49,7 @@
 
 #include "putil.h"
 
-static short module = MOD_TXN; 
+static QofLogModule log_module = GNC_MOD_TXN;
 
 /* ============================================================= */
 /* ============================================================= */
@@ -177,7 +173,7 @@ pgendStoreTransactionNoLock (PGBackend *be, Transaction *trans,
         {
           Split *s = split_node->data;
 
-          if (s && guid_equal (&s->guid, &dti->guid))
+          if (s && guid_equal (&s->inst.entity.guid, &dti->guid))
           {
             pgendStoreAuditSplit (be, s, SQL_DELETE);
             break;
@@ -217,8 +213,8 @@ pgendStoreTransactionNoLock (PGBackend *be, Transaction *trans,
    /* Update the rest */
    start = xaccTransGetSplitList(trans);
 
-   PINFO ("split-list=%p, do_free=%d", start, trans->do_free);
-   if ((start) && !(trans->do_free))
+   PINFO ("split-list=%p, do_free=%d", start, trans->inst.do_free);
+   if ((start) && !(trans->inst.do_free))
    {
       gnc_commodity *com;
 
@@ -234,7 +230,7 @@ pgendStoreTransactionNoLock (PGBackend *be, Transaction *trans,
          if (s->idata)
          {
            pgendKVPDelete (be, s->idata);
-           pgendKVPStore (be, s->idata, s->kvp_data);
+           pgendKVPStore (be, s->idata, s->inst.kvp_data);
          }
       }
 
@@ -254,7 +250,7 @@ pgendStoreTransactionNoLock (PGBackend *be, Transaction *trans,
       if (trans->idata)
       {
         pgendKVPDelete (be, trans->idata);
-        pgendKVPStore (be, trans->idata, trans->kvp_data);
+        pgendKVPStore (be, trans->idata, trans->inst.kvp_data);
       }
    }
    else
@@ -472,14 +468,14 @@ pgendCopySplitsToEngine (PGBackend *be, Transaction *trans)
             s = pgendSplitLookup (be, &guid);
             if (!s)
             {
-               s = xaccMallocSplit(trans->book);
+               s = xaccMallocSplit(trans->inst.book);
                xaccSplitSetGUID(s, &guid);
             }
 
             /* next, restore all split data */
             xaccSplitSetMemo(s, DB_GET_VAL("memo",j));
             xaccSplitSetAction(s, DB_GET_VAL("action",j));
-            ts = gnc_iso8601_to_timespec_local
+            ts = gnc_iso8601_to_timespec_gmt
               (DB_GET_VAL("date_reconciled",j));
             xaccSplitSetDateReconciledTS (s, &ts);
 
@@ -595,11 +591,16 @@ pgendCopySplitsToEngine (PGBackend *be, Transaction *trans)
      }
      else
      {
+       gchar str1[GUID_ENCODING_LENGTH+1];
+       gchar str2[GUID_ENCODING_LENGTH+1];
+
+       guid_to_string_buff(xaccSplitGetGUID (sri->split), str1);
+       guid_to_string_buff(&sri->account_guid, str2);
+
        PERR ("account not found, will delete this split\n"
              "\t(split with  guid=%s\n" 
              "\twants an acct with guid=%s)\n", 
-             guid_to_string(xaccSplitGetGUID (sri->split)),
-             guid_to_string(&sri->account_guid));
+	     str1, str2);
 
        /* Remove the split from the list */
        db_splits = g_list_remove (db_splits, sri->split);
@@ -656,7 +657,7 @@ pgendCopyTransactionToEngine (PGBackend *be, const GUID *trans_guid)
    if (!be || !trans_guid) return 0;
 
    /* disable callbacks into the backend, and events to GUI */
-   gnc_engine_suspend_events();
+   qof_event_suspend();
    pgendDisable(be);
 
    /* first, see if we already have such a transaction */
@@ -675,7 +676,7 @@ pgendCopyTransactionToEngine (PGBackend *be, const GUID *trans_guid)
       {
          PINFO ("fresh data, skip check");
          pgendEnable(be);
-         gnc_engine_resume_events();
+         qof_event_resume();
          return 0;
       }
    }
@@ -704,9 +705,9 @@ pgendCopyTransactionToEngine (PGBackend *be, const GUID *trans_guid)
        /* I beleive its a programming error to get this case.
         * Print a warning for now... */
        PERR ("no such transaction in the database. This is unexpected ...\n");
-       xaccBackendSetError (&be->be, ERR_SQL_MISSING_DATA);
+       qof_backend_set_error (&be->be, ERR_SQL_MISSING_DATA);
        pgendEnable(be);
-       gnc_engine_resume_events();
+       qof_event_resume();
        return 0;
      }
 
@@ -717,9 +718,9 @@ pgendCopyTransactionToEngine (PGBackend *be, const GUID *trans_guid)
        PERR ("!!!!!!!!!!!SQL database is corrupt!!!!!!!\n"
              "too many transactions with GUID=%s\n",
              guid_to_string (trans_guid));
-       xaccBackendSetError (&be->be, ERR_BACKEND_DATA_CORRUPT);
+       qof_backend_set_error (&be->be, ERR_BACKEND_DATA_CORRUPT);
        pgendEnable(be);
-       gnc_engine_resume_events();
+       qof_event_resume();
        return 0;
      }
 
@@ -768,9 +769,9 @@ pgendCopyTransactionToEngine (PGBackend *be, const GUID *trans_guid)
        if (do_set_guid) xaccTransSetGUID (trans, trans_guid);
        xaccTransSetNum (trans, DB_GET_VAL("num",j));
        xaccTransSetDescription (trans, DB_GET_VAL("description",j));
-       ts = gnc_iso8601_to_timespec_local (DB_GET_VAL("date_posted",j));
+       ts = gnc_iso8601_to_timespec_gmt (DB_GET_VAL("date_posted",j));
        xaccTransSetDatePostedTS (trans, &ts);
-       ts = gnc_iso8601_to_timespec_local (DB_GET_VAL("date_entered",j));
+       ts = gnc_iso8601_to_timespec_gmt (DB_GET_VAL("date_entered",j));
        xaccTransSetDateEnteredTS (trans, &ts);
        xaccTransSetVersion (trans, atoi(DB_GET_VAL("version",j)));
        xaccTransSetCurrency (trans, currency);
@@ -787,7 +788,7 @@ pgendCopyTransactionToEngine (PGBackend *be, const GUID *trans_guid)
    if (0 <= engine_data_is_newer) 
    {
       pgendEnable(be);
-      gnc_engine_resume_events();
+      qof_event_resume();
       return engine_data_is_newer;
    }
 
@@ -803,13 +804,13 @@ pgendCopyTransactionToEngine (PGBackend *be, const GUID *trans_guid)
 
    if (0 != trans->idata)
    {
-      if (!kvp_frame_is_empty (trans->kvp_data))
+      if (!kvp_frame_is_empty (trans->inst.kvp_data))
       {
-        kvp_frame_delete (trans->kvp_data);
-        trans->kvp_data = kvp_frame_new ();
+        kvp_frame_delete (trans->inst.kvp_data);
+        trans->inst.kvp_data = kvp_frame_new ();
       }
 
-      trans->kvp_data = pgendKVPFetch (be, trans->idata, trans->kvp_data);
+      trans->inst.kvp_data = pgendKVPFetch (be, trans->idata, trans->inst.kvp_data);
    }
 
    engine_splits = xaccTransGetSplitList(trans);
@@ -818,13 +819,13 @@ pgendCopyTransactionToEngine (PGBackend *be, const GUID *trans_guid)
       Split *s = node->data;
       if (0 != s->idata)
       {
-         if (!kvp_frame_is_empty (s->kvp_data))
+         if (!kvp_frame_is_empty (s->inst.kvp_data))
          {
-           kvp_frame_delete (s->kvp_data);
-           s->kvp_data = kvp_frame_new ();
+           kvp_frame_delete (s->inst.kvp_data);
+           s->inst.kvp_data = kvp_frame_new ();
          }
 
-         s->kvp_data = pgendKVPFetch (be, s->idata, s->kvp_data);
+         s->inst.kvp_data = pgendKVPFetch (be, s->idata, s->inst.kvp_data);
       }
    }
 
@@ -834,7 +835,7 @@ pgendCopyTransactionToEngine (PGBackend *be, const GUID *trans_guid)
 
    /* re-enable events to the backend and GUI */
    pgendEnable(be);
-   gnc_engine_resume_events();
+   qof_event_resume();
 
    LEAVE (" ");
    return -1;
@@ -871,7 +872,7 @@ pgendSyncTransaction (PGBackend *be, GUID *trans_guid)
    if (!be || !trans_guid) return;
 
    /* disable callbacks into the backend, and events to GUI */
-   gnc_engine_suspend_events();
+   qof_event_suspend();
    pgendDisable(be);
 
    engine_data_is_newer = xxxpgendCopyTransactionToEngine (be, trans_guid);
@@ -893,13 +894,13 @@ pgendSyncTransaction (PGBackend *be, GUID *trans_guid)
        * framework */
       pgendStoreTransaction (be, trans);
 
-      gnc_engine_resume_events();
+      qof_event_resume();
       return;
    }
 
    /* re-enable events to the backend and GUI */
    pgendEnable(be);
-   gnc_engine_resume_events();
+   qof_event_resume();
 
    LEAVE (" ");
 }
@@ -917,7 +918,7 @@ pgendSyncTransaction (PGBackend *be, GUID *trans_guid)
 /* ============================================================= */
 
 void
-pgend_trans_commit_edit (Backend * bend, 
+pgend_trans_commit_edit (QofBackend * bend, 
                          Transaction * trans,
                          Transaction * oldtrans)
 {
@@ -997,7 +998,7 @@ pgend_trans_commit_edit (Backend * bend,
                   "\ttransaction is '%s' %s\n",
                   xaccTransGetDescription (trans), buf);
             rollback = 0;
-            trans->do_free = TRUE;
+            trans->inst.do_free = TRUE;
          }
          else
          {
@@ -1019,7 +1020,7 @@ pgend_trans_commit_edit (Backend * bend,
           * routine.  Our rollback routine updates from the latest in 
           * the sql database, and voila! we are good to go. 
           */
-         xaccBackendSetError (&be->be, ERR_BACKEND_MODIFIED);
+         qof_backend_set_error (&be->be, ERR_BACKEND_MODIFIED);
          return;
       } 
    }
@@ -1082,7 +1083,7 @@ pgend_trans_commit_edit (Backend * bend,
  */
 
 void
-pgend_trans_rollback_edit (Backend * bend, Transaction * trans)
+pgend_trans_rollback_edit (QofBackend * bend, Transaction * trans)
 {
    PGBackend *be = (PGBackend *)bend;
    const GUID * trans_guid;
@@ -1096,7 +1097,7 @@ pgend_trans_rollback_edit (Backend * bend, Transaction * trans)
    if (-1 < pgendTransactionGetDeletedVersion (be, trans))
    {
       LEAVE ("destroyed");
-      xaccBackendSetError (&be->be, ERR_BACKEND_MOD_DESTROY);
+      qof_backend_set_error (&be->be, ERR_BACKEND_MOD_DESTROY);
       return;
    }
 
