@@ -17,8 +17,8 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
 \********************************************************************/
 
@@ -41,16 +41,13 @@
 #include "io-gncxml-gen.h"
 #include "io-gncxml-v2.h"
 
+#include "gncEntry.h"
 #include "gncTaxTableP.h"
 #include "gnc-tax-table-xml-v2.h"
-#include "gnc-engine-util.h"
 
-#include "gncObject.h"
-#include "gncEntry.h"
+#define _GNC_MOD_NAME	GNC_ID_TAXTABLE
 
-#define _GNC_MOD_NAME	GNC_TAXTABLE_MODULE_NAME
-
-static short module = MOD_IO;
+static QofLogModule log_module = GNC_MOD_IO;
 
 const gchar *taxtable_version_string = "2.0.0";
 
@@ -74,7 +71,8 @@ static void
 maybe_add_guid (xmlNodePtr ptr, const char *tag, GncTaxTable *table)
 {
   if (table)
-    xmlAddChild (ptr, guid_to_dom_tree (tag, gncTaxTableGetGUID (table)));
+    xmlAddChild (ptr, guid_to_dom_tree (tag, 
+            qof_instance_get_guid(QOF_INSTANCE(table))));
 }
 
 static xmlNodePtr
@@ -84,12 +82,12 @@ ttentry_dom_tree_create (GncTaxTableEntry *entry)
   Account *account;
   gnc_numeric amount;
 
-  ret = xmlNewNode(NULL, gnc_taxtableentry_string);
+  ret = xmlNewNode(NULL, BAD_CAST gnc_taxtableentry_string);
 
   account = gncTaxTableEntryGetAccount (entry);
   if (account)
     xmlAddChild(ret, guid_to_dom_tree (ttentry_account_string,
-				       xaccAccountGetGUID (account)));
+				       qof_instance_get_guid (QOF_INSTANCE(account))));
 
   amount = gncTaxTableEntryGetAmount (entry);
   xmlAddChild (ret, gnc_numeric_to_dom_tree (ttentry_amount_string, &amount));
@@ -107,8 +105,8 @@ taxtable_dom_tree_create (GncTaxTable *table)
     xmlNodePtr ret, entries;
     GList *list;
 
-    ret = xmlNewNode(NULL, gnc_taxtable_string);
-    xmlSetProp(ret, "version", taxtable_version_string);
+    ret = xmlNewNode(NULL, BAD_CAST gnc_taxtable_string);
+    xmlSetProp(ret, BAD_CAST "version", BAD_CAST taxtable_version_string);
 
     maybe_add_guid(ret, taxtable_guid_string, table);
     xmlAddChild(ret, text_to_dom_tree (taxtable_name_string,
@@ -125,7 +123,7 @@ taxtable_dom_tree_create (GncTaxTable *table)
 
     maybe_add_guid(ret, taxtable_parent_string, gncTaxTableGetParent (table));
 
-    entries = xmlNewChild (ret, NULL, taxtable_entries_string, NULL);
+    entries = xmlNewChild (ret, NULL, BAD_CAST taxtable_entries_string, NULL);
     for (list = gncTaxTableGetEntries (table); list; list = list->next) {
       GncTaxTableEntry *entry = list->data;
       xmlAddChild(entries, ttentry_dom_tree_create (entry));
@@ -139,7 +137,7 @@ taxtable_dom_tree_create (GncTaxTable *table)
 struct ttentry_pdata
 {
   GncTaxTableEntry *ttentry;
-  GNCBook *book;
+  QofBook *book;
 };
 
 static gboolean
@@ -199,7 +197,7 @@ static struct dom_tree_handler ttentry_handlers_v2[] = {
 };
 
 static GncTaxTableEntry*
-dom_tree_to_ttentry (xmlNodePtr node, GNCBook *book)
+dom_tree_to_ttentry (xmlNodePtr node, QofBook *book)
 {
   struct ttentry_pdata ttentry_pdata;
   gboolean successful;
@@ -224,7 +222,7 @@ dom_tree_to_ttentry (xmlNodePtr node, GNCBook *book)
 struct taxtable_pdata
 {
   GncTaxTable *table;
-  GNCBook *book;
+  QofBook *book;
 };
 
 static gboolean
@@ -341,10 +339,10 @@ taxtable_entries_handler (xmlNodePtr node, gpointer taxtable_pdata)
   for (mark = node->xmlChildrenNode; mark; mark = mark->next) {
     GncTaxTableEntry *entry;
         
-    if (safe_strcmp ("text", mark->name) == 0)
+    if (safe_strcmp ("text", (char*)mark->name) == 0)
       continue;
 
-    if (safe_strcmp (gnc_taxtableentry_string, mark->name))
+    if (safe_strcmp (gnc_taxtableentry_string, (char*)mark->name))
       return FALSE;
 
     entry = dom_tree_to_ttentry (mark, pdata->book);
@@ -377,7 +375,7 @@ static struct dom_tree_handler taxtable_handlers_v2[] = {
 };
 
 static GncTaxTable*
-dom_tree_to_taxtable (xmlNodePtr node, GNCBook *book)
+dom_tree_to_taxtable (xmlNodePtr node, QofBook *book)
 {
   struct taxtable_pdata taxtable_pdata;
   gboolean successful;
@@ -411,7 +409,7 @@ gnc_taxtable_end_handler(gpointer data_for_children,
     GncTaxTable *table;
     xmlNodePtr tree = (xmlNodePtr)data_for_children;
     gxpf_data *gdata = (gxpf_data*)global_data;
-    GNCBook *book = gdata->bookdata;
+    QofBook *book = gdata->bookdata;
 
     successful = TRUE;
 
@@ -447,25 +445,25 @@ taxtable_sixtp_parser_create(void)
 }
 
 static void
-do_count (gpointer table_p, gpointer count_p)
+do_count (QofEntity * table_p, gpointer count_p)
 {
   int *count = count_p;
   (*count)++;
 }
 
 static int
-taxtable_get_count (GNCBook *book)
+taxtable_get_count (QofBook *book)
 {
   int count = 0;
-  gncObjectForeach (_GNC_MOD_NAME, book, do_count, (gpointer) &count);
+  qof_object_foreach (_GNC_MOD_NAME, book, do_count, (gpointer) &count);
   return count;
 }
 
 static void
-xml_add_taxtable (gpointer table_p, gpointer out_p)
+xml_add_taxtable (QofEntity * table_p, gpointer out_p)
 {
   xmlNodePtr node;
-  GncTaxTable *table = table_p;
+  GncTaxTable *table = (GncTaxTable *) table_p;
   FILE *out = out_p;
 
   node = taxtable_dom_tree_create (table);
@@ -475,9 +473,9 @@ xml_add_taxtable (gpointer table_p, gpointer out_p)
 }
 
 static void
-taxtable_write (FILE *out, GNCBook *book)
+taxtable_write (FILE *out, QofBook *book)
 {
-  gncObjectForeach (_GNC_MOD_NAME, book, xml_add_taxtable, (gpointer) out);
+  qof_object_foreach (_GNC_MOD_NAME, book, xml_add_taxtable, (gpointer) out);
 }
 
 
@@ -522,9 +520,9 @@ taxtable_find_senior (GncTaxTable *table)
 
 /* build a list of tax tables that are grandchildren or bogus (empty entry list). */
 static void
-taxtable_scrub_cb (gpointer table_p, gpointer list_p)
+taxtable_scrub_cb (QofEntity * table_p, gpointer list_p)
 {
-  GncTaxTable *table = table_p;
+  GncTaxTable *table = GNC_TAXTABLE(table_p);
   GList **list = list_p;
 
   if (taxtable_is_grandchild(table) || gncTaxTableGetEntries(table) == NULL)
@@ -535,10 +533,10 @@ taxtable_scrub_cb (gpointer table_p, gpointer list_p)
  * grandchildren, then fix them to point to the most senior child
  */
 static void
-taxtable_scrub_entries (gpointer entry_p, gpointer ht_p)
+taxtable_scrub_entries (QofEntity * entry_p, gpointer ht_p)
 {
   GHashTable *ht = ht_p;
-  GncEntry *entry = entry_p;
+  GncEntry *entry = GNC_ENTRY(entry_p);
   GncTaxTable *table, *new_tt;
   gint32 count;
 
@@ -546,7 +544,7 @@ taxtable_scrub_entries (gpointer entry_p, gpointer ht_p)
   if (table) {
     if (taxtable_is_grandchild(table)) {
       PINFO("Fixing i-taxtable on entry %s\n",
-	     guid_to_string(gncEntryGetGUID(entry)));
+	     guid_to_string(qof_instance_get_guid(QOF_INSTANCE(entry))));
       new_tt = taxtable_find_senior(table);
       gncEntryBeginEdit(entry);
       gncEntrySetInvTaxTable(entry, new_tt);
@@ -564,7 +562,7 @@ taxtable_scrub_entries (gpointer entry_p, gpointer ht_p)
   if (table) {
     if (taxtable_is_grandchild(table)) {
       PINFO("Fixing b-taxtable on entry %s\n",
-	     guid_to_string(gncEntryGetGUID(entry)));
+	     guid_to_string(qof_instance_get_guid(QOF_INSTANCE(entry))));
       new_tt = taxtable_find_senior(table);
       gncEntryBeginEdit(entry);
       gncEntrySetBillTaxTable(entry, new_tt);
@@ -580,10 +578,10 @@ taxtable_scrub_entries (gpointer entry_p, gpointer ht_p)
 }
 
 static void
-taxtable_scrub_cust (gpointer cust_p, gpointer ht_p)
+taxtable_scrub_cust (QofEntity * cust_p, gpointer ht_p)
 {
   GHashTable *ht = ht_p;
-  GncCustomer *cust = cust_p;
+  GncCustomer *cust = GNC_CUSTOMER(cust_p);
   GncTaxTable *table;
   gint32 count;
   
@@ -596,10 +594,10 @@ taxtable_scrub_cust (gpointer cust_p, gpointer ht_p)
 }
 
 static void
-taxtable_scrub_vendor (gpointer vendor_p, gpointer ht_p)
+taxtable_scrub_vendor (QofEntity * vendor_p, gpointer ht_p)
 {
   GHashTable *ht = ht_p;
-  GncVendor *vendor = vendor_p;
+  GncVendor *vendor = GNC_VENDOR(vendor_p);
   GncTaxTable *table;
   gint32 count;
 
@@ -618,32 +616,32 @@ taxtable_reset_refcount (gpointer key, gpointer value, gpointer notused)
   gint32 count = GPOINTER_TO_INT(value);
 
   if (count != gncTaxTableGetRefcount(table) && !gncTaxTableGetInvisible(table)) {
-    PWARN("Fixing refcount on taxtable %s (%lld -> %d)\n",
-	  guid_to_string(gncTaxTableGetGUID(table)),
-	  gncTaxTableGetRefcount(table), count)
+    PWARN("Fixing refcount on taxtable %s (%" G_GINT64_FORMAT " -> %d)\n",
+	  guid_to_string(qof_instance_get_guid(QOF_INSTANCE(table))),
+	  gncTaxTableGetRefcount(table), count);
       gncTaxTableSetRefcount(table, count);
   }
 }
 
 static void
-taxtable_scrub (GNCBook *book)
+taxtable_scrub (QofBook *book)
 {
   GList *list = NULL;
   GList *node;
   GncTaxTable *parent, *table;
   GHashTable *ht = g_hash_table_new(g_direct_hash, g_direct_equal);
 
-  gncObjectForeach (GNC_ENTRY_MODULE_NAME, book, taxtable_scrub_entries, ht);
-  gncObjectForeach (GNC_CUSTOMER_MODULE_NAME, book, taxtable_scrub_cust, ht);
-  gncObjectForeach (GNC_VENDOR_MODULE_NAME, book, taxtable_scrub_vendor, ht);
-  gncObjectForeach (_GNC_MOD_NAME, book, taxtable_scrub_cb, &list);
+  qof_object_foreach (GNC_ID_ENTRY, book, taxtable_scrub_entries, ht);
+  qof_object_foreach (GNC_ID_CUSTOMER, book, taxtable_scrub_cust, ht);
+  qof_object_foreach (GNC_ID_VENDOR, book, taxtable_scrub_vendor, ht);
+  qof_object_foreach (GNC_ID_TAXTABLE, book, taxtable_scrub_cb, &list);
 
   /* destroy the list of "grandchildren" tax tables */
   for (node = list; node; node = node->next) {
     table = node->data;
 
     PINFO ("deleting grandchild taxtable: %s\n",
-	   guid_to_string(gncTaxTableGetGUID(table)));
+	   guid_to_string(qof_instance_get_guid(QOF_INSTANCE(table))));
 
     /* Make sure the parent has no children */
     parent = gncTaxTableGetParent(table);
@@ -661,6 +659,14 @@ taxtable_scrub (GNCBook *book)
   g_hash_table_destroy(ht);
 }
 
+static void
+taxtable_ns(FILE *out)
+{
+  g_return_if_fail(out);
+  gnc_xml2_write_namespace_decl(out, "taxtable");
+  gnc_xml2_write_namespace_decl(out, "tte");
+}
+
 void
 gnc_taxtable_xml_initialize (void)
 {
@@ -672,9 +678,10 @@ gnc_taxtable_xml_initialize (void)
     taxtable_get_count,
     taxtable_write,
     taxtable_scrub,
+    taxtable_ns,
   };
 
-  gncObjectRegisterBackend (_GNC_MOD_NAME,
+  qof_object_register_backend (_GNC_MOD_NAME,
 			    GNC_FILE_BACKEND,
 			    &be_data);
 }
