@@ -1,75 +1,81 @@
+/********************************************************************\
+ * gncInvoice.c -- the Core Business Invoice                        *
+ *                                                                  *
+ * This program is free software; you can redistribute it and/or    *
+ * modify it under the terms of the GNU General Public License as   *
+ * published by the Free Software Foundation; either version 2 of   *
+ * the License, or (at your option) any later version.              *
+ *                                                                  *
+ * This program is distributed in the hope that it will be useful,  *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
+ * GNU General Public License for more details.                     *
+ *                                                                  *
+ * You should have received a copy of the GNU General Public License*
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
+ *                                                                  *
+\********************************************************************/
+
 /*
- * gncInvoice.c -- the Core Business Invoice
- * Copyright (C) 2001,2002 Derek Atkins
+ * Copyright (C) 2001,2002,2006 Derek Atkins
+ * Copyright (C) 2003 Linas Vepstas <linas@linas.org>
+ * Copyright (c) 2005 Neil Williams <linux@codehelp.co.uk>
  * Author: Derek Atkins <warlord@MIT.EDU>
  */
 
 #include "config.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 
 #include "Transaction.h"
 #include "Account.h"
-#include "messages.h"
-#include "gnc-numeric.h"
-#include "kvp_frame.h"
-#include "gnc-engine-util.h"
-#include "gnc-book-p.h"
-#include "GNCIdP.h"
-#include "QueryObject.h"
-#include "gnc-event-p.h"
-#include "gnc-lot.h"
-#include "gnc-be-utils.h"
-
-#include "gncBusiness.h"
+#include "gncBillTermP.h"
 #include "gncEntry.h"
 #include "gncEntryP.h"
+#include "gncJobP.h"
 #include "gncInvoice.h"
 #include "gncInvoiceP.h"
-#include "gncOwner.h"
+#include "gncOwnerP.h"
 
-struct _gncInvoice {
-  GNCBook *book;
+struct _gncInvoice 
+{
+  QofInstance inst;
   
-  GUID		guid;
-  char *	id;
-  char *	notes;
-  char *	billing_id;
-  char *	printname;
-  GncBillTerm *	terms;
-  GList * 	entries;
-  GncOwner	owner;
-  GncOwner	billto;
-  GncJob *	job;
-  Timespec 	date_opened;
-  Timespec 	date_posted;
+  char        *id;
+  char        *notes;
+  gboolean    active;
+
+  char        *billing_id;
+  char        *printname;
+  GncBillTerm *terms;
+  GList       *entries;
+  GncOwner    owner;
+  GncOwner    billto;
+  GncJob      *job;
+  Timespec    date_opened;
+  Timespec    date_posted;
 
   gnc_numeric	to_charge_amount;
 
   gnc_commodity * currency;
 
-  Account * 	posted_acc;
-  Transaction * posted_txn;
-  GNCLot *	posted_lot;
-
-  gboolean 	active;
-
-  int		editlevel;
-  gboolean	do_free;
-
-  gboolean	dirty;
+  Account     *posted_acc;
+  Transaction *posted_txn;
+  GNCLot      *posted_lot;
 };
 
-static short	module = MOD_BUSINESS;
+static QofLogModule log_module = GNC_MOD_BUSINESS;
 
-#define _GNC_MOD_NAME	GNC_INVOICE_MODULE_NAME
+#define _GNC_MOD_NAME	GNC_ID_INVOICE
 
 #define GNC_INVOICE_ID		"gncInvoice"
 #define GNC_INVOICE_GUID	"invoice-guid"
 
-#define CACHE_INSERT(str) g_cache_insert(gnc_engine_get_string_cache(), (gpointer)(str));
-#define CACHE_REMOVE(str) g_cache_remove(gnc_engine_get_string_cache(), (str));
-
 #define SET_STR(obj, member, str) { \
 	char * tmp; \
 	\
@@ -80,29 +86,25 @@ static short	module = MOD_BUSINESS;
 	member = tmp; \
 	}
 
-static void addObj (GncInvoice *invoice);
-static void remObj (GncInvoice *invoice);
-
 static void mark_invoice (GncInvoice *invoice);
 static void
 mark_invoice (GncInvoice *invoice)
 {
-  invoice->dirty = TRUE;
-  gncBusinessSetDirtyFlag (invoice->book, _GNC_MOD_NAME, TRUE);
-
-  gnc_engine_generate_event (&invoice->guid, GNC_EVENT_MODIFY);
+  qof_instance_set_dirty(&invoice->inst);
+  qof_event_gen (&invoice->inst.entity, QOF_EVENT_MODIFY, NULL);
 }
 
+/* ================================================================== */
 /* Create/Destroy Functions */
 
-GncInvoice *gncInvoiceCreate (GNCBook *book)
+GncInvoice *gncInvoiceCreate (QofBook *book)
 {
   GncInvoice *invoice;
 
   if (!book) return NULL;
 
   invoice = g_new0 (GncInvoice, 1);
-  invoice->book = book;
+  qof_instance_init (&invoice->inst, _GNC_MOD_NAME, book);
 
   invoice->id = CACHE_INSERT ("");
   invoice->notes = CACHE_INSERT ("");
@@ -113,10 +115,7 @@ GncInvoice *gncInvoiceCreate (GNCBook *book)
 
   invoice->to_charge_amount = gnc_numeric_zero();
 
-  xaccGUIDNew (&invoice->guid, book);
-  addObj (invoice);
-
-  gnc_engine_generate_event (&invoice->guid, GNC_EVENT_CREATE);
+  qof_event_gen (&invoice->inst.entity, QOF_EVENT_CREATE, NULL);
 
   return invoice;
 }
@@ -124,7 +123,7 @@ GncInvoice *gncInvoiceCreate (GNCBook *book)
 void gncInvoiceDestroy (GncInvoice *invoice)
 {
   if (!invoice) return;
-  invoice->do_free = TRUE;
+  invoice->inst.do_free = TRUE;
   gncInvoiceCommitEdit (invoice);
 }
 
@@ -132,36 +131,92 @@ static void gncInvoiceFree (GncInvoice *invoice)
 {
   if (!invoice) return;
 
-  gnc_engine_generate_event (&invoice->guid, GNC_EVENT_DESTROY);
+  qof_event_gen (&invoice->inst.entity, QOF_EVENT_DESTROY, NULL);
 
   CACHE_REMOVE (invoice->id);
   CACHE_REMOVE (invoice->notes);
   CACHE_REMOVE (invoice->billing_id);
   g_list_free (invoice->entries);
-  remObj (invoice);
 
   if (invoice->printname) g_free (invoice->printname);
 
   if (invoice->terms)
     gncBillTermDecRef (invoice->terms);
 
+  qof_instance_release (&invoice->inst);
   g_free (invoice);
 }
 
-/* Set Functions */
-
-void gncInvoiceSetGUID (GncInvoice *invoice, const GUID *guid)
+GncInvoice *
+gncCloneInvoice (GncInvoice *from, QofBook *book)
 {
-  if (!invoice || !guid) return;
-  if (guid_equal (guid, &invoice->guid)) return;
+  GList *node;
+  GncInvoice *invoice;
 
-  gncInvoiceBeginEdit (invoice);
-  remObj (invoice);
-  invoice->guid = *guid;
-  addObj (invoice);
-  gncInvoiceCommitEdit (invoice);
+  if (!book) return NULL;
+
+  invoice = g_new0 (GncInvoice, 1);
+  qof_instance_init (&invoice->inst, _GNC_MOD_NAME, book);
+
+  invoice->id = CACHE_INSERT (from->id);
+  invoice->notes = CACHE_INSERT (from->notes);
+  invoice->billing_id = CACHE_INSERT (from->billing_id);
+  invoice->active = from->active;
+
+  invoice->billto = gncCloneOwner (&from->billto, book);
+  invoice->owner = gncCloneOwner (&from->owner, book);
+  invoice->job = (GncJob*)gncJobObtainTwin (from->job, book);
+  invoice->terms = gncBillTermObtainTwin (from->terms, book);
+  gncBillTermIncRef (invoice->terms);
+
+
+  invoice->to_charge_amount = from->to_charge_amount;
+  invoice->printname = NULL; /* that's right, NULL. See below. */
+  invoice->date_opened = from->date_opened;
+  invoice->date_posted = from->date_posted;
+
+  invoice->currency = gnc_commodity_obtain_twin (from->currency, book);
+
+  invoice->entries = NULL;
+  for (node = g_list_last(from->entries); node; node=node->next)
+  {
+    GncEntry *entry = node->data;
+    entry = gncEntryObtainTwin (entry, book);
+    invoice->entries = g_list_prepend (invoice->entries, entry);
+  }
+
+  /* XXX should probably be obtain-twin not lookup-twin */
+  invoice->posted_acc = 
+     GNC_ACCOUNT(qof_instance_lookup_twin(QOF_INSTANCE(from->posted_acc), book));
+#if 0
+XXX not done */
+  Transaction * posted_txn;
+  GNCLot *	posted_lot;
+#endif
+
+  qof_event_gen (&invoice->inst.entity, QOF_EVENT_CREATE, NULL);
+
+  return invoice;
 }
 
+GncInvoice *
+gncInvoiceObtainTwin (GncInvoice *from, QofBook *book)
+{
+  GncInvoice *invoice;
+  if (!book) return NULL;
+
+  invoice = (GncInvoice *) qof_instance_lookup_twin (QOF_INSTANCE(from), book);
+  if (!invoice)
+  {
+    invoice = gncCloneInvoice (from, book);
+  }
+
+  return invoice;
+}
+
+/* ================================================================== */
+/* Set Functions */
+
 void gncInvoiceSetID (GncInvoice *invoice, const char *id)
 {
   if (!invoice || !id) return;
@@ -180,6 +235,26 @@ void gncInvoiceSetOwner (GncInvoice *invoice, GncOwner *owner)
   gncInvoiceCommitEdit (invoice);
 }
 
+static void
+qofInvoiceSetOwner (GncInvoice *invoice, QofEntity *ent)
+{
+	if(!invoice || !ent) { return; }
+	gncInvoiceBeginEdit (invoice);
+	qofOwnerSetEntity(&invoice->owner, ent);
+	mark_invoice (invoice);
+	gncInvoiceCommitEdit (invoice);
+}
+
+static void
+qofInvoiceSetBillTo (GncInvoice *invoice, QofEntity *ent)
+{
+	if(!invoice || !ent) { return; }
+	gncInvoiceBeginEdit (invoice);
+	qofOwnerSetEntity(&invoice->billto, ent);
+	mark_invoice (invoice);
+	gncInvoiceCommitEdit (invoice);
+}
+
 void gncInvoiceSetDateOpened (GncInvoice *invoice, Timespec date)
 {
   if (!invoice) return;
@@ -273,12 +348,6 @@ void gncInvoiceSetToChargeAmount (GncInvoice *invoice, gnc_numeric amount)
   gncInvoiceCommitEdit (invoice);
 }
 
-void gncInvoiceSetDirty (GncInvoice *invoice, gboolean dirty)
-{
-  if (!invoice) return;
-  invoice->dirty = dirty;
-}
-
 void gncInvoiceSetPostedTxn (GncInvoice *invoice, Transaction *txn)
 {
   if (!invoice) return;
@@ -362,20 +431,17 @@ void gncBillRemoveEntry (GncInvoice *bill, GncEntry *entry)
   mark_invoice (bill);
 }
 
+void gncInvoiceSortEntries (GncInvoice *invoice)
+{
+  if (!invoice) return;
+  invoice->entries = g_list_sort(invoice->entries,
+				 (GCompareFunc)gncEntryCompare);
+  mark_invoice(invoice);
+}
+
+/* ================================================================== */
 /* Get Functions */
 
-GNCBook * gncInvoiceGetBook (GncInvoice *invoice)
-{
-  if (!invoice) return NULL;
-  return invoice->book;
-}
-
-const GUID * gncInvoiceGetGUID (GncInvoice *invoice)
-{
-  if (!invoice) return NULL;
-  return &(invoice->guid);
-}
-
 const char * gncInvoiceGetID (GncInvoice *invoice)
 {
   if (!invoice) return NULL;
@@ -388,6 +454,20 @@ GncOwner * gncInvoiceGetOwner (GncInvoice *invoice)
   return &invoice->owner;
 }
 
+static QofEntity*
+qofInvoiceGetOwner (GncInvoice *invoice)
+{
+	if(!invoice) { return NULL; }
+	return (QofEntity*)&invoice->owner;
+}
+
+static QofEntity*
+qofInvoiceGetBillTo (GncInvoice *invoice)
+{
+	if(!invoice) { return NULL; }
+	return (QofEntity*)&invoice->billto;
+}
+
 Timespec gncInvoiceGetDateOpened (GncInvoice *invoice)
 {
   Timespec ts; ts.tv_sec = 0; ts.tv_nsec = 0;
@@ -565,16 +645,69 @@ GList * gncInvoiceGetEntries (GncInvoice *invoice)
   return invoice->entries;
 }
 
-gboolean gncInvoiceIsDirty (GncInvoice *invoice)
+static QofCollection*
+qofInvoiceGetEntries (GncInvoice *invoice)
 {
-  if (!invoice) return FALSE;
-  return invoice->dirty;
+	QofCollection *entry_coll;
+	GList         *list;
+	QofEntity     *entry;
+
+	entry_coll = qof_collection_new(GNC_ID_ENTRY);
+	for(list = gncInvoiceGetEntries(invoice); list != NULL; list = list->next)
+	{
+		entry = (QofEntity*)list->data;
+		qof_collection_add_entity(entry_coll, entry);
+	}
+	return entry_coll;
+}
+
+static void
+qofInvoiceEntryCB (QofEntity *ent, gpointer user_data)
+{
+	GncInvoice *invoice;
+
+	invoice = (GncInvoice*)user_data;
+	if(!invoice || !ent) { return; }
+	switch (gncInvoiceGetOwnerType (invoice)) {
+		case GNC_OWNER_VENDOR: {
+		gncBillAddEntry (invoice, (GncEntry*) ent);
+		break;
+		}
+		default : {
+			gncInvoiceAddEntry(invoice, (GncEntry*)ent);
+			break;
+		}
+	}
+}
+
+static void 
+qofInvoiceSetEntries(GncInvoice *invoice, QofCollection *entry_coll)
+{
+	if(!entry_coll) { return; }
+	if(0 == safe_strcmp(qof_collection_get_type(entry_coll), GNC_ID_ENTRY))
+	{
+		qof_collection_foreach(entry_coll, qofInvoiceEntryCB, invoice);
+	}
+}
+
+static GncJob*
+qofInvoiceGetJob (GncInvoice *invoice)
+{
+	if(!invoice) { return NULL; }
+	return invoice->job;
+}
+
+static void 
+qofInvoiceSetJob (GncInvoice *invoice, GncJob *job)
+{
+	if(!invoice) { return; }
+	invoice->job = job;
 }
 
 static void
 gncInvoiceDetachFromLot (GNCLot *lot)
 {
-  kvp_frame *kvp;
+  KvpFrame *kvp;
 
   if (!lot) return;
 
@@ -585,8 +718,8 @@ gncInvoiceDetachFromLot (GNCLot *lot)
 static void
 gncInvoiceAttachToLot (GncInvoice *invoice, GNCLot *lot)
 {
-  kvp_frame *kvp;
-  kvp_value *value;
+  KvpFrame *kvp;
+  KvpValue *value;
   
   if (!invoice || !lot)
     return;
@@ -594,7 +727,7 @@ gncInvoiceAttachToLot (GncInvoice *invoice, GNCLot *lot)
   if (invoice->posted_lot) return;	/* Cannot reset invoice's lot */
 
   kvp = gnc_lot_get_slots (lot);
-  value = kvp_value_new_guid (gncInvoiceGetGUID (invoice));
+  value = kvp_value_new_guid (qof_instance_get_guid (QOF_INSTANCE(invoice)));
   kvp_frame_set_slot_path (kvp, value, GNC_INVOICE_ID, GNC_INVOICE_GUID, NULL);
   kvp_value_delete (value);
   gncInvoiceSetPostedLot (invoice, lot);
@@ -602,10 +735,10 @@ gncInvoiceAttachToLot (GncInvoice *invoice, GNCLot *lot)
 
 GncInvoice * gncInvoiceGetInvoiceFromLot (GNCLot *lot)
 {
-  kvp_frame *kvp;
-  kvp_value *value;
+  KvpFrame *kvp;
+  KvpValue *value;
   GUID *guid;
-  GNCBook *book;
+  QofBook *book;
 
   if (!lot) return NULL;
 
@@ -615,16 +748,14 @@ GncInvoice * gncInvoiceGetInvoiceFromLot (GNCLot *lot)
   if (!value) return NULL;
 
   guid = kvp_value_get_guid (value);
-
-  return xaccLookupEntity (gnc_book_get_entity_table (book),
-			   guid, _GNC_MOD_NAME);
+  return gncInvoiceLookup(book, guid);
 }
 
 static void
 gncInvoiceAttachToTxn (GncInvoice *invoice, Transaction *txn)
 {
-  kvp_frame *kvp;
-  kvp_value *value;
+  KvpFrame *kvp;
+  KvpValue *value;
   
   if (!invoice || !txn)
     return;
@@ -633,7 +764,7 @@ gncInvoiceAttachToTxn (GncInvoice *invoice, Transaction *txn)
 
   xaccTransBeginEdit (txn);
   kvp = xaccTransGetSlots (txn);
-  value = kvp_value_new_guid (gncInvoiceGetGUID (invoice));
+  value = kvp_value_new_guid (qof_instance_get_guid(QOF_INSTANCE(invoice)));
   kvp_frame_set_slot_path (kvp, value, GNC_INVOICE_ID, GNC_INVOICE_GUID, NULL);
   kvp_value_delete (value);
   xaccTransSetTxnType (txn, TXN_TYPE_INVOICE);
@@ -641,12 +772,13 @@ gncInvoiceAttachToTxn (GncInvoice *invoice, Transaction *txn)
   gncInvoiceSetPostedTxn (invoice, txn);
 }
 
-GncInvoice * gncInvoiceGetInvoiceFromTxn (Transaction *txn)
+GncInvoice * 
+gncInvoiceGetInvoiceFromTxn (Transaction *txn)
 {
-  kvp_frame *kvp;
-  kvp_value *value;
+  KvpFrame *kvp;
+  KvpValue *value;
   GUID *guid;
-  GNCBook *book;
+  QofBook *book;
 
   if (!txn) return NULL;
 
@@ -656,9 +788,7 @@ GncInvoice * gncInvoiceGetInvoiceFromTxn (Transaction *txn)
   if (!value) return NULL;
 
   guid = kvp_value_get_guid (value);
-
-  return xaccLookupEntity (gnc_book_get_entity_table (book),
-			   guid, _GNC_MOD_NAME);
+  return gncInvoiceLookup(book,guid);
 }
 
 struct lotmatch {
@@ -701,6 +831,7 @@ Transaction * gncInvoicePostToAccount (GncInvoice *invoice, Account *acc,
   gnc_numeric total;
   gboolean reverse;
   const char *name, *type;
+  char *lot_title;
   Account *ccard_acct = NULL;
   GncOwner *owner;
 
@@ -739,14 +870,20 @@ Transaction * gncInvoicePostToAccount (GncInvoice *invoice, Account *acc,
 
   /* Create a new lot for this invoice, if we need to do so */
   if (!lot)
-    lot = gnc_lot_new (invoice->book);
+    lot = gnc_lot_new (invoice->inst.book);
+
+  type = gncInvoiceGetType (invoice);
+
+  /* Set the lot title */
+  lot_title = g_strdup_printf ("%s %s", type, gncInvoiceGetID (invoice));
+  gnc_lot_set_title (lot, lot_title);
+  g_free (lot_title);
 
   /* Create a new transaction */
-  txn = xaccMallocTransaction (invoice->book);
+  txn = xaccMallocTransaction (invoice->inst.book);
   xaccTransBeginEdit (txn);
 
   name = gncOwnerGetName (gncOwnerGetEndOwner (gncInvoiceGetOwner (invoice)));
-  type = gncInvoiceGetType (invoice);
 
   /* Set Transaction Description (Owner Name) , Num (invoice ID), Currency */
   xaccTransSetDescription (txn, name ? name : "");
@@ -797,23 +934,27 @@ Transaction * gncInvoicePostToAccount (GncInvoice *invoice, Account *acc,
     if (this_acc) {
       if (gnc_numeric_check (value) == GNC_ERROR_OK) {
 	if (accumulatesplits) {
-	  splitinfo = gncAccountValueAdd (splitinfo, this_acc, value);
+	    splitinfo = gncAccountValueAdd (splitinfo, this_acc, value);
 	} else {
 	  Split *split;
 
-	  split = xaccMallocSplit (invoice->book);
+	  split = xaccMallocSplit (invoice->inst.book);
 	  /* set action and memo? */
 
 	  xaccSplitSetMemo (split, gncEntryGetDescription (entry));
 	  xaccSplitSetAction (split, type);
 
-	  xaccSplitSetBaseValue (split, (reverse ? gnc_numeric_neg (value)
-					 : value),
-				 invoice->currency);
+	  /* Need to insert this split into the account AND txn before
+	   * we set the Base Value.  Otherwise SetBaseValue complains
+	   * that we don't have an account and fails to set the value.
+	   */
 	  xaccAccountBeginEdit (this_acc);
 	  xaccAccountInsertSplit (this_acc, split);
 	  xaccAccountCommitEdit (this_acc);
 	  xaccTransAppendSplit (txn, split);
+	  xaccSplitSetBaseValue (split, (reverse ? gnc_numeric_neg (value)
+					 : value),
+				 invoice->currency);
 	}
 
 	/* If there is a credit-card account, and this is a CCard
@@ -828,16 +969,16 @@ Transaction * gncInvoicePostToAccount (GncInvoice *invoice, Account *acc,
 	if (ccard_acct && gncEntryGetBillPayment (entry) == GNC_PAYMENT_CARD) {
 	  Split *split;
 
-	  split = xaccMallocSplit (invoice->book);
+	  split = xaccMallocSplit (invoice->inst.book);
 	  /* set action? */
 	  xaccSplitSetMemo (split, gncEntryGetDescription (entry));
 	  xaccSplitSetAction (split, type);
-	  xaccSplitSetBaseValue (split, (reverse ? value : gnc_numeric_neg (value)),
-				 invoice->currency);
 	  xaccAccountBeginEdit (ccard_acct);
 	  xaccAccountInsertSplit (ccard_acct, split);
 	  xaccAccountCommitEdit (ccard_acct);
 	  xaccTransAppendSplit (txn, split);
+	  xaccSplitSetBaseValue (split, (reverse ? value : gnc_numeric_neg (value)),
+				 invoice->currency);
 	  
 	} else
 	  total = gnc_numeric_add (total, value, GNC_DENOM_AUTO, GNC_DENOM_LCD);
@@ -862,38 +1003,38 @@ Transaction * gncInvoicePostToAccount (GncInvoice *invoice, Account *acc,
     Split *split;
     GncAccountValue *acc_val = iter->data;
 
-    split = xaccMallocSplit (invoice->book);
+    split = xaccMallocSplit (invoice->inst.book);
     /* set action and memo? */
 
     xaccSplitSetMemo (split, memo);
     xaccSplitSetAction (split, type);
 
-    xaccSplitSetBaseValue (split, (reverse ? gnc_numeric_neg (acc_val->value)
-				   : acc_val->value),
-			   invoice->currency);
     xaccAccountBeginEdit (acc_val->account);
     xaccAccountInsertSplit (acc_val->account, split);
     xaccAccountCommitEdit (acc_val->account);
     xaccTransAppendSplit (txn, split);
+    xaccSplitSetBaseValue (split, (reverse ? gnc_numeric_neg (acc_val->value)
+				   : acc_val->value),
+			   invoice->currency);
   }
 
   /* If there is a ccard account, we may have an additional "to_card" payment.
    * we should make that now..
    */
   if (ccard_acct && !gnc_numeric_zero_p (invoice->to_charge_amount)) {
-    Split *split = xaccMallocSplit (invoice->book);
+    Split *split = xaccMallocSplit (invoice->inst.book);
 
     /* Set memo.  action? */
     xaccSplitSetMemo (split, _("Extra to Charge Card"));
     xaccSplitSetAction (split, type);
     
-    xaccSplitSetBaseValue (split, (reverse ? invoice->to_charge_amount :
-				   gnc_numeric_neg(invoice->to_charge_amount)),
-			   invoice->currency);
     xaccAccountBeginEdit (ccard_acct);
     xaccAccountInsertSplit (ccard_acct, split);
     xaccAccountCommitEdit (ccard_acct);
     xaccTransAppendSplit (txn, split);
+    xaccSplitSetBaseValue (split, (reverse ? invoice->to_charge_amount :
+				   gnc_numeric_neg(invoice->to_charge_amount)),
+			   invoice->currency);
 
     total = gnc_numeric_sub (total, invoice->to_charge_amount,
 			     GNC_DENOM_AUTO, GNC_DENOM_LCD);
@@ -901,18 +1042,18 @@ Transaction * gncInvoicePostToAccount (GncInvoice *invoice, Account *acc,
 
   /* Now create the Posted split (which is negative -- it's a credit) */
   {
-    Split *split = xaccMallocSplit (invoice->book);
+    Split *split = xaccMallocSplit (invoice->inst.book);
 
     /* Set action/memo */
     xaccSplitSetMemo (split, memo);
     xaccSplitSetAction (split, type);
 			   
-    xaccSplitSetBaseValue (split, (reverse ? total : gnc_numeric_neg (total)),
-			   invoice->currency);
     xaccAccountBeginEdit (acc);
     xaccAccountInsertSplit (acc, split);
     xaccAccountCommitEdit (acc);
     xaccTransAppendSplit (txn, split);
+    xaccSplitSetBaseValue (split, (reverse ? total : gnc_numeric_neg (total)),
+			   invoice->currency);
 
     /* add this split to the lot */
     gnc_lot_add_split (lot, split);
@@ -944,8 +1085,8 @@ Transaction * gncInvoicePostToAccount (GncInvoice *invoice, Account *acc,
     char *memo2 = _("Automatic Payment Forward");
     char *action2 = _("Auto Split");
 
-    t2 = xaccMallocTransaction (invoice->book);
-    lot2 = gnc_lot_new (invoice->book);
+    t2 = xaccMallocTransaction (invoice->inst.book);
+    lot2 = gnc_lot_new (invoice->inst.book);
     gncOwnerAttachToLot (gncOwnerGetEndOwner (gncInvoiceGetOwner (invoice)),
 			 lot2);
     
@@ -962,22 +1103,22 @@ Transaction * gncInvoicePostToAccount (GncInvoice *invoice, Account *acc,
       xaccTransSetDatePostedTS (t2, post_date);
 
     /* Balance out this lot */
-    split = xaccMallocSplit (invoice->book);
+    split = xaccMallocSplit (invoice->inst.book);
     xaccSplitSetMemo (split, memo2);
     xaccSplitSetAction (split, action2);
-    xaccSplitSetBaseValue (split, gnc_numeric_neg (total),
-			   invoice->currency);
     xaccAccountInsertSplit (acc, split);
     xaccTransAppendSplit (t2, split);
+    xaccSplitSetBaseValue (split, gnc_numeric_neg (total),
+			   invoice->currency);
     gnc_lot_add_split (lot, split);
 
     /* And apply the pre-payment to a new lot */
-    split = xaccMallocSplit (invoice->book);
+    split = xaccMallocSplit (invoice->inst.book);
     xaccSplitSetMemo (split, memo2);
     xaccSplitSetAction (split, action2);
-    xaccSplitSetBaseValue (split, total, invoice->currency);
     xaccAccountInsertSplit (acc, split);
     xaccTransAppendSplit (t2, split);
+    xaccSplitSetBaseValue (split, total, invoice->currency);
     gnc_lot_add_split (lot2, split);
 
     xaccTransCommitEdit (t2);
@@ -990,7 +1131,7 @@ Transaction * gncInvoicePostToAccount (GncInvoice *invoice, Account *acc,
 }
 
 gboolean
-gncInvoiceUnpost (GncInvoice *invoice)
+gncInvoiceUnpost (GncInvoice *invoice, gboolean reset_tax_tables)
 {
   Transaction *txn;
   GNCLot *lot;
@@ -1026,6 +1167,25 @@ gncInvoiceUnpost (GncInvoice *invoice)
   invoice->posted_lot = NULL;
   invoice->date_posted.tv_sec = invoice->date_posted.tv_nsec = 0;
 
+  /* if we've been asked to reset the tax tables, then do so */
+  if (reset_tax_tables) {
+    gboolean reverse = (gncInvoiceGetOwnerType(invoice) == GNC_OWNER_CUSTOMER);
+    GList *iter;
+
+    for (iter = gncInvoiceGetEntries(invoice); iter; iter = iter->next) {
+      GncEntry *entry = iter->data;
+
+      gncEntryBeginEdit(entry);
+      if (reverse)
+	gncEntrySetInvTaxTable(entry,
+			       gncTaxTableGetParent(gncEntryGetInvTaxTable(entry)));
+      else
+	gncEntrySetBillTaxTable(entry,
+			       gncTaxTableGetParent(gncEntryGetBillTaxTable(entry)));
+      gncEntryCommitEdit(entry);
+    }
+  }
+
   mark_invoice (invoice);
   gncInvoiceCommitEdit (invoice);
 
@@ -1075,20 +1235,22 @@ gnc_lot_sort_func (GNCLot *a, GNCLot *b)
  * in gncInvoice...
  */
 Transaction *
-gncOwnerApplyPayment (GncOwner *owner, Account *posted_acc, Account *xfer_acc,
+gncOwnerApplyPayment (GncOwner *owner, GncInvoice* invoice,
+		      Account *posted_acc, Account *xfer_acc,
 		      gnc_numeric amount, Timespec date,
 		      const char *memo, const char *num)
 {
-  GNCBook *book;
+  QofBook *book;
+  Account *inv_posted_acc;
   Transaction *txn;
   Split *split;
   GList *lot_list, *fifo = NULL;
-  GNCLot *lot, *prepay_lot = NULL;
-  GncInvoice *invoice;
+  GNCLot *lot, *inv_posted_lot = NULL, *prepay_lot = NULL;
+  GncInvoice *this_invoice;
   const char *name;
   gnc_commodity *commodity;
   gnc_numeric split_amt;
-  gboolean reverse;
+  gboolean reverse, inv_passed = TRUE;
 
   /* Verify our arguments */
   if (!owner || !posted_acc || !xfer_acc) return NULL;
@@ -1115,12 +1277,12 @@ gncOwnerApplyPayment (GncOwner *owner, Account *posted_acc, Account *xfer_acc,
   split = xaccMallocSplit (book);
   xaccSplitSetMemo (split, memo);
   xaccSplitSetAction (split, _("Payment"));
-  xaccSplitSetBaseValue (split, reverse ? amount :
-			 gnc_numeric_neg (amount), commodity);
   xaccAccountBeginEdit (xfer_acc);
   xaccAccountInsertSplit (xfer_acc, split);
   xaccAccountCommitEdit (xfer_acc);
   xaccTransAppendSplit (txn, split);
+  xaccSplitSetBaseValue (split, reverse ? amount :
+			 gnc_numeric_neg (amount), commodity);
 
   /* Now, find all "open" lots in the posting account for this
    * company and apply the payment on a FIFO basis.  Create
@@ -1131,6 +1293,23 @@ gncOwnerApplyPayment (GncOwner *owner, Account *posted_acc, Account *xfer_acc,
 				  owner,
 				  (GCompareFunc)gnc_lot_sort_func);
 
+  /* Check if an invoice was passed in, and if so, does it match the
+   * account, and is it an open lot?  If so, put it at the beginning
+   * of the lot list fifo so we post to this invoice's lot first.
+   */
+  if (invoice) {
+    inv_posted_acc = gncInvoiceGetPostedAcc(invoice);
+    inv_posted_lot = gncInvoiceGetPostedLot(invoice);
+    if (inv_posted_acc && inv_posted_lot &&
+	guid_equal(xaccAccountGetGUID(inv_posted_acc),
+		   xaccAccountGetGUID(posted_acc)) &&
+	!gnc_lot_is_closed(inv_posted_lot)) {
+      /* Put this invoice at the beginning of the FIFO */
+      fifo = g_list_prepend (fifo, inv_posted_lot);
+      inv_passed = FALSE;
+    }
+  }
+
   xaccAccountBeginEdit (posted_acc);
 
   /* Now iterate over the fifo until the payment is fully applied
@@ -1140,6 +1319,21 @@ gncOwnerApplyPayment (GncOwner *owner, Account *posted_acc, Account *xfer_acc,
     gnc_numeric balance;
 
     lot = lot_list->data;
+
+    /* Skip this lot if it matches the invoice that was passed in and
+     * we've seen it already.  This way we post to it the first time
+     * (from the beginning of the lot-list) but not when we reach it
+     * the second time.
+     */
+    if (inv_posted_lot &&
+	guid_equal(qof_instance_get_guid(QOF_INSTANCE(lot)),
+		   qof_instance_get_guid(QOF_INSTANCE(inv_posted_lot)))) {
+      if (inv_passed)
+	continue;
+      else
+	inv_passed = TRUE;
+    }
+
     balance = gnc_lot_get_balance (lot);
 
     if (!reverse)
@@ -1177,16 +1371,16 @@ gncOwnerApplyPayment (GncOwner *owner, Account *posted_acc, Account *xfer_acc,
     split = xaccMallocSplit (book);
     xaccSplitSetMemo (split, memo);
     xaccSplitSetAction (split, _("Payment"));
-    xaccSplitSetBaseValue (split, reverse ? gnc_numeric_neg (split_amt) :
-			   split_amt, commodity);
     xaccAccountInsertSplit (posted_acc, split);
     xaccTransAppendSplit (txn, split);
+    xaccSplitSetBaseValue (split, reverse ? gnc_numeric_neg (split_amt) :
+			   split_amt, commodity);
     gnc_lot_add_split (lot, split);
 
     /* Now send an event for the invoice so it gets updated as paid */
-    invoice = gncInvoiceGetInvoiceFromLot(lot);
-    if (invoice)
-      gnc_engine_generate_event (&invoice->guid, GNC_EVENT_MODIFY);
+    this_invoice = gncInvoiceGetInvoiceFromLot(lot);
+    if (this_invoice)
+      qof_event_gen (&this_invoice->inst.entity, QOF_EVENT_MODIFY, NULL);
 
     if (gnc_numeric_zero_p (amount))
       break;
@@ -1204,10 +1398,10 @@ gncOwnerApplyPayment (GncOwner *owner, Account *posted_acc, Account *xfer_acc,
     split = xaccMallocSplit (book);
     xaccSplitSetMemo (split, memo);
     xaccSplitSetAction (split, _("Pre-Payment"));
-    xaccSplitSetBaseValue (split, reverse ? gnc_numeric_neg (amount) :
-			   amount, commodity);
     xaccAccountInsertSplit (posted_acc, split);
     xaccTransAppendSplit (txn, split);
+    xaccSplitSetBaseValue (split, reverse ? gnc_numeric_neg (amount) :
+			   amount, commodity);
     gnc_lot_add_split (prepay_lot, split);
   }
 
@@ -1239,47 +1433,31 @@ gboolean gncInvoiceIsPaid (GncInvoice *invoice)
   return gnc_lot_is_closed(invoice->posted_lot);
 }
 
-GUID gncInvoiceRetGUID (GncInvoice *invoice)
-{
-  if (!invoice)
-    return *xaccGUIDNULL();
-
-  return invoice->guid;
-}
-
-GncInvoice * gncInvoiceLookupDirect (GUID guid, GNCBook *book)
-{
-  if (!book) return NULL;
-  return gncInvoiceLookup (book, &guid);
-}
-
-GncInvoice * gncInvoiceLookup (GNCBook *book, const GUID *guid)
-{
-  if (!book || !guid) return NULL;
-  return xaccLookupEntity (gnc_book_get_entity_table (book),
-			   guid, _GNC_MOD_NAME);
-}
+/* ================================================================== */
 
 void gncInvoiceBeginEdit (GncInvoice *invoice)
 {
-  GNC_BEGIN_EDIT (invoice, _GNC_MOD_NAME);
+  QOF_BEGIN_EDIT (&invoice->inst);
 }
 
-static void gncInvoiceOnError (GncInvoice *invoice, GNCBackendError errcode)
+static inline void gncInvoiceOnError (QofInstance *inst, QofBackendError errcode)
 {
-  PERR("Invoice Backend Failure: %d", errcode);
+  PERR("Invoice QofBackend Failure: %d", errcode);
 }
 
-static void gncInvoiceOnDone (GncInvoice *invoice)
+static inline void gncInvoiceOnDone (QofInstance *invoice) { }
+
+static inline void invoice_free (QofInstance *inst)
 {
-  invoice->dirty = FALSE;
+  GncInvoice *invoice = (GncInvoice *) inst;
+  gncInvoiceFree (invoice);
 }
 
 void gncInvoiceCommitEdit (GncInvoice *invoice)
 {
-  GNC_COMMIT_EDIT_PART1 (invoice);
-  GNC_COMMIT_EDIT_PART2 (invoice, _GNC_MOD_NAME, gncInvoiceOnError,
-			 gncInvoiceOnDone, gncInvoiceFree);
+  if (!qof_commit_edit (QOF_INSTANCE(invoice))) return;
+  qof_commit_edit_part2 (&invoice->inst, gncInvoiceOnError,
+			 gncInvoiceOnDone, invoice_free);
 }
 
 int gncInvoiceCompare (GncInvoice *a, GncInvoice *b)
@@ -1299,54 +1477,19 @@ int gncInvoiceCompare (GncInvoice *a, GncInvoice *b)
   compare = timespec_cmp (&(a->date_posted), &(b->date_posted));
   if (compare) return compare;
 
-  return guid_compare (&(a->guid), &(b->guid));
+  return guid_compare (&(a->inst.entity.guid), &(b->inst.entity.guid));
 }
 
+/* ============================================================= */
 /* Package-Private functions */
 
-static void addObj (GncInvoice *invoice)
-{
-  gncBusinessAddObject (invoice->book, _GNC_MOD_NAME, invoice, &invoice->guid);
-}
-
-static void remObj (GncInvoice *invoice)
-{
-  gncBusinessRemoveObject (invoice->book, _GNC_MOD_NAME, &invoice->guid);
-}
-
-static void _gncInvoiceCreate (GNCBook *book)
-{
-  gncBusinessCreate (book, _GNC_MOD_NAME);
-}
-
-static void _gncInvoiceDestroy (GNCBook *book)
-{
-  gncBusinessDestroy (book, _GNC_MOD_NAME);
-}
-
-static gboolean _gncInvoiceIsDirty (GNCBook *book)
-{
-  return gncBusinessIsDirty (book, _GNC_MOD_NAME);
-}
-
-static void _gncInvoiceMarkClean (GNCBook *book)
-{
-  gncBusinessSetDirtyFlag (book, _GNC_MOD_NAME, FALSE);
-}
-
-static void _gncInvoiceForeach (GNCBook *book, foreachObjectCB cb,
-				gpointer user_data)
-{
-  gncBusinessForeach (book, _GNC_MOD_NAME, cb, user_data);
-}
-
 static const char * _gncInvoicePrintable (gpointer obj)
 {
   GncInvoice *invoice = obj;
 
   g_return_val_if_fail (invoice, NULL);
 
-  if (invoice->dirty || invoice->printname == NULL) {
+  if (invoice->inst.dirty || invoice->printname == NULL) {
     if (invoice->printname) g_free (invoice->printname);
 
     invoice->printname =
@@ -1357,74 +1500,89 @@ static const char * _gncInvoicePrintable (gpointer obj)
   return invoice->printname;
 }
 
-static GncObject_t gncInvoiceDesc = {
-  GNC_OBJECT_VERSION,
-  _GNC_MOD_NAME,
-  "Invoice",
-  _gncInvoiceCreate,
-  _gncInvoiceDestroy,
-  _gncInvoiceIsDirty,
-  _gncInvoiceMarkClean,
-  _gncInvoiceForeach,
-  _gncInvoicePrintable,
+static QofObject gncInvoiceDesc = 
+{
+  interface_version:  QOF_OBJECT_VERSION,
+  e_type:             _GNC_MOD_NAME,
+  type_label:         "Invoice",
+  create:             (gpointer)gncInvoiceCreate,
+  book_begin:         NULL,
+  book_end:           NULL,
+  is_dirty:           qof_collection_is_dirty,
+  mark_clean:         qof_collection_mark_clean,
+  foreach:            qof_collection_foreach,
+  printable:          _gncInvoicePrintable,
+  version_cmp:        (int (*)(gpointer, gpointer)) qof_instance_version_cmp,
 };
 
 static void
 reg_lot (void)
 {
-  static QueryObjectDef params[] = {
+  static QofParam params[] = {
     { INVOICE_FROM_LOT, _GNC_MOD_NAME,
-      (QueryAccess)gncInvoiceGetInvoiceFromLot },
+      (QofAccessFunc)gncInvoiceGetInvoiceFromLot, NULL },
     { NULL },
   };
 
-  gncQueryObjectRegister (GNC_ID_LOT, NULL, params);
+  qof_class_register (GNC_ID_LOT, NULL, params);
 }
 
 static void
 reg_txn (void)
 {
-  static QueryObjectDef params[] = {
+  static QofParam params[] = {
     { INVOICE_FROM_TXN, _GNC_MOD_NAME,
-      (QueryAccess)gncInvoiceGetInvoiceFromTxn },
+      (QofAccessFunc)gncInvoiceGetInvoiceFromTxn, NULL },
     { NULL },
   };
 
-  gncQueryObjectRegister (GNC_ID_TRANS, NULL, params);
+  qof_class_register (GNC_ID_TRANS, NULL, params);
 }
 
 gboolean gncInvoiceRegister (void)
 {
-  static QueryObjectDef params[] = {
-    { INVOICE_ID, QUERYCORE_STRING, (QueryAccess)gncInvoiceGetID },
-    { INVOICE_OWNER, GNC_OWNER_MODULE_NAME, (QueryAccess)gncInvoiceGetOwner },
-    { INVOICE_OPENED, QUERYCORE_DATE, (QueryAccess)gncInvoiceGetDateOpened },
-    { INVOICE_DUE, QUERYCORE_DATE, (QueryAccess)gncInvoiceGetDateDue },
-    { INVOICE_POSTED, QUERYCORE_DATE, (QueryAccess)gncInvoiceGetDatePosted },
-    { INVOICE_IS_POSTED, QUERYCORE_BOOLEAN, (QueryAccess)gncInvoiceIsPosted },
-    { INVOICE_IS_PAID, QUERYCORE_BOOLEAN, (QueryAccess)gncInvoiceIsPaid },
-    { INVOICE_BILLINGID, QUERYCORE_STRING, (QueryAccess)gncInvoiceGetBillingID },
-    { INVOICE_NOTES, QUERYCORE_STRING, (QueryAccess)gncInvoiceGetNotes },
-    { INVOICE_ACC, GNC_ID_ACCOUNT, (QueryAccess)gncInvoiceGetPostedAcc },
-    { INVOICE_POST_TXN, GNC_ID_TRANS, (QueryAccess)gncInvoiceGetPostedTxn },
-    { INVOICE_POST_LOT, GNC_ID_LOT, (QueryAccess)gncInvoiceGetPostedLot },
-    { INVOICE_TYPE, QUERYCORE_STRING, (QueryAccess)gncInvoiceGetType },
-    { INVOICE_TERMS, GNC_BILLTERM_MODULE_NAME, (QueryAccess)gncInvoiceGetTerms },
-    { INVOICE_BILLTO, GNC_OWNER_MODULE_NAME, (QueryAccess)gncInvoiceGetBillTo },
-    { QUERY_PARAM_BOOK, GNC_ID_BOOK, (QueryAccess)gncInvoiceGetBook },
-    { QUERY_PARAM_GUID, QUERYCORE_GUID, (QueryAccess)gncInvoiceGetGUID },
-    { QUERY_PARAM_ACTIVE, QUERYCORE_BOOLEAN, (QueryAccess)gncInvoiceGetActive },
+  static QofParam params[] = {
+    { INVOICE_ID,      QOF_TYPE_STRING, (QofAccessFunc)gncInvoiceGetID,     (QofSetterFunc)gncInvoiceSetID },
+    { INVOICE_OWNER,   GNC_ID_OWNER, (QofAccessFunc)gncInvoiceGetOwner, NULL },
+    { INVOICE_OPENED,  QOF_TYPE_DATE,   (QofAccessFunc)gncInvoiceGetDateOpened, (QofSetterFunc)gncInvoiceSetDateOpened },
+    { INVOICE_DUE,     QOF_TYPE_DATE,   (QofAccessFunc)gncInvoiceGetDateDue, NULL },
+    { INVOICE_POSTED,  QOF_TYPE_DATE,   (QofAccessFunc)gncInvoiceGetDatePosted, (QofSetterFunc)gncInvoiceSetDatePosted },
+    { INVOICE_IS_POSTED, QOF_TYPE_BOOLEAN, (QofAccessFunc)gncInvoiceIsPosted, NULL },
+    { INVOICE_IS_PAID, QOF_TYPE_BOOLEAN, (QofAccessFunc)gncInvoiceIsPaid,    NULL },
+    { INVOICE_BILLINGID, QOF_TYPE_STRING, (QofAccessFunc)gncInvoiceGetBillingID, (QofSetterFunc)gncInvoiceSetBillingID },
+    { INVOICE_NOTES,   QOF_TYPE_STRING, (QofAccessFunc)gncInvoiceGetNotes,   (QofSetterFunc)gncInvoiceSetNotes },
+    { INVOICE_ACC,     GNC_ID_ACCOUNT,  (QofAccessFunc)gncInvoiceGetPostedAcc, (QofSetterFunc)gncInvoiceSetPostedAcc },
+    { INVOICE_POST_TXN, GNC_ID_TRANS,   (QofAccessFunc)gncInvoiceGetPostedTxn, (QofSetterFunc)gncInvoiceSetPostedTxn },
+    { INVOICE_POST_LOT, GNC_ID_LOT,     (QofAccessFunc)gncInvoiceGetPostedLot, NULL/*(QofSetterFunc)gncInvoiceSetPostedLot*/ },
+    { INVOICE_TYPE,    QOF_TYPE_STRING, (QofAccessFunc)gncInvoiceGetType,    NULL },
+    { INVOICE_TERMS,   GNC_ID_BILLTERM, (QofAccessFunc)gncInvoiceGetTerms,   (QofSetterFunc)gncInvoiceSetTerms },
+    { INVOICE_BILLTO,  GNC_ID_OWNER, (QofAccessFunc)gncInvoiceGetBillTo, NULL  },
+    { INVOICE_ENTRIES, QOF_TYPE_COLLECT, (QofAccessFunc)qofInvoiceGetEntries, (QofSetterFunc)qofInvoiceSetEntries },
+    { INVOICE_JOB,     GNC_ID_JOB,      (QofAccessFunc)qofInvoiceGetJob,     (QofSetterFunc)qofInvoiceSetJob }, 
+    { QOF_PARAM_ACTIVE, QOF_TYPE_BOOLEAN, (QofAccessFunc)gncInvoiceGetActive, (QofSetterFunc)gncInvoiceSetActive },
+    { QOF_PARAM_BOOK, QOF_ID_BOOK, (QofAccessFunc)qof_instance_get_book, NULL },
+    { QOF_PARAM_GUID, QOF_TYPE_GUID, (QofAccessFunc)qof_instance_get_guid, NULL },
     { NULL },
   };
 
-  gncQueryObjectRegister (_GNC_MOD_NAME, (QuerySort)gncInvoiceCompare, params);
+  qof_class_register (_GNC_MOD_NAME, (QofSortFunc)gncInvoiceCompare, params);
   reg_lot ();
   reg_txn ();
 
-  return gncObjectRegister (&gncInvoiceDesc);
+  /* Make the compiler happy... */
+  if (0) {
+    qofInvoiceSetEntries(NULL, NULL);
+    qofInvoiceGetEntries(NULL);
+    qofInvoiceSetOwner(NULL, NULL);
+    qofInvoiceGetOwner(NULL);
+    qofInvoiceSetBillTo(NULL, NULL);
+    qofInvoiceGetBillTo(NULL);
+  }
+  if(!qof_choice_create(GNC_ID_INVOICE)) { return FALSE;}
+  return qof_object_register (&gncInvoiceDesc);
 }
 
-gint64 gncInvoiceNextID (GNCBook *book)
+gint64 gncInvoiceNextID (QofBook *book)
 {
-  return gnc_book_get_counter (book, _GNC_MOD_NAME);
+  return qof_book_get_counter (book, _GNC_MOD_NAME);
 }
