@@ -2,25 +2,51 @@
  * dialog-billterms.c -- Dialog to create and edit billing terms
  * Copyright (C) 2002 Derek Atkins
  * Author: Derek Atkins <warlord@MIT.EDU>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, contact:
+ *
+ * Free Software Foundation           Voice:  +1-617-542-5942
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org
  */
 
 #include "config.h"
 
-#include <gnome.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 
 #include "dialog-utils.h"
 #include "gnc-component-manager.h"
 #include "gnc-ui.h"
 #include "gnc-gui-query.h"
+#include "gnc-gconf-utils.h"
 #include "gnc-ui-util.h"
-#include "gnc-engine-util.h"
-#include "gnc-numeric.h"
+#include "qof.h"
 
 #include "gncBillTerm.h"
 #include "dialog-billterms.h"
 
 #define DIALOG_BILLTERMS_CM_CLASS "billterms-dialog"
 
+void billterms_row_selected (GtkCList *clist, gint row, gint column,
+			     GdkEventButton *event, gpointer user_data);
+void billterms_new_term_cb (GtkButton *button, BillTermsWindow *btw);
+void billterms_delete_term_cb (GtkButton *button, BillTermsWindow *btw);
+void billterms_edit_term_cb (GtkButton *button, BillTermsWindow *btw);
+void billterms_window_close (GtkWidget *widget, gpointer data);
+void billterms_window_destroy_cb (GtkWidget *widget, gpointer data);
+
 typedef struct _billterm_notebook {
   GtkTooltips *		tooltips;
   GtkWidget *		notebook;
@@ -83,7 +109,7 @@ read_widget (GladeXML *xml, char *name, gboolean read_only)
 /* NOTE: The caller needs to unref once they attach */
 static void
 init_notebook_widgets (BillTermNB *notebook, gboolean read_only,
-		       GnomeDialog *dialog, gpointer user_data)
+		       GtkDialog *dialog, gpointer user_data)
 {
   GladeXML *xml;
   GtkWidget *parent;
@@ -108,7 +134,7 @@ init_notebook_widgets (BillTermNB *notebook, gboolean read_only,
   notebook->prox_cutoff = read_widget (xml, "prox:cutoff_day", read_only);
 
   /* Disconnect the notebook from the window */
-  gtk_object_ref (GTK_OBJECT (notebook->notebook));
+  g_object_ref (notebook->notebook);
   gtk_container_remove (GTK_CONTAINER (parent), notebook->notebook);
   gtk_widget_destroy (parent);
 
@@ -122,7 +148,7 @@ set_numeric (GtkWidget *widget, GncBillTerm *term,
   gnc_numeric val;
   gdouble fl = 0.0;
 
-  fl = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (widget));
+  fl = gtk_spin_button_get_value (GTK_SPIN_BUTTON (widget));
   val = double_to_gnc_numeric (fl, 100000, GNC_RND_ROUND);
   func (term, val);
 }
@@ -165,7 +191,7 @@ ui_to_billterm (NewBillTerm *nbt)
 {
   BillTermNB *notebook;
   GncBillTerm *term;
-  char *text;
+  const char *text;
 
   term = nbt->this_term;
   notebook = &nbt->notebook;
@@ -228,26 +254,25 @@ verify_term_ok (NewBillTerm *nbt)
   num = gnc_numeric_zero ();
   if (gnc_numeric_negative_p (num)) {
     message = _("Negative amounts are not allowed.");
-    gnc_error_dialog_parented (GTK_WINDOW (nbt->dialog), message);
+    gnc_error_dialog (nbt->dialog, message);
     return FALSE;
   }
   if (gnc_numeric_compare (num, gnc_numeric_create (100, 1)) > 0) {
     message = _("Percentage amount must be between 0 and 100.");
-    gnc_error_dialog_parented (GTK_WINDOW (nbt->dialog), message);
+    gnc_error_dialog (nbt->dialog, message);
     return FALSE;
   }
   return TRUE;
 }
 
-static void
-new_billterm_ok_cb (GtkWidget *widget, gpointer data)
+static gboolean
+new_billterm_ok_cb (NewBillTerm *nbt)
 {
-  NewBillTerm *nbt = data;
   BillTermsWindow *btw;
-  char *name = NULL;
+  const char *name = NULL;
   char *message;
 
-  g_return_if_fail (nbt);
+  g_return_val_if_fail (nbt, FALSE);
   btw = nbt->btw;
 
   /* Verify that we've got real, valid data */
@@ -257,22 +282,22 @@ new_billterm_ok_cb (GtkWidget *widget, gpointer data)
     name = gtk_entry_get_text (GTK_ENTRY (nbt->name_entry));
     if (name == NULL || *name == '\0') {
       message = _("You must provide a name for this Billing Term.");
-      gnc_error_dialog_parented (GTK_WINDOW (nbt->dialog), message);
-      return;
+      gnc_error_dialog (nbt->dialog, message);
+      return FALSE;
     }
     if (gncBillTermLookupByName (btw->book, name)) {
       message = g_strdup_printf(_(
-			 "You must provide a unique name for this Billing Term.\n"
+			 "You must provide a unique name for this Billing Term. "
 			 "Your choice \"%s\" is already in use."), name);
-      gnc_error_dialog_parented (GTK_WINDOW (nbt->dialog), "%s", message);
+      gnc_error_dialog (nbt->dialog, "%s", message);
       g_free (message);
-      return;
+      return FALSE;
     }
   }
 
   /* Verify the actual data */
   if (!verify_term_ok (nbt))
-    return;
+    return FALSE;
 
   gnc_suspend_gui_refresh ();
 
@@ -294,33 +319,15 @@ new_billterm_ok_cb (GtkWidget *widget, gpointer data)
   gncBillTermCommitEdit (btw->current_term);
 
   gnc_resume_gui_refresh();
-
-  /* Then close/refresh the dialog/window */
-  gnome_dialog_close (GNOME_DIALOG (nbt->dialog));
-}
-
-static void
-new_billterm_cancel_cb (GtkWidget *widget, gpointer data)
-{
-  NewBillTerm *nbt = data;
-  g_return_if_fail (nbt);
-  gnome_dialog_close (GNOME_DIALOG (nbt->dialog));
-}
-
-static void
-new_billterm_dialog_destroy_cb (GtkWidget *widget, gpointer data)
-{
-  NewBillTerm *nbt = data;
-
-  if (!nbt) return;
-  g_free (nbt);
+  return TRUE;
 }
 
 static void
 show_notebook (BillTermNB *notebook)
 {
   g_return_if_fail (notebook->type > 0);
-  gtk_notebook_set_page (GTK_NOTEBOOK (notebook->notebook), notebook->type-1);
+  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook->notebook),
+				 notebook->type-1);
 }
 
 static void
@@ -362,7 +369,7 @@ make_menu (GtkWidget *omenu, NewBillTerm *nbt)
   popup = glade_xml_get_widget (xml, "Term Type Popup");
 
   /* Glade insists on making this a tearoff menu. */
-  if (gnome_preferences_get_menus_have_tearoff ()) {
+  if (gnc_gconf_menus_have_tearoff()) {
     GtkMenuShell *ms = GTK_MENU_SHELL (popup);
     GtkWidget *tearoff;
 
@@ -373,21 +380,26 @@ make_menu (GtkWidget *omenu, NewBillTerm *nbt)
 
   /* attach the signal handlers */
   glade_xml_signal_connect_data (xml, "on_days1_activate",
-				 on_days1_activate, nbt);
+				 G_CALLBACK (on_days1_activate), nbt);
   glade_xml_signal_connect_data (xml, "on_proximo1_activate",
-				 on_proximo1_activate, nbt);
+				 G_CALLBACK (on_proximo1_activate), nbt);
 
   gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), popup);
   gtk_option_menu_set_history (GTK_OPTION_MENU (omenu),
 			       nbt->notebook.type - 1);
 }
 
-static NewBillTerm *
-new_billterm_dialog (BillTermsWindow *btw, GncBillTerm *term)
+static GncBillTerm *
+new_billterm_dialog (BillTermsWindow *btw, GncBillTerm *term,
+		     const char *name)
 {
+  GncBillTerm *created_term = NULL;
   NewBillTerm *nbt;
   GladeXML *xml;
-  GtkWidget *box, *widget;
+  GtkWidget *box;
+  gint response;
+  gboolean done;
+  const gchar *dialog_name;
 
   if (!btw) return NULL;
 
@@ -396,19 +408,22 @@ new_billterm_dialog (BillTermsWindow *btw, GncBillTerm *term)
   nbt->this_term = term;
 
   /* Open and read the XML */
-  xml = gnc_glade_xml_new ("billterms.glade", "New Term Dialog");
-  nbt->dialog = glade_xml_get_widget (xml, "New Term Dialog");
+  dialog_name = term ? "Edit Term Dialog" : "New Term Dialog";
+  xml = gnc_glade_xml_new ("billterms.glade", dialog_name);
+  nbt->dialog = glade_xml_get_widget (xml, dialog_name);
   nbt->name_entry = glade_xml_get_widget (xml, "name_entry");
   nbt->desc_entry = glade_xml_get_widget (xml, "desc_entry");
+  if (name)
+    gtk_entry_set_text (GTK_ENTRY (nbt->name_entry), name);
 
   /* Initialize the notebook widgets */
   init_notebook_widgets (&nbt->notebook, FALSE,
-			 GNOME_DIALOG (nbt->dialog), nbt);
+			 GTK_DIALOG (nbt->dialog), nbt);
 
   /* Attach the notebook */
   box = glade_xml_get_widget (xml, "notebook_box");
   gtk_box_pack_start (GTK_BOX (box), nbt->notebook.notebook, TRUE, TRUE, 0);
-  gtk_object_unref (GTK_OBJECT (nbt->notebook.notebook));
+  g_object_unref (nbt->notebook.notebook);
 
   /* Fill in the widgets appropriately */
   if (term)
@@ -422,39 +437,41 @@ new_billterm_dialog (BillTermsWindow *btw, GncBillTerm *term)
   /* Show the right notebook page */
   show_notebook (&nbt->notebook);
 
-  /* Make 'enter' do the right thing */
-  gnome_dialog_set_default (GNOME_DIALOG (nbt->dialog), 0);
-  gnome_dialog_editable_enters (GNOME_DIALOG (nbt->dialog),
-				GTK_EDITABLE (nbt->name_entry));
-  gnome_dialog_editable_enters (GNOME_DIALOG (nbt->dialog),
-				GTK_EDITABLE (nbt->desc_entry));
-
-  /* Connect the dialog buttons */
-  gnome_dialog_button_connect (GNOME_DIALOG (nbt->dialog), 0,
-			       new_billterm_ok_cb, nbt);
-
-  gnome_dialog_button_connect (GNOME_DIALOG (nbt->dialog), 1,
-			       new_billterm_cancel_cb, nbt);
-
-  /* Set our modality */
-  gnome_dialog_set_parent (GNOME_DIALOG (nbt->dialog),
-			   GTK_WINDOW (btw->dialog));
-  gtk_window_set_modal (GTK_WINDOW (nbt->dialog), TRUE);
-
-  gtk_signal_connect (GTK_OBJECT (nbt->dialog), "destroy",
-		      new_billterm_dialog_destroy_cb, nbt);
+  /* Setup signals */
+  glade_xml_signal_autoconnect_full( xml,
+                                     gnc_glade_autoconnect_full_func,
+                                     nbt);
 
+  gtk_window_set_transient_for (GTK_WINDOW(nbt->dialog),
+				GTK_WINDOW(btw->dialog));
 
   /* Show what we should */
   gtk_widget_show_all (nbt->dialog);
   if (term) {
-    widget = glade_xml_get_widget (xml, "term_frame");
-    gtk_widget_hide_all (widget);
     gtk_widget_grab_focus (nbt->desc_entry);
   } else
     gtk_widget_grab_focus (nbt->name_entry);
 
-  return nbt;
+  done = FALSE;
+  while (!done) {
+    response = gtk_dialog_run (GTK_DIALOG (nbt->dialog));
+    switch (response) {
+     case GTK_RESPONSE_OK:
+      if (new_billterm_ok_cb (nbt)) {
+	created_term = nbt->this_term;
+	done = TRUE;
+      }
+      break;
+     default:
+      done = TRUE;
+      break;
+    }
+  }
+
+  gtk_widget_destroy(nbt->dialog);
+  g_free(nbt);
+
+  return created_term;
 }
 
 /***********************************************************************/
@@ -527,7 +544,7 @@ billterms_window_refresh (BillTermsWindow *btw)
 
     gnc_gui_component_watch_entity (btw->component_id,
 				    gncBillTermGetGUID (term),
-				    GNC_EVENT_MODIFY);
+				    QOF_EVENT_MODIFY);
 
     row_text[0] = (char *)gncBillTermGetName (term);
     row_text[1] = NULL;
@@ -541,7 +558,7 @@ billterms_window_refresh (BillTermsWindow *btw)
 
   gnc_gui_component_watch_entity_type (btw->component_id,
 				       GNC_BILLTERM_MODULE_NAME,
-				       GNC_EVENT_CREATE | GNC_EVENT_DESTROY);
+				       QOF_EVENT_CREATE | QOF_EVENT_DESTROY);
 
   if (vadjustment) {
     save_value = CLAMP (save_value, vadjustment->lower,
@@ -566,7 +583,7 @@ billterms_window_refresh (BillTermsWindow *btw)
   /* select_row() above will refresh the term display */
 }
 
-static void
+void
 billterms_row_selected (GtkCList *clist, gint row, gint column,
 			GdkEventButton *event, gpointer user_data)
 {
@@ -585,17 +602,17 @@ billterms_row_selected (GtkCList *clist, gint row, gint column,
 
   /* If the user double-clicked on the item, pop up the edit window */
   if (event && event->type == GDK_2BUTTON_PRESS)
-    new_billterm_dialog (btw, term);
+    new_billterm_dialog (btw, term, NULL);
 }
 
-static void
+void
 billterms_new_term_cb (GtkButton *button, BillTermsWindow *btw)
 {
   g_return_if_fail (btw);
-  new_billterm_dialog (btw, NULL);
+  new_billterm_dialog (btw, NULL, NULL);
 }
 
-static void
+void
 billterms_delete_term_cb (GtkButton *button, BillTermsWindow *btw)
 {
   g_return_if_fail (btw);
@@ -604,15 +621,15 @@ billterms_delete_term_cb (GtkButton *button, BillTermsWindow *btw)
     return;
 
   if (gncBillTermGetRefcount (btw->current_term) > 0) {
-    gnc_error_dialog_parented (GTK_WINDOW (btw->dialog),
-			       _("Term \"%s\" is in use.  You cannot delete it."),
-			       gncBillTermGetName (btw->current_term));
+    gnc_error_dialog (btw->dialog,
+		      _("Term \"%s\" is in use.  You cannot delete it."),
+		      gncBillTermGetName (btw->current_term));
     return;
   }
 
-  if (gnc_verify_dialog_parented (btw->dialog, FALSE,
-				  _("Are you sure you want to delete \"%s\"?"),
-				  gncBillTermGetName (btw->current_term))) {
+  if (gnc_verify_dialog (btw->dialog, FALSE,
+			 _("Are you sure you want to delete \"%s\"?"),
+			 gncBillTermGetName (btw->current_term))) {
     /* Ok, let's remove it */
     gnc_suspend_gui_refresh ();
     gncBillTermBeginEdit (btw->current_term);
@@ -622,13 +639,13 @@ billterms_delete_term_cb (GtkButton *button, BillTermsWindow *btw)
   }
 }
 
-static void
+void
 billterms_edit_term_cb (GtkButton *button, BillTermsWindow *btw)
 {
   g_return_if_fail (btw);
   if (!btw->current_term)
     return;
-  new_billterm_dialog (btw, btw->current_term);
+  new_billterm_dialog (btw, btw->current_term, NULL);
 }
 
 static void
@@ -645,10 +662,11 @@ billterms_window_close_handler (gpointer data)
 {
   BillTermsWindow *btw = data;
   g_return_if_fail (btw);
-  gnome_dialog_close (GNOME_DIALOG (btw->dialog));
+
+  gtk_widget_destroy (btw->dialog);
 }
 
-static void
+void
 billterms_window_close (GtkWidget *widget, gpointer data)
 {
   BillTermsWindow *btw = data;
@@ -656,7 +674,7 @@ billterms_window_close (GtkWidget *widget, gpointer data)
   gnc_ui_billterms_window_destroy (btw);
 }
 
-static void
+void
 billterms_window_destroy_cb (GtkWidget *widget, gpointer data)
 {
   BillTermsWindow *btw = data;
@@ -683,7 +701,7 @@ gnc_ui_billterms_window_new (GNCBook *book)
 {
   BillTermsWindow *btw;
   GladeXML *xml;
-  GtkWidget *button, *widget;
+  GtkWidget *widget;
 
   if (!book) return NULL;
 
@@ -711,40 +729,20 @@ gnc_ui_billterms_window_new (GNCBook *book)
   btw->type_label = glade_xml_get_widget (xml, "type_label");
   btw->term_vbox = glade_xml_get_widget (xml, "term_vbox");
 
-  gnome_dialog_editable_enters (GNOME_DIALOG (btw->dialog),
-				GTK_EDITABLE (btw->desc_entry));
-
   /* Initialize the notebook widgets */
   init_notebook_widgets (&btw->notebook, TRUE,
-			 GNOME_DIALOG (btw->dialog), btw);
+			 GTK_DIALOG (btw->dialog), btw);
 
   /* Attach the notebook */
   widget = glade_xml_get_widget (xml, "notebook_box");
   gtk_box_pack_start (GTK_BOX (widget), btw->notebook.notebook,
 		      TRUE, TRUE, 0);
-  gtk_object_unref (GTK_OBJECT (btw->notebook.notebook));
+  g_object_unref (btw->notebook.notebook);
 
-  /* Connect all the buttons */
-  button = glade_xml_get_widget (xml, "new_term_button");
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      billterms_new_term_cb, btw);
-  button = glade_xml_get_widget (xml, "delete_term_button");
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      billterms_delete_term_cb, btw);
-  button = glade_xml_get_widget (xml, "edit_term_button");
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      billterms_edit_term_cb, btw);
-
-  /* Set the row-select callbacks */
-  gtk_signal_connect (GTK_OBJECT (btw->terms_clist), "select-row",
-		      billterms_row_selected, btw);
-
-  /* Connect the dialog buttons */
-  gnome_dialog_button_connect (GNOME_DIALOG (btw->dialog), 0,
-			       billterms_window_close, btw);
-
-  gtk_signal_connect (GTK_OBJECT (btw->dialog), "destroy",
-		      billterms_window_destroy_cb, btw);
+  /* Setup signals */
+  glade_xml_signal_autoconnect_full( xml,
+                                     gnc_glade_autoconnect_full_func,
+                                     btw);
 
   /* register with component manager */
   btw->component_id =
@@ -770,50 +768,17 @@ gnc_ui_billterms_window_destroy (BillTermsWindow *btw)
 }
 
 #if 0
-static int
-from_name_close_cb (GnomeDialog *dialog, gpointer data)
-{
-  NewBillTerm *nbt;
-  GncBillTerm **created_term = data;
-
-  nbt = gtk_object_get_data (GTK_OBJECT (dialog), "dialog_info");
-
-  *created_term = nbt->created_term;
-
-  gtk_main_quit ();
-
-  return FALSE;
-}
-
 /* Create a new billterms by name */
 GncBillTerm *
 gnc_ui_billterms_new_from_name (GNCBook *book, const char *name)
 {
-  GncBillTerm *created_term = NULL;
   BillTermsWindow *btw;
-  NewBillTerm *nbt;
 
   if (!book) return NULL;
 
   btw = gnc_ui_billterms_window_new (book);
   if (!btw) return NULL;
 
-  nbt = new_billterm_dialog (btw, NULL);
-  if (!nbt) return NULL;
-
-  gtk_object_set_data (GTK_OBJECT (nbt->dialog), "dialog_info", nbt);
-  gtk_signal_connect (GTK_OBJECT (nbt->dialog), "close",
-		      GTK_SIGNAL_FUNC (from_name_close_cb), &created_term);
-
-  /* Preset the name in the new dialog */
-  if (name)
-    gtk_entry_set_text (GTK_ENTRY (nbt->name_entry), name);
-
-  /* I know that NBT is already modal, no need to reset it here */
-
-  /* Now run the dialog -- wait for it to close */
-  gtk_main ();
-
-  return created_term;
+  return new_billterm_dialog (btw, NULL, name);
 }
 #endif
