@@ -2,22 +2,38 @@
  * dialog-employee.c -- Dialog for Employee entry
  * Copyright (C) 2001 Derek Atkins
  * Author: Derek Atkins <warlord@MIT.EDU>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, contact:
+ *
+ * Free Software Foundation           Voice:  +1-617-542-5942
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org
  */
 
 #include "config.h"
 
-#include <gnome.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 
 #include "dialog-utils.h"
-#include "global-options.h"
 #include "gnc-amount-edit.h"
 #include "gnc-currency-edit.h"
 #include "gnc-component-manager.h"
 #include "gnc-ui.h"
 #include "gnc-gui-query.h"
 #include "gnc-ui-util.h"
-#include "gnc-engine-util.h"
-#include "window-help.h"
+#include "qof.h"
 #include "dialog-search.h"
 #include "search-param.h"
 #include "gnc-account-sel.h"
@@ -34,6 +50,15 @@
 #define DIALOG_NEW_EMPLOYEE_CM_CLASS "dialog-new-employee"
 #define DIALOG_EDIT_EMPLOYEE_CM_CLASS "dialog-edit-employee"
 
+#define GCONF_SECTION_SEARCH "dialogs/business/employee_search"
+
+void gnc_employee_window_ok_cb (GtkWidget *widget, gpointer data);
+void gnc_employee_window_cancel_cb (GtkWidget *widget, gpointer data);
+void gnc_employee_window_help_cb (GtkWidget *widget, gpointer data);
+void gnc_employee_window_destroy_cb (GtkWidget *widget, gpointer data);
+void gnc_employee_name_changed_cb (GtkWidget *widget, gpointer data);
+void gnc_employee_ccard_acct_toggled_cb (GtkToggleButton *button, gpointer data);
+
 typedef enum
 {
   NEW_EMPLOYEE,
@@ -151,7 +176,7 @@ static gboolean check_edit_amount (GtkWidget *dialog, GtkWidget *amount,
 {
   if (!gnc_amount_edit_evaluate (GNC_AMOUNT_EDIT (amount))) {
     if (error_message)
-      gnc_error_dialog_parented (GTK_WINDOW (dialog), error_message);
+      gnc_error_dialog (dialog, error_message);
     return TRUE;
   }
   /* We've got a valid-looking number; check mix/max */
@@ -160,7 +185,7 @@ static gboolean check_edit_amount (GtkWidget *dialog, GtkWidget *amount,
     if ((min && gnc_numeric_compare (*min, val) > 0) ||
 	(max && gnc_numeric_compare (val, *max) > 0)) {
       if (error_message)
-	gnc_error_dialog_parented (GTK_WINDOW (dialog), error_message);
+	gnc_error_dialog (dialog, error_message);
       return TRUE;
     }
   }
@@ -174,16 +199,17 @@ static gboolean check_entry_nonempty (GtkWidget *dialog, GtkWidget *entry,
   const char *res = gtk_entry_get_text (GTK_ENTRY (entry));
   if (safe_strcmp (res, "") == 0) {
     if (error_message)
-      gnc_error_dialog_parented (GTK_WINDOW (dialog), "%s", error_message);
+      gnc_error_dialog (dialog, "%s", error_message);
     return TRUE;
   }
   return FALSE;
 }
 
-static void
+void
 gnc_employee_window_ok_cb (GtkWidget *widget, gpointer data)
 {
   EmployeeWindow *ew = data;
+  gchar *string;
 
   /* Check for valid username */
   if (check_entry_nonempty (ew->dialog, ew->username_entry,
@@ -201,15 +227,16 @@ gnc_employee_window_ok_cb (GtkWidget *widget, gpointer data)
       check_entry_nonempty (ew->dialog, ew->addr3_entry, NULL) &&
       check_entry_nonempty (ew->dialog, ew->addr4_entry, NULL)) {
     const char *msg = _("You must enter an address.");
-    gnc_error_dialog_parented (GTK_WINDOW (ew->dialog), msg);
+    gnc_error_dialog (ew->dialog, msg);
     return;
   }
 
   /* Set the employee id if one has not been chosen */
   if (safe_strcmp (gtk_entry_get_text (GTK_ENTRY (ew->id_entry)), "") == 0) {
-    gtk_entry_set_text (GTK_ENTRY (ew->id_entry),
-			g_strdup_printf ("%.6lld",
-					 gncEmployeeNextID (ew->book)));
+    string = g_strdup_printf ("%.6" G_GINT64_FORMAT,
+			      gncEmployeeNextID (ew->book));
+    gtk_entry_set_text (GTK_ENTRY (ew->id_entry), string);
+    g_free(string);
   }
 
   /* Now save it off */
@@ -225,7 +252,7 @@ gnc_employee_window_ok_cb (GtkWidget *widget, gpointer data)
   gnc_close_gui_component (ew->component_id);
 }
 
-static void
+void
 gnc_employee_window_cancel_cb (GtkWidget *widget, gpointer data)
 {
   EmployeeWindow *ew = data;
@@ -233,15 +260,13 @@ gnc_employee_window_cancel_cb (GtkWidget *widget, gpointer data)
   gnc_close_gui_component (ew->component_id);
 }
 
-static void
+void
 gnc_employee_window_help_cb (GtkWidget *widget, gpointer data)
 {
-  char *help_file = HH_EMPLOYEE;
-
-  helpWindow(NULL, NULL, help_file);
+  gnc_gnome_help(HF_HELP, HL_USAGE);
 }
 
-static void
+void
 gnc_employee_window_destroy_cb (GtkWidget *widget, gpointer data)
 {
   EmployeeWindow *ew = data;
@@ -261,11 +286,12 @@ gnc_employee_window_destroy_cb (GtkWidget *widget, gpointer data)
   g_free (ew);
 }
 
-static void
+void
 gnc_employee_name_changed_cb (GtkWidget *widget, gpointer data)
 {
   EmployeeWindow *ew = data;
-  char *name, *id, *fullname, *title;
+  char *fullname, *title;
+  const char *name, *id;
 
   if (!ew)
     return;
@@ -276,12 +302,12 @@ gnc_employee_name_changed_cb (GtkWidget *widget, gpointer data)
 
   id = gtk_entry_get_text (GTK_ENTRY (ew->id_entry));
 
-  fullname = g_strconcat (name, " (", id, ")", NULL);
+  fullname = g_strconcat (name, " (", id, ")", (char *)NULL);
 
   if (ew->dialog_type == EDIT_EMPLOYEE)
-    title = g_strconcat (_("Edit Employee"), " - ", fullname, NULL);
+    title = g_strconcat (_("Edit Employee"), " - ", fullname, (char *)NULL);
   else
-    title = g_strconcat (_("New Employee"), " - ", fullname, NULL);
+    title = g_strconcat (_("New Employee"), " - ", fullname, (char *)NULL);
 
   gtk_window_set_title (GTK_WINDOW (ew->dialog), title);
 
@@ -289,7 +315,7 @@ gnc_employee_name_changed_cb (GtkWidget *widget, gpointer data)
   g_free (title);
 }
 
-static void
+void
 gnc_employee_ccard_acct_toggled_cb (GtkToggleButton *button, gpointer data)
 {
   EmployeeWindow *ew = data;
@@ -311,7 +337,7 @@ gnc_employee_window_close_handler (gpointer user_data)
 {
   EmployeeWindow *ew = user_data;
 
-  gnome_dialog_close (GNOME_DIALOG (ew->dialog));
+  gtk_widget_destroy (ew->dialog);
 }
 
 static void
@@ -330,7 +356,7 @@ gnc_employee_window_refresh_handler (GHashTable *changes, gpointer user_data)
   /* Next, close if this is a destroy event */
   if (changes) {
     info = gnc_gui_get_entity_events (changes, &ew->employee_guid);
-    if (info && (info->event_mask & GNC_EVENT_DESTROY)) {
+    if (info && (info->event_mask & QOF_EVENT_DESTROY)) {
       gnc_close_gui_component (ew->component_id);
       return;
     }
@@ -353,7 +379,6 @@ gnc_employee_new_window (GNCBook *bookp,
   EmployeeWindow *ew;
   GladeXML *xml;
   GtkWidget *hbox, *edit;
-  GnomeDialog *ewd;
   gnc_commodity *currency;
   GNCPrintAmountInfo print_info;
   GList *acct_types;
@@ -391,12 +416,8 @@ gnc_employee_new_window (GNCBook *bookp,
   /* Find the dialog */
   xml = gnc_glade_xml_new ("employee.glade", "Employee Dialog");
   ew->dialog = glade_xml_get_widget (xml, "Employee Dialog");
-  ewd = GNOME_DIALOG (ew->dialog);
 
-  gtk_object_set_data (GTK_OBJECT (ew->dialog), "dialog_info", ew);
-
-  /* default to ok */
-  gnome_dialog_set_default (ewd, 0);
+  g_object_set_data (G_OBJECT (ew->dialog), "dialog_info", ew);
 
   /* Get entry points */
   ew->id_entry = glade_xml_get_widget (xml, "id_entry");
@@ -462,45 +483,10 @@ gnc_employee_new_window (GNCBook *bookp,
   hbox = glade_xml_get_widget (xml, "ccard_acct_hbox");
   gtk_box_pack_start (GTK_BOX (hbox), edit, TRUE, TRUE, 0);
 
-
-  /* Setup Dialog for Editing */
-  gnome_dialog_set_default (ewd, 0);
-
-  /* Attach <Enter> to default button */
-  gnome_dialog_editable_enters (ewd, GTK_EDITABLE (ew->id_entry));
-  gnome_dialog_editable_enters (ewd, GTK_EDITABLE (ew->username_entry));
-
-  gnome_dialog_editable_enters (ewd, GTK_EDITABLE (ew->name_entry));
-  gnome_dialog_editable_enters (ewd, GTK_EDITABLE (ew->addr1_entry));
-  gnome_dialog_editable_enters (ewd, GTK_EDITABLE (ew->addr2_entry));
-  gnome_dialog_editable_enters (ewd, GTK_EDITABLE (ew->addr3_entry));
-  gnome_dialog_editable_enters (ewd, GTK_EDITABLE (ew->addr4_entry));
-  gnome_dialog_editable_enters (ewd, GTK_EDITABLE (ew->phone_entry));
-  gnome_dialog_editable_enters (ewd, GTK_EDITABLE (ew->fax_entry));
-  gnome_dialog_editable_enters (ewd, GTK_EDITABLE (ew->email_entry));
-
-  /* Set focus to username */
-  gtk_widget_grab_focus (ew->username_entry);
-
   /* Setup signals */
-  gnome_dialog_button_connect
-    (ewd, 0, GTK_SIGNAL_FUNC(gnc_employee_window_ok_cb), ew);
-  gnome_dialog_button_connect
-    (ewd, 1, GTK_SIGNAL_FUNC(gnc_employee_window_cancel_cb), ew);
-  gnome_dialog_button_connect
-    (ewd, 2, GTK_SIGNAL_FUNC(gnc_employee_window_help_cb), ew);
-
-  gtk_signal_connect (GTK_OBJECT (ew->dialog), "destroy",
-		      GTK_SIGNAL_FUNC(gnc_employee_window_destroy_cb), ew);
-
-  gtk_signal_connect(GTK_OBJECT (ew->id_entry), "changed",
-		     GTK_SIGNAL_FUNC(gnc_employee_name_changed_cb), ew);
-
-  gtk_signal_connect(GTK_OBJECT (ew->username_entry), "changed",
-		     GTK_SIGNAL_FUNC(gnc_employee_name_changed_cb), ew);
-
-  gtk_signal_connect(GTK_OBJECT (ew->ccard_acct_check), "toggled",
-		     GTK_SIGNAL_FUNC(gnc_employee_ccard_acct_toggled_cb), ew);
+  glade_xml_signal_autoconnect_full( xml,
+                                     gnc_glade_autoconnect_full_func,
+                                     ew);
 
   /* Setup initial values */
   if (employee != NULL) {
@@ -570,7 +556,7 @@ gnc_employee_new_window (GNCBook *bookp,
 
   gnc_gui_component_watch_entity_type (ew->component_id,
 				       GNC_EMPLOYEE_MODULE_NAME,
-				       GNC_EVENT_MODIFY | GNC_EVENT_DESTROY);
+				       QOF_EVENT_MODIFY | QOF_EVENT_DESTROY);
 
   gtk_widget_show_all (ew->dialog);
 
@@ -738,9 +724,11 @@ gnc_employee_search (GncEmployee *start, GNCBook *book)
   sw->book = book;
   sw->q = q;
 
-  return gnc_search_dialog_create (type, params, columns, q, q2,
+  return gnc_search_dialog_create (type, _("Find Employee"),
+				   params, columns, q, q2,
 				   buttons, NULL, new_employee_cb,
-				   sw, free_employee_cb);
+				   sw, free_employee_cb,
+				   GCONF_SECTION_SEARCH, NULL);
 }
 
 GNCSearchWindow *
