@@ -2,21 +2,37 @@
  * dialog-vendor.c -- Dialog for Vendor entry
  * Copyright (C) 2001 Derek Atkins
  * Author: Derek Atkins <warlord@MIT.EDU>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, contact:
+ *
+ * Free Software Foundation           Voice:  +1-617-542-5942
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org
  */
 
 #include "config.h"
 
-#include <gnome.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 
 #include "dialog-utils.h"
-#include "global-options.h"
 #include "gnc-currency-edit.h"
 #include "gnc-component-manager.h"
 #include "gnc-ui.h"
 #include "gnc-gui-query.h"
 #include "gnc-ui-util.h"
-#include "gnc-engine-util.h"
-#include "window-help.h"
+#include "qof.h"
 #include "dialog-search.h"
 #include "search-param.h"
 
@@ -34,6 +50,15 @@
 #define DIALOG_NEW_VENDOR_CM_CLASS "dialog-new-vendor"
 #define DIALOG_EDIT_VENDOR_CM_CLASS "dialog-edit-vendor"
 
+#define GCONF_SECTION_SEARCH "dialogs/business/vendor_search"
+
+void gnc_vendor_taxtable_check_cb (GtkToggleButton *togglebutton, gpointer user_data);
+void gnc_vendor_window_ok_cb (GtkWidget *widget, gpointer data);
+void gnc_vendor_window_cancel_cb (GtkWidget *widget, gpointer data);
+void gnc_vendor_window_help_cb (GtkWidget *widget, gpointer data);
+void gnc_vendor_window_destroy_cb (GtkWidget *widget, gpointer data);
+void gnc_vendor_name_changed_cb (GtkWidget *widget, gpointer data);
+
 typedef enum
 {
   NEW_VENDOR,
@@ -80,7 +105,7 @@ struct _vendor_window {
   GncTaxTable *	taxtable;
 };
 
-static void
+void
 gnc_vendor_taxtable_check_cb (GtkToggleButton *togglebutton,
 				gpointer user_data)
 {
@@ -103,6 +128,9 @@ vw_get_vendor (VendorWindow *vw)
 
 static void gnc_ui_to_vendor (VendorWindow *vw, GncVendor *vendor)
 {
+  GtkTextBuffer* text_buffer;
+  GtkTextIter start, end;
+  gchar *text;
   GncAddress *addr;
 
   addr = gncVendorGetAddr (vendor);
@@ -135,8 +163,12 @@ static void gnc_ui_to_vendor (VendorWindow *vw, GncVendor *vendor)
   gncVendorSetActive (vendor, gtk_toggle_button_get_active
 			(GTK_TOGGLE_BUTTON (vw->active_check)));
   gncVendorSetTaxIncluded (vendor, vw->taxincluded);
-  gncVendorSetNotes (vendor, gtk_editable_get_chars
-		       (GTK_EDITABLE (vw->notes_text), 0, -1));
+
+  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(vw->notes_text));
+  gtk_text_buffer_get_bounds (text_buffer, &start, &end);
+  text = gtk_text_buffer_get_text (text_buffer, &start, &end, FALSE);
+  gncVendorSetNotes (vendor, text);
+
   gncVendorSetTerms (vendor, vw->terms);
   gncVendorSetCurrency (vendor,
 			gnc_currency_edit_get_currency (GNC_CURRENCY_EDIT
@@ -156,20 +188,21 @@ static gboolean check_entry_nonempty (GtkWidget *dialog, GtkWidget *entry,
   const char *res = gtk_entry_get_text (GTK_ENTRY (entry));
   if (safe_strcmp (res, "") == 0) {
     if (error_message)
-      gnc_error_dialog_parented (GTK_WINDOW (dialog), "%s", error_message);
+      gnc_error_dialog (dialog, "%s", error_message);
     return TRUE;
   }
   return FALSE;
 }
 
-static void
+void
 gnc_vendor_window_ok_cb (GtkWidget *widget, gpointer data)
 {
   VendorWindow *vw = data;
+  gchar *string;
 
   /* Check for valid company name */
   if (check_entry_nonempty (vw->dialog, vw->company_entry,
-		   _("You must enter a company name.\n"
+		   _("You must enter a company name. "
 		     "If this vendor is an individual (and not a company) "
 		     "you should set the \"company name\" and \"contact name\" "
 		     "the same.")))
@@ -181,15 +214,17 @@ gnc_vendor_window_ok_cb (GtkWidget *widget, gpointer data)
       check_entry_nonempty (vw->dialog, vw->addr3_entry, NULL) &&
       check_entry_nonempty (vw->dialog, vw->addr4_entry, NULL)) {
     const char *msg = _("You must enter a payment address.");
-    gnc_error_dialog_parented (GTK_WINDOW (vw->dialog), msg);
+    gnc_error_dialog (vw->dialog, msg);
     return;
   }
 
   /* Check for valid id and set one if necessary */
-  if (safe_strcmp (gtk_entry_get_text (GTK_ENTRY (vw->id_entry)), "") == 0)
-    gtk_entry_set_text (GTK_ENTRY (vw->id_entry),
-			g_strdup_printf ("%.6lld",
-					 gncVendorNextID(vw->book)));
+  if (safe_strcmp (gtk_entry_get_text (GTK_ENTRY (vw->id_entry)), "") == 0) {
+    string = g_strdup_printf ("%.6" G_GINT64_FORMAT,
+			      gncVendorNextID(vw->book));
+    gtk_entry_set_text (GTK_ENTRY (vw->id_entry), string);
+    g_free(string);
+  }
 
   /* Now save it off */
   {
@@ -204,7 +239,7 @@ gnc_vendor_window_ok_cb (GtkWidget *widget, gpointer data)
   gnc_close_gui_component (vw->component_id);
 }
 
-static void
+void
 gnc_vendor_window_cancel_cb (GtkWidget *widget, gpointer data)
 {
   VendorWindow *vw = data;
@@ -212,15 +247,13 @@ gnc_vendor_window_cancel_cb (GtkWidget *widget, gpointer data)
   gnc_close_gui_component (vw->component_id);
 }
 
-static void
+void
 gnc_vendor_window_help_cb (GtkWidget *widget, gpointer data)
 {
-  char *help_file = HH_VENDOR;
-
-  helpWindow(NULL, NULL, help_file);
+  gnc_gnome_help(HF_HELP, HL_USAGE);
 }
 
-static void
+void
 gnc_vendor_window_destroy_cb (GtkWidget *widget, gpointer data)
 {
   VendorWindow *vw = data;
@@ -240,7 +273,7 @@ gnc_vendor_window_destroy_cb (GtkWidget *widget, gpointer data)
   g_free (vw);
 }
 
-static void
+void
 gnc_vendor_name_changed_cb (GtkWidget *widget, gpointer data)
 {
   VendorWindow *vw = data;
@@ -249,21 +282,23 @@ gnc_vendor_name_changed_cb (GtkWidget *widget, gpointer data)
   if (!vw)
     return;
 
-  name = gtk_entry_get_text (GTK_ENTRY (vw->company_entry));
+  name = gtk_editable_get_chars (GTK_EDITABLE (vw->company_entry), 0, -1);
   if (!name || *name == '\0')
-    name = _("<No name>");
+    name = g_strdup (_("<No name>"));
 
-  id = gtk_entry_get_text (GTK_ENTRY (vw->id_entry));
+  id = gtk_editable_get_chars (GTK_EDITABLE (vw->id_entry), 0, -1);
 
-  fullname = g_strconcat (name, " (", id, ")", NULL);
+  fullname = g_strconcat (name, " (", id, ")", (char *)NULL);
 
   if (vw->dialog_type == EDIT_VENDOR)
-    title = g_strconcat (_("Edit Vendor"), " - ", fullname, NULL);
+    title = g_strconcat (_("Edit Vendor"), " - ", fullname, (char *)NULL);
   else
-    title = g_strconcat (_("New Vendor"), " - ", fullname, NULL);
+    title = g_strconcat (_("New Vendor"), " - ", fullname, (char *)NULL);
 
   gtk_window_set_title (GTK_WINDOW (vw->dialog), title);
 
+  g_free (name);
+  g_free (id);
   g_free (fullname);
   g_free (title);
 }
@@ -273,7 +308,7 @@ gnc_vendor_window_close_handler (gpointer user_data)
 {
   VendorWindow *vw = user_data;
 
-  gnome_dialog_close (GNOME_DIALOG (vw->dialog));
+  gtk_widget_destroy (vw->dialog);
 }
 
 static void
@@ -292,7 +327,7 @@ gnc_vendor_window_refresh_handler (GHashTable *changes, gpointer user_data)
   /* Next, close if this is a destroy event */
   if (changes) {
     info = gnc_gui_get_entity_events (changes, &vw->vendor_guid);
-    if (info && (info->event_mask & GNC_EVENT_DESTROY)) {
+    if (info && (info->event_mask & QOF_EVENT_DESTROY)) {
       gnc_close_gui_component (vw->component_id);
       return;
     }
@@ -313,7 +348,6 @@ gnc_vendor_new_window (GNCBook *bookp, GncVendor *vendor)
 {
   VendorWindow *vw;
   GladeXML *xml;
-  GnomeDialog *vwd;
   GtkWidget *edit, *hbox;
   gnc_commodity *currency;
 
@@ -349,12 +383,6 @@ gnc_vendor_new_window (GNCBook *bookp, GncVendor *vendor)
   /* Find the dialog */
   xml = gnc_glade_xml_new ("vendor.glade", "Vendor Dialog");
   vw->dialog = glade_xml_get_widget (xml, "Vendor Dialog");
-  vwd = GNOME_DIALOG (vw->dialog);
-
-  gtk_object_set_data (GTK_OBJECT (vw->dialog), "dialog_info", vw);
-
-  /* default to ok */
-  gnome_dialog_set_default (vwd, 0);
 
   /* Get entry points */
   vw->id_entry = glade_xml_get_widget (xml, "id_entry");
@@ -385,50 +413,16 @@ gnc_vendor_new_window (GNCBook *bookp, GncVendor *vendor)
   hbox = glade_xml_get_widget (xml, "currency_box");
   gtk_box_pack_start (GTK_BOX (hbox), edit, TRUE, TRUE, 0);
 
-  /* Setup Dialog for Editing */
-  gnome_dialog_set_default (vwd, 0);
-
-  /* Attach <Enter> to default button */
-  gnome_dialog_editable_enters (vwd, GTK_EDITABLE (vw->id_entry));
-  gnome_dialog_editable_enters (vwd, GTK_EDITABLE (vw->company_entry));
-
-  gnome_dialog_editable_enters (vwd, GTK_EDITABLE (vw->name_entry));
-  gnome_dialog_editable_enters (vwd, GTK_EDITABLE (vw->addr1_entry));
-  gnome_dialog_editable_enters (vwd, GTK_EDITABLE (vw->addr2_entry));
-  gnome_dialog_editable_enters (vwd, GTK_EDITABLE (vw->addr3_entry));
-  gnome_dialog_editable_enters (vwd, GTK_EDITABLE (vw->addr4_entry));
-  gnome_dialog_editable_enters (vwd, GTK_EDITABLE (vw->phone_entry));
-  gnome_dialog_editable_enters (vwd, GTK_EDITABLE (vw->fax_entry));
-  gnome_dialog_editable_enters (vwd, GTK_EDITABLE (vw->email_entry));
-
-  /* Set focus to company name */
-  gtk_widget_grab_focus (vw->company_entry);
-
   /* Setup signals */
-  gnome_dialog_button_connect
-    (vwd, 0, GTK_SIGNAL_FUNC(gnc_vendor_window_ok_cb), vw);
-  gnome_dialog_button_connect
-    (vwd, 1, GTK_SIGNAL_FUNC(gnc_vendor_window_cancel_cb), vw);
-  gnome_dialog_button_connect
-    (vwd, 2, GTK_SIGNAL_FUNC(gnc_vendor_window_help_cb), vw);
-
-  gtk_signal_connect (GTK_OBJECT (vw->dialog), "destroy",
-		      GTK_SIGNAL_FUNC(gnc_vendor_window_destroy_cb), vw);
-
-  gtk_signal_connect(GTK_OBJECT (vw->id_entry), "changed",
-		     GTK_SIGNAL_FUNC(gnc_vendor_name_changed_cb), vw);
-
-  gtk_signal_connect(GTK_OBJECT (vw->company_entry), "changed",
-		     GTK_SIGNAL_FUNC(gnc_vendor_name_changed_cb), vw);
-
-  gtk_signal_connect(GTK_OBJECT (vw->taxtable_check), "toggled",
-		     GTK_SIGNAL_FUNC(gnc_vendor_taxtable_check_cb), vw);
+  glade_xml_signal_autoconnect_full( xml,
+                                     gnc_glade_autoconnect_full_func,
+                                     vw);
 
   /* Setup initial values */
   if (vendor != NULL) {
+    GtkTextBuffer* text_buffer;
     GncAddress *addr;
     const char *string;
-    gint pos = 0;
 
     vw->dialog_type = EDIT_VENDOR;
     vw->vendor_guid = *gncVendorGetGUID (vendor);
@@ -453,9 +447,8 @@ gnc_vendor_new_window (GNCBook *bookp, GncVendor *vendor)
                                 gncVendorGetActive (vendor));
 
     string = gncVendorGetNotes (vendor);
-    gtk_editable_delete_text (GTK_EDITABLE (vw->notes_text), 0, -1);
-    gtk_editable_insert_text (GTK_EDITABLE (vw->notes_text), string,
-			      strlen(string), &pos);
+    text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(vw->notes_text));
+    gtk_text_buffer_set_text (text_buffer, string, -1);
 
     vw->component_id =
       gnc_register_gui_component (DIALOG_EDIT_VENDOR_CM_CLASS,
@@ -494,7 +487,7 @@ gnc_vendor_new_window (GNCBook *bookp, GncVendor *vendor)
 
   gnc_gui_component_watch_entity_type (vw->component_id,
 				       GNC_VENDOR_MODULE_NAME,
-				       GNC_EVENT_MODIFY | GNC_EVENT_DESTROY);
+				       QOF_EVENT_MODIFY | QOF_EVENT_DESTROY);
 
   gtk_widget_show_all (vw->dialog);
 
@@ -698,9 +691,10 @@ gnc_vendor_search (GncVendor *start, GNCBook *book)
   sw->book = book;
   sw->q = q;
 
-  return gnc_search_dialog_create (type, params, columns, q, q2,
-				   buttons, NULL,
-				   new_vendor_cb, sw, free_vendor_cb);
+  return gnc_search_dialog_create (type, _("Find Vendor"),
+				   params, columns, q, q2, buttons, NULL,
+				   new_vendor_cb, sw, free_vendor_cb,
+				   GCONF_SECTION_SEARCH, NULL);
 }
 
 GNCSearchWindow *
