@@ -2,22 +2,38 @@
  * gncEntryLedger.c -- a Ledger widget for entering GncEntry objects
  * Copyright (C) 2001, 2002, 2003 Derek Atkins
  * Author: Derek Atkins <warlord@MIT.EDU>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, contact:
+ *
+ * Free Software Foundation           Voice:  +1-617-542-5942
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org
  */
 
-#define _GNU_SOURCE
-
 #include "config.h"
 
-#include <glib.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 
 #include "Account.h"
-#include "AccWindow.h"
+#include "dialog-account.h"
+#include "dialog-utils.h"
 #include "gnc-ui-util.h"
 #include "combocell.h"
 #include "pricecell.h"
 #include "recncell.h"
 #include "checkboxcell.h"
-#include "messages.h"
 
 #include "gnc-component-manager.h"
 #include "gnc-ui.h"
@@ -62,20 +78,19 @@ Account *
 gnc_entry_ledger_get_account_by_name (GncEntryLedger *ledger, BasicCell * bcell,
 				      const char *name, gboolean *new)
 {
-  const char *placeholder = _("The account %s does not allow transactions.\n");
-  const char *missing = _("The account %s does not exist.\n"
+  const char *placeholder = _("The account %s does not allow transactions.");
+  const char *missing = _("The account %s does not exist. "
 			  "Would you like to create it?");
   char *fullname;
   ComboCell *cell = (ComboCell *) bcell;
   Account *account;
 
   /* Find the account */
-  account = xaccGetAccountFromFullName (gnc_get_current_group (),
-					name, gnc_get_account_separator ());
+  account = xaccGetAccountFromFullName (gnc_get_current_group (), name);
 
   if (!account) {
     /* Ask if they want to create a new one. */
-    if (!gnc_verify_dialog_parented (ledger->parent, TRUE, missing, name))
+    if (!gnc_verify_dialog (ledger->parent, TRUE, missing, name))
       return NULL;
     
     /* No changes, as yet. */
@@ -88,7 +103,7 @@ gnc_entry_ledger_get_account_by_name (GncEntryLedger *ledger, BasicCell * bcell,
     *new = TRUE;
 
     /* Now have a new account. Update the cell with the name as created. */
-    fullname = xaccAccountGetFullName (account, gnc_get_account_separator ());
+    fullname = xaccAccountGetFullName (account);
     gnc_combo_cell_set_value (cell, fullname);
     gnc_basic_cell_set_changed (&cell->cell, TRUE);
     g_free (fullname);
@@ -96,7 +111,7 @@ gnc_entry_ledger_get_account_by_name (GncEntryLedger *ledger, BasicCell * bcell,
 
   /* See if the account (either old or new) is a placeholder. */
   if (xaccAccountGetPlaceholder (account)) {
-    gnc_error_dialog_parented (GTK_WINDOW(ledger->parent), placeholder, name);
+    gnc_error_dialog (ledger->parent, placeholder, name);
   }
 
   /* Be seeing you. */
@@ -260,6 +275,7 @@ GncEntryLedger * gnc_entry_ledger_new (GNCBook *book, GncEntryLedgerType type)
   ledger->type = type;
   ledger->book = book;
   ledger->traverse_to_new = TRUE;
+  ledger->gconf_section = NULL;
 
   /* Orders and Invoices are "invoices" for lookups */
   switch (type) {
@@ -611,6 +627,7 @@ gnc_entry_ledger_compute_value (GncEntryLedger *ledger,
   gboolean taxable, taxincluded;
   GncTaxTable *table;
   GList *taxes = NULL;
+  int denom = 100;
 
   gnc_entry_ledger_get_numeric (ledger, ENTRY_QTY_CELL, &qty);
   gnc_entry_ledger_get_numeric (ledger, ENTRY_PRIC_CELL, &price);
@@ -654,18 +671,20 @@ gnc_entry_ledger_compute_value (GncEntryLedger *ledger,
     table = NULL;
   }
 
+  if (ledger->invoice)
+  {
+    gnc_commodity *currency = gncInvoiceGetCurrency(ledger->invoice);
+    if (currency)
+      denom = gnc_commodity_get_fraction(currency);
+  }
+
   gncEntryComputeValue (qty, price, (taxable ? table : NULL), taxincluded,
-			discount, disc_type, disc_how,
+			discount, disc_type, disc_how, denom,
 			value, NULL, &taxes);
 
-  /* Now convert the values to the proper denomination */
-  if (value)
-    *value = gnc_numeric_convert (*value, 100 /* XXX */, GNC_RND_ROUND);
-
-  if (tax_value) {
+  /* return the tax value */
+  if (tax_value)
     *tax_value = gncAccountValueTotal (taxes);
-    *tax_value = gnc_numeric_convert (*tax_value, 100 /* XXX */, GNC_RND_ROUND);
-  }
 }
 
 gboolean
@@ -783,14 +802,29 @@ gnc_entry_ledger_duplicate_current_entry (GncEntryLedger *ledger)
   /* If the cursor has been edited, we are going to have to commit
    * it before we can duplicate. Make sure the user wants to do that. */
   if (changed) {
-    const char *message = _("The current entry has been changed.\n"
-			  "Would you like to save it?");
-    GNCVerifyResult result;
+    const char *title = _("Save the current entry?");
+    const char *message =
+      _("The current transaction has been changed. Would you like to "
+	"record the changes before duplicating this entry, or "
+	"cancel the duplication?");
+    GtkWidget *dialog;
+    gint response;
 
-    result = gnc_ok_cancel_dialog_parented (ledger->parent,      
-					    GNC_VERIFY_OK, message);
+    dialog = gtk_message_dialog_new(GTK_WINDOW(ledger->parent),
+				    GTK_DIALOG_DESTROY_WITH_PARENT,
+				    GTK_MESSAGE_QUESTION,
+				    GTK_BUTTONS_NONE,
+				    "%s", title);
+    gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
+					     "%s", message);
+    gtk_dialog_add_buttons(GTK_DIALOG(dialog),
+			   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			   _("_Record"), GTK_RESPONSE_ACCEPT,
+			   NULL);
+    response = gnc_dialog_run(GTK_DIALOG(dialog), "invoice_entry_duplicated");
+    gtk_widget_destroy(dialog);
 
-    if (result == GNC_VERIFY_CANCEL) {
+    if (response != GTK_RESPONSE_ACCEPT) {
       gnc_resume_gui_refresh ();
       return;
     }
@@ -825,3 +859,12 @@ gnc_entry_ledger_get_query (GncEntryLedger *ledger)
 
   return ledger->query;
 }
+
+void
+gnc_entry_ledger_set_gconf_section (GncEntryLedger *ledger, const gchar *string)
+{
+  if (!ledger)
+    return;
+
+  ledger->gconf_section = string;
+}
