@@ -2,18 +2,33 @@
  * gncEntryLedgerModel.c -- Model for GncEntry ledger
  * Copyright (C) 2001, 2002, 2003 Derek Atkins
  * Author: Derek Atkins <warlord@MIT.EDU>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, contact:
+ *
+ * Free Software Foundation           Voice:  +1-617-542-5942
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org
  */
 
-#define _GNU_SOURCE
-
 #include "config.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 
-#include "messages.h"
 #include "Account.h"
 #include "gnc-ui-util.h"
-#include "gnc-engine-util.h"	/* for safe_strcmp */
+#include "qof.h"	/* for safe_strcmp */
 
 #include "datecell.h"
 #include "checkboxcell.h"
@@ -23,12 +38,12 @@
 
 static GncEntryLedgerColors reg_colors =
 {
-  0xffffff, /* white */
-  0xffffff,
-  0xffffff,
+  0x96B183,
+  0xBFDEB9,
+  0xF6FFDA,
 
-  0xffffff,
-  0xffffff
+  0xFFEF98,
+  0xFFEF98,
 };
 
 /** Private Interfaces ***********************************************/
@@ -140,8 +155,7 @@ static const char * get_iacct_entry (VirtualLocation virt_loc,
   entry = gnc_entry_ledger_get_entry (ledger, virt_loc.vcell_loc);
 
   g_free (name);
-  name = xaccAccountGetFullName (gncEntryGetInvAccount (entry),
-				 gnc_get_account_separator ());
+  name = xaccAccountGetFullName (gncEntryGetInvAccount (entry));
   return name;
 }
 
@@ -158,8 +172,7 @@ static const char * get_bacct_entry (VirtualLocation virt_loc,
   entry = gnc_entry_ledger_get_entry (ledger, virt_loc.vcell_loc);
 
   g_free (name);
-  name = xaccAccountGetFullName (gncEntryGetBillAccount (entry),
-				 gnc_get_account_separator ());
+  name = xaccAccountGetFullName (gncEntryGetBillAccount (entry));
   return name;
 }
 
@@ -1180,8 +1193,3 @@ TableModel * gnc_entry_ledger_model_new (GncEntryLedgerType type)
 
   return model;
 }
-
-void gnc_entry_ledger_set_colors (GncEntryLedgerColors reg_colors_new)
-{
-  reg_colors = reg_colors_new;
-}
