@@ -3,6 +3,7 @@
 ;;  
 ;; By Derek Atkins <warlord@MIT.EDU> taken from the original...
 ;; By Robert Merkel (rgmerk@mira.net) 
+;; Copyright (c) 2002, 2003 Derek Atkins <warlord@MIT.EDU>
 ;;
 ;; This program is free software; you can redistribute it and/or    
 ;; modify it under the terms of the GNU General Public License as   
@@ -18,8 +19,8 @@
 ;; along with this program; if not, contact:
 ;;
 ;; Free Software Foundation           Voice:  +1-617-542-5942
-;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
-;; Boston, MA  02111-1307,  USA       gnu@gnu.org
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -44,6 +45,7 @@
 (define optname-report-currency (N_ "Report's currency"))
 (define optname-price-source (N_ "Price Source"))
 (define optname-multicurrency-totals (N_ "Show Multi-currency Totals?"))
+(define optname-show-zeros (N_ "Show zero balance items?"))
 
 ;; The idea is:  have a hash with the key being the contact name
 ;; (In future this might be GUID'ed, but for now it's a string
@@ -174,7 +176,7 @@
 ;; a new company record in the hash
 
 (define (update-company-hash hash split bucket-intervals
-			     reverse?)
+			     reverse? show-zeros)
 
   (define (do-update value)
     (let* ((transaction (gnc:split-get-parent split))
@@ -201,8 +203,8 @@
 			 this-currency
 			 (company-get-currency company-info)))
 		   (cons #f (sprintf
-			     (_ "Transactions relating to company  %d contain \
-more than one currency.  This report is not designed to cope with this possibility.")))
+			     (_ "Transactions relating to '%s' contain \
+more than one currency.  This report is not designed to cope with this possibility.")  (gnc:owner-get-name owner)))
 		   (begin
 		     (gnc:debug "it's an old company")
 		     (if (gnc:numeric-negative-p value)
@@ -233,7 +235,7 @@ more than one currency.  This report is not designed to cope with this possibili
     ;; XXX: we _could_ just set the value to 0 in order to list
     ;;      the company.  I'm not sure what to do.  Perhaps add an
     ;;      option?
-    (if (not is-paid?)
+    (if (or (not is-paid?) show-zeros)
 	(do-update value))))
 
 ;; get the total debt from the buckets
@@ -362,6 +364,14 @@ more than one currency.  This report is not designed to cope with this possibili
 totals to report currency")
       #f))
 
+    (add-option
+     (gnc:make-simple-boolean-option
+      gnc:pagename-general
+      optname-show-zeros
+      "j"
+      (N_ "Show all vendors/customers even if they have a zero balance.")
+      #f))
+
     (gnc:options-set-default-section options "General")      
     options))
 
@@ -518,6 +528,7 @@ totals to report currency")
 	(report-currency (op-value gnc:pagename-general optname-report-currency))
 	(price-source (op-value gnc:pagename-general optname-price-source))
 	(multi-totals-p (op-value gnc:pagename-general optname-multicurrency-totals))
+	(show-zeros (op-value gnc:pagename-general optname-show-zeros))
 	(heading-list (make-heading-list))
 	(exchange-fn (gnc:case-exchange-fn price-source report-currency report-date))
 	(total-collector-list (make-collector-list))
@@ -529,16 +540,22 @@ totals to report currency")
         (document (gnc:make-html-document)))
 ;    (gnc:debug "Account: " account)
 
-    (if account
-	(set! report-title (gnc:html-markup
-			    "!" 
-			    report-title
-			    ": "
-			    (gnc:html-markup-anchor
-			     (gnc:account-anchor-text account)
-			     (gnc:account-get-name account)))))
-
+    ;; set default title
     (gnc:html-document-set-title! document report-title)
+    ;; maybe redefine better...
+    (if account
+        (begin
+          (gnc:html-document-set-title!
+           document (string-append report-title ": " (gnc:account-get-name account)))
+          (gnc:html-document-set-headline! document
+                                           (gnc:html-markup
+                                            "!" 
+                                            report-title
+                                            ": "
+                                            (gnc:html-markup-anchor
+                                             (gnc:account-anchor-text account)
+                                             (gnc:account-get-name account))))))
+
     (gnc:html-table-set-col-headers! table heading-list)
 				     
     (if account
@@ -557,7 +574,7 @@ totals to report currency")
 			(update-company-hash companys 
 					      split 
 					      interval-vec 
-					      reverse?))
+					      reverse? show-zeros))
 			splits)
 ;	    (gnc:debug "companys" companys)
 	    ;; turn the hash into a list
