@@ -4,7 +4,25 @@
 ;;		       company (the owner) applied to an account.
 ;;
 ;; Created by:  Derek Atkins <warlord@MIT.EDU>
+;; Copyright (c) 2002, 2003 Derek Atkins <warlord@MIT.EDU>
 ;;
+;; This program is free software; you can redistribute it and/or    
+;; modify it under the terms of the GNU General Public License as   
+;; published by the Free Software Foundation; either version 2 of   
+;; the License, or (at your option) any later version.              
+;;                                                                  
+;; This program is distributed in the hope that it will be useful,  
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
+;; GNU General Public License for more details.                     
+;;                                                                  
+;; You should have received a copy of the GNU General Public License
+;; along with this program; if not, contact:
+;;
+;; Free Software Foundation           Voice:  +1-617-542-5942
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
+
 
 (define-module (gnucash report owner-report))
 
@@ -444,7 +462,7 @@
     (gnc:html-table-append-row!
      table
      (list
-      (string-expand (gnc:owner-get-address-dep owner) #\newline "<br>")))
+      (string-expand (gnc:owner-get-name-and-address-dep owner) #\newline "<br>")))
     (gnc:html-table-append-row!
      table
      (list "<br>"))
@@ -522,7 +540,6 @@
 	 (end-date (gnc:timepair-end-day-time 
 		       (gnc:date-option-absolute-time
 			(opt-val gnc:pagename-general (N_ "To")))))
-	 (title #f)
 	 (book (gnc:get-current-book)) ;XXX Grab this from elsewhere
 	 (owner-type (opt-val "__reg" "owner-type"))
 	 (type-str ""))
@@ -535,20 +552,26 @@
       ((gnc-owner-employee)
        (set! type-str (N_ "Employee"))))
 
+    (gnc:html-document-set-title!
+     document (string-append (_ type-str) " " (_ "Report")))
+
     (if (gnc:owner-is-valid? owner)
 	(begin
 	  (setup-query query owner account end-date)
 
-	  (set! title (gnc:html-markup
-		       "!" 
-		       (_ type-str )
-		       (_ " Report: ")
-		       (gnc:html-markup-anchor
-			(gnc:owner-anchor-text owner)
-			(gnc:owner-get-name owner))))
-	  
-	  (gnc:html-document-set-title! document title)
+	  (gnc:html-document-set-title!
+	   document
+           (string-append (_ type-str ) " " (_ "Report:") " " (gnc:owner-get-name owner)))
 
+           (gnc:html-document-set-headline!
+            document (gnc:html-markup
+                      "!" 
+                      (_ type-str )
+                      " " (_ "Report:") " "
+                      (gnc:html-markup-anchor
+                       (gnc:owner-anchor-text owner)
+                       (gnc:owner-get-name owner))))
+	  
 	  (if account
 	      (begin
 		(set! table (make-txn-table (gnc:report-options report-obj)
@@ -562,9 +585,7 @@
 	      (set!
 	       table
 	       (gnc:make-html-text
-		(string-append 
-		 "No Valid Account Selected.  "
-		 "Click on the Options button and select the account to use."))))
+		(_ "No valid account selected.  Click on the Options button and select the account to use."))))
 
 	  (gnc:html-document-add-object!
 	   document
@@ -594,9 +615,9 @@
 	(gnc:html-document-add-object!
 	 document
 	 (gnc:make-html-text
-	  (string-append
-	   "No Valid " type-str	" Selected.  "
-	   "Click on the Options button to select a company."))))
+	  (sprintf #f 
+		   (_ "No valid %s selected.  Click on the Options button to select a company.")
+		   type-str))))
 
     (gnc:free-query query)
     document))
