@@ -17,16 +17,13 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
 \********************************************************************/
 
-/*
- * FILE:
- * AccountP.h
+/** @file AccountP.h
  *
- * FUNCTION:
  * This is the *private* header for the account structure.
  * No one outside of the engine should ever include this file.
  *
@@ -42,23 +39,23 @@
 #ifndef XACC_ACCOUNT_P_H
 #define XACC_ACCOUNT_P_H
 
-#include "config.h"
-
 #include "Account.h"
-#include "GNCIdP.h"
 #include "gnc-commodity.h"
 #include "gnc-engine.h"
-#include "gnc-numeric.h"
-#include "kvp_frame.h"
-
+#include "policy.h"
 
 /** STRUCTS *********************************************************/
+
+/** This is the data that describes an account. 
+ *
+ * This is the *private* header for the account structure.
+ * No one outside of the engine should ever include this file.
+*/
+
+/** \struct Account */
 struct account_s
 {
-  /* public data, describes account */
-  GUID      guid;          /* globally unique account id */
-
-  GNCBook *book;   /* the entity_table in which this account is stored */
+  QofInstance inst;
 
   /* The accountName is an arbitrary string assigned by the user. 
    * It is intended to a short, 5 to 30 character long string that
@@ -70,7 +67,7 @@ struct account_s
    * It is intended to be reporting code that is a synonym for the 
    * accountName. Typically, it will be a numeric value that follows 
    * the numbering assignments commonly used by accountants, such 
-   * as 100, 200 or 600 for top-level * accounts, and 101, 102..  etc.
+   * as 100, 200 or 600 for top-level accounts, and 101, 102..  etc.
    * for detail accounts.
    */
   char     *accountCode;
@@ -81,12 +78,6 @@ struct account_s
    */
   char     *description;
 
-  /* kvp_data is a key-value pair database for storing simple "extra"
-   * information in splits, transactions, and accounts.  it's NULL
-   * until accessed.  See src/engine/kvp_doc.txt for a list and 
-   * description of the important keys. */
-  kvp_frame * kvp_data;
-
   /* The type field is the account type, picked from the enumerated 
    * list that includes BANK, STOCK, CREDIT, INCOME, etc.  Its
    * intended use is to be a hint to the GUI as to how to display   
@@ -125,13 +116,11 @@ struct account_s
   SplitList *splits;       /* list of split pointers */
   LotList   *lots;         /* list of lot pointers */
 
-  /* keep track of nesting level of begin/end edit calls */
-  gint32 editlevel;
+  /* Cached pointer to policy method */
+  GNCPolicy *policy;
 
   gboolean balance_dirty;  /* balances in splits incorrect */
   gboolean sort_dirty;     /* sort order of splits is bad */
-  gboolean core_dirty;     /* fields in this struct have changed */
-  gboolean do_free;        /* in process of being destroyed */
 
   /* The "mark" flag can be used by the user to mark this account
    * in any way desired.  Handy for specialty traversals of the 
@@ -143,15 +132,6 @@ struct account_s
   guint32  idata;     /* used by the sql backend for kvp management */
 };
 
-/* The xaccAccountRemoveSplit() routine will remove the indicated split
- *    from the indicated account.  Note that this will leave the split
- *    "dangling", i.e. unassigned to any account, and therefore will put
- *    the engine into an inconsistent state.  After removing a split, 
- *    it should be immediately destroyed, or it should be inserted into  
- *    an account.
- */
-void         xaccAccountRemoveSplit (Account *, Split *);
-
 /* The xaccAccountSortSplits() routine will resort the account's 
  * splits if the sort is dirty. If 'force' is true, the account 
  * is sorted even if the editlevel is not zero. 
@@ -161,12 +141,12 @@ void xaccAccountSortSplits (Account *acc, gboolean force);
 /* The following recompute the partial balances (stored with the
  * transaction) and the total balance, for this account 
  */
-void         xaccAccountRecomputeBalance (Account *);
+void xaccAccountRecomputeBalance (Account *);
 
 /* Set the account's GUID. This should only be done when reading
  * an account from a datafile, or some other external source. Never
  * call this on an existing account! */
-void         xaccAccountSetGUID (Account *account, const GUID *guid);
+void xaccAccountSetGUID (Account *account, const GUID *guid);
 
 /* The xaccAccountSetStartingBalance() routine will set the starting
  *    commodity balance for this account.  This routine is intended for
@@ -199,9 +179,14 @@ void xaccFreeAccount (Account *account);
  *    want anyone except the backend to mess with them.
  */
 void xaccAccountSetVersion (Account*, gint32);
-gint32 xaccAccountGetVersion (Account*);
+gint32 xaccAccountGetVersion (const Account* acc);
 
 /* Register Accounts with the engine */
 gboolean xaccAccountRegister (void);
 
+/** killed for now, need to resurect this or something similar
+ *  * for transactional/dirty kvp.  Later.  Right now a place holder
+ *   */
+#define xaccAccountSetSlots_nc(A,S) qof_instance_set_slots(QOF_INSTANCE(A),S)
+
 #endif /* XACC_ACCOUNT_P_H */
