@@ -1,7 +1,7 @@
 /********************************************************************\
  * Group.c -- chart of accounts (hierarchical tree of accounts)     *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997-2001 Linas Vepstas <linas@linas.org>          *
+ * Copyright (C) 1997-2001,2003 Linas Vepstas <linas@linas.org>     *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -17,8 +17,8 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
 \********************************************************************/
 
@@ -29,17 +29,12 @@
 
 #include "Account.h"
 #include "AccountP.h"
-#include "BackendP.h"
-#include "GNCIdP.h"
 #include "Group.h"
 #include "GroupP.h"
 #include "TransactionP.h"
-#include "gnc-book-p.h"
-#include "gnc-engine-util.h"
-#include "gnc-event-p.h"
-#include "gnc-numeric.h"
+#include "gnc-event.h"
 
-static short module = MOD_ENGINE;
+static QofLogModule log_module = GNC_MOD_ENGINE;
 
 /********************************************************************\
  * Because I can't use C++ for this project, doesn't mean that I    *
@@ -52,7 +47,7 @@ static short module = MOD_ENGINE;
 \********************************************************************/
 
 static void
-xaccInitializeAccountGroup (AccountGroup *grp, GNCBook *book)
+xaccInitializeAccountGroup (AccountGroup *grp, QofBook *book)
 {
   grp->saved       = 1;
 
@@ -67,7 +62,7 @@ xaccInitializeAccountGroup (AccountGroup *grp, GNCBook *book)
 \********************************************************************/
 
 AccountGroup *
-xaccMallocAccountGroup (GNCBook *book)
+xaccMallocAccountGroup (QofBook *book)
 {
   AccountGroup *grp;
   g_return_val_if_fail (book, NULL);
@@ -81,9 +76,59 @@ xaccMallocAccountGroup (GNCBook *book)
 /********************************************************************\
 \********************************************************************/
 
+AccountGroup * 
+xaccCollGetAccountGroup (QofCollection *col)
+{
+  if (!col) return NULL;
+  return qof_collection_get_data (col);
+}
+
+void
+xaccCollSetAccountGroup (QofCollection *col, AccountGroup *grp)
+{
+  AccountGroup *old_grp;
+  if (!col) return;
+
+  old_grp = xaccCollGetAccountGroup (col);
+  if (old_grp == grp) return;
+
+  qof_collection_set_data (col, grp);
+
+  xaccAccountGroupBeginEdit (old_grp);
+  xaccAccountGroupDestroy (old_grp);
+}
+
+AccountGroup * 
+xaccGetAccountGroup (QofBook *book)
+{
+  QofCollection *col;
+  if (!book) return NULL;
+  col = qof_book_get_collection (book, GNC_ID_GROUP);
+  return xaccCollGetAccountGroup (col);
+}
+
+void
+xaccSetAccountGroup (QofBook *book, AccountGroup *grp)
+{
+  QofCollection *col;
+  if (!book) return;
+
+  if (grp && grp->book != book)
+  {
+     PERR ("cannot mix and match books freely!");
+     return;
+  }
+
+  col = qof_book_get_collection (book, GNC_ID_GROUP);
+  xaccCollSetAccountGroup (col, grp);
+}
+
+/********************************************************************\
+\********************************************************************/
+
 gboolean
-xaccGroupEqual(AccountGroup *ga,
-               AccountGroup *gb,
+xaccGroupEqual(const AccountGroup *ga,
+               const AccountGroup *gb,
                gboolean check_guids)
 {
   GList *na;
@@ -190,7 +235,7 @@ xaccGroupMarkDoFree (AccountGroup *grp)
   for (node = grp->accounts; node; node = node->next)
   {
     Account *account = node->data;
-    account->do_free = TRUE;
+    account->inst.do_free = TRUE;
     xaccGroupMarkDoFree (account->children); 
   }
 }
@@ -205,8 +250,8 @@ xaccAccountGroupDestroy (AccountGroup *grp)
 /********************************************************************\
 \********************************************************************/
 
-GNCBook *
-xaccGroupGetBook (AccountGroup *group)
+QofBook *
+xaccGroupGetBook (const AccountGroup *group)
 {
   if (!group) return NULL;
   return group->book;
@@ -242,13 +287,13 @@ xaccFreeAccountGroup (AccountGroup *grp)
       /* FIXME: this and the same code below is kind of hacky.
        *        actually, all this code seems to assume that
        *        the account edit levels are all 1. */
-      if (account->editlevel == 0)
+      if (account->inst.editlevel == 0)
         xaccAccountBeginEdit (account);
 
       xaccAccountDestroy (account);
     }
     account = grp->accounts->data;
-    if (account->editlevel == 0)
+    if (account->inst.editlevel == 0) 
       xaccAccountBeginEdit (account);
     xaccAccountDestroy (account);
 
@@ -297,7 +342,7 @@ xaccGroupMarkNotSaved (AccountGroup *grp)
 \********************************************************************/
 
 gboolean
-xaccGroupNotSaved (AccountGroup *grp)
+xaccGroupNotSaved (const AccountGroup *grp)
 {
   GList *node;
 
@@ -321,7 +366,7 @@ xaccGroupNotSaved (AccountGroup *grp)
 \********************************************************************/
 
 int
-xaccGroupGetNumSubAccounts (AccountGroup *grp)
+xaccGroupGetNumSubAccounts (const AccountGroup *grp)
 {
   GList *node;
   int num_acc;
@@ -341,11 +386,11 @@ xaccGroupGetNumSubAccounts (AccountGroup *grp)
 }
 
 /********************************************************************\
- * Get all of the accounts, including subaccounts                   *
+ * Recursively get all of the accounts, including subaccounts       *
 \********************************************************************/
 
 static void
-xaccPrependAccounts (AccountGroup *grp, GList **accounts_p)
+xaccPrependAccounts (const AccountGroup *grp, GList **accounts_p)
 {
   GList *node;
 
@@ -362,7 +407,7 @@ xaccPrependAccounts (AccountGroup *grp, GList **accounts_p)
 }
 
 AccountList *
-xaccGroupGetSubAccounts (AccountGroup *grp)
+xaccGroupGetSubAccounts (const AccountGroup *grp)
 {
   GList *accounts = NULL;
 
@@ -373,22 +418,75 @@ xaccGroupGetSubAccounts (AccountGroup *grp)
   return g_list_reverse (accounts);
 }
 
+static int
+group_sort_helper (gconstpointer a, gconstpointer b)
+{
+  const Account *aa = (const Account *) a;
+  const Account *bb = (const Account *) b;
+
+  /* xaccAccountOrder returns > 1 if aa should come after bb.
+   * This funciton is building a reversed list. */
+  return xaccAccountOrder (&aa, &bb);
+}
+
+static void
+xaccPrependAccountsSorted (const AccountGroup *grp, GList **accounts_p)
+{
+  GList *node, *tmp_list;
+
+  if (!grp || !accounts_p) return;
+
+  tmp_list = g_list_copy(grp->accounts);
+  tmp_list = g_list_sort(tmp_list, group_sort_helper);
+
+  for (node = tmp_list; node; node = node->next)
+  {
+    Account *account = node->data;
+
+    *accounts_p = g_list_prepend (*accounts_p, account);
+
+    xaccPrependAccountsSorted (account->children, accounts_p);
+  }
+  g_list_free(tmp_list);
+}
+
 AccountList *
-xaccGroupGetAccountList (AccountGroup *grp)
+xaccGroupGetSubAccountsSorted (const AccountGroup *grp)
+{
+  GList *accounts = NULL;
+
+  if (!grp) return NULL;
+
+  xaccPrependAccountsSorted (grp, &accounts);
+
+  return g_list_reverse (accounts);
+}
+
+AccountList *
+xaccGroupGetAccountList (const AccountGroup *grp)
 {
   if (!grp) return NULL;
 
   return grp->accounts;
 }
 
+GList *
+xaccGroupGetAccountListSorted (const AccountGroup *grp)
+{
+  if (!grp) return NULL;
+
+  return g_list_sort(g_list_copy(grp->accounts), group_sort_helper);
+
+}
+
 /********************************************************************\
  * Fetch the root of the tree                                       *
 \********************************************************************/
 
 AccountGroup *
-xaccGroupGetRoot (AccountGroup * grp) 
+xaccGroupGetRoot (const AccountGroup * grp) 
 {
-  AccountGroup * root = NULL;
+  const AccountGroup * root = NULL;
 
   /* find the root of the account group structure */
   while (grp)
@@ -404,14 +502,13 @@ xaccGroupGetRoot (AccountGroup * grp)
       grp = NULL;
   }
 
-  return root;
+  return (AccountGroup*)root;
 }
 
 AccountGroup *
-xaccAccountGetRoot (Account * acc) 
+xaccAccountGetRoot (const Account * acc) 
 {
-  if (!acc) return NULL;
-  return xaccGroupGetRoot (acc->parent);
+  return acc ? xaccGroupGetRoot (acc->parent) : NULL;
 }
 
 /********************************************************************\
@@ -419,7 +516,7 @@ xaccAccountGetRoot (Account * acc)
 \********************************************************************/
 
 Account *
-xaccGetAccountFromName (AccountGroup *grp, const char * name)
+xaccGetAccountFromName (const AccountGroup *grp, const char * name)
 {
   GList *node;
 
@@ -455,78 +552,100 @@ xaccGetAccountFromName (AccountGroup *grp, const char * name)
  * Fetch an account, given its full name                            *
 \********************************************************************/
 
-Account *
-xaccGetAccountFromFullName (AccountGroup *grp,
-                            const char *name,
-                            const char separator)
+static Account *
+xaccGetAccountFromFullNameHelper (const AccountGroup *grp,
+				  gchar **names)
 {
+  Account *found = NULL;
   GList *node;
-  Account *found;
-  char *p;
+  gchar *name_str, *temp_str;
+  const gchar *separator;
+  gboolean str_alloced = FALSE;
 
-  if (!grp) return NULL;
-  if (!name) return NULL;
+  g_return_val_if_fail(grp, NULL);
+  g_return_val_if_fail(names, NULL);
 
-  p = (char *) name;
-  found = NULL;
+  separator = gnc_get_account_separator_string();
+  /* start with the first name in the list */
+  name_str = names[0];
+
+  /* Make sure we actually HAVE a string! */
+  if (name_str == NULL)
+    return NULL;
 
   while (1)
   {
-    /* Look for the first separator. */
-    p = strchr(p, separator);
-
-    /* If found, switch it to the null char. */
-    if (p != NULL)
-      *p = 0;
-
-    /* Now look for that name in the children. */
-    for (node = grp->accounts; node; node = node->next)
-    {
+    /* Look for the first name in the children. */
+    for (node = grp->accounts; node; node = node->next) {
       Account *account = node->data;
 
-      if (safe_strcmp(xaccAccountGetName (account), name) == 0)
-      {
-        /* We found an account.
-         * If p == NULL, there is nothing left
-         * in the name, so just return the account.
-         * We don't need to put back the separator,
-         * because it was never erased (p == NULL). */
-        if (p == NULL)
-          return account;
+      if (safe_strcmp(xaccAccountGetName (account), name_str) == 0) {
+	/* We found an account.  If the next entry is NULL, there is
+	 * nothing left in the name, so just return the account. */
+	if (names[1] == NULL) {
+	  found = account;
+	  goto done;
+	}
 
-        /* There's stuff left to search for.
-         * Search recursively after the separator. */
-        found = xaccGetAccountFromFullName(account->children,
-                                           p + 1, separator);
+	/* No children?  We're done. */
+	if (!account->children) {
+	  found = NULL;
+	  goto done;
+	}
 
-        /* If we found the account, break out. */
-        if (found != NULL)
-          break;
+	/* There's stuff left to search for.  Search recursively. */
+	found = xaccGetAccountFromFullNameHelper(account->children, &names[1]);
+	if (found != NULL) {
+	  goto done;
+	}
       }
     }
 
-    /* If found != NULL, an account was found. */
-    /* If p == NULL, there are no more separators left. */
+    /* If we got here then we didn't find a match based on name_str
+     * so build a new name_str using the next token and try again.
+     */
 
-    /* Put back the separator. */
-    if (p != NULL)
-      *p = separator;
-
-    /* If we found the account, return it. */
-    if (found != NULL)
-      return found;
-
-    /* We didn't find the account. If there
-     * are no more separators, return NULL. */
-    if (p == NULL)
-      return NULL;
+    /* If there's no more names then we're done.  We didn't find anything */
+    if (names[1] == NULL) {
+      found = NULL;
+      break;
+    }
 
     /* If we are here, we didn't find anything and there
      * must be more separators. So, continue looking with
      * a longer name, in case there is a name with the
-     * separator character in it. */
-    p++;
+     * separator character in it. */ 
+
+    /* Build the new name string */
+    temp_str = g_strconcat(name_str, separator, names[1], NULL);
+    if (str_alloced)
+      g_free(name_str);
+    str_alloced = TRUE;
+    names++;
+    name_str = temp_str;    
   }
+
+done:
+  if (str_alloced)
+    g_free(name_str);
+  return found;
+}
+
+
+Account *
+xaccGetAccountFromFullName (const AccountGroup *grp,
+			    const char *name)
+{
+  Account *found;
+  gchar **names;
+
+  if (!grp) return NULL;
+  if (!name) return NULL;
+
+  names = g_strsplit(name, gnc_get_account_separator_string(), -1);
+  found = xaccGetAccountFromFullNameHelper(grp, names);
+  g_strfreev(names);
+  return found;
 }
 
 /********************************************************************\
@@ -534,7 +653,7 @@ xaccGetAccountFromFullName (AccountGroup *grp,
 \********************************************************************/
 
 Account *
-xaccGetPeerAccountFromName (Account *acc, const char * name)
+xaccGetPeerAccountFromName (const Account *acc, const char * name)
 {
   AccountGroup * root;
   Account *peer_acc;
@@ -556,8 +675,7 @@ xaccGetPeerAccountFromName (Account *acc, const char * name)
 \********************************************************************/
 
 Account *
-xaccGetPeerAccountFromFullName (Account *acc, const char * name,
-                                const char separator)
+xaccGetPeerAccountFromFullName (const Account *acc, const char * name)
 {
   AccountGroup * root;
   Account *peer_acc;
@@ -569,7 +687,7 @@ xaccGetPeerAccountFromFullName (Account *acc, const char * name,
   root = xaccAccountGetRoot (acc);
 
   /* now search all acounts hanging off the root */
-  peer_acc = xaccGetAccountFromFullName (root, name, separator);
+  peer_acc = xaccGetAccountFromFullName (root, name);
 
   return peer_acc;
 }
@@ -597,7 +715,7 @@ xaccAccountRemoveGroup (Account *acc)
 
   grp->saved = 0;
 
-  gnc_engine_generate_event (&acc->guid, GNC_EVENT_MODIFY);
+  qof_event_gen (&acc->inst.entity, QOF_EVENT_MODIFY, NULL);
 }
 
 /********************************************************************\
@@ -606,6 +724,8 @@ xaccAccountRemoveGroup (Account *acc)
 void
 xaccGroupRemoveAccount (AccountGroup *grp, Account *acc)
 {
+  GncEventData ed;
+
   if (!acc) return;
 
   /* Note this routine might be called on accounts which 
@@ -618,10 +738,18 @@ xaccGroupRemoveAccount (AccountGroup *grp, Account *acc)
     return;
   }
 
-  acc->parent = NULL;
+  /* Gather event data */
+  ed.node = grp->parent;
+  ed.idx = g_list_index(grp->accounts, acc);
 
   grp->accounts = g_list_remove (grp->accounts, acc);
 
+  /* Now send the event. */
+  qof_event_gen(&acc->inst.entity, QOF_EVENT_REMOVE, &ed);
+
+  /* clear the account's group pointer after REMOVE event generation. */
+  acc->parent = NULL;
+
   grp->saved = 0;
 
   /* if this was the last account in a group, delete
@@ -632,7 +760,7 @@ xaccGroupRemoveAccount (AccountGroup *grp, Account *acc)
     xaccFreeAccountGroup (grp);
   }
 
-  gnc_engine_generate_event (&acc->guid, GNC_EVENT_MODIFY);
+  qof_event_gen (&acc->inst.entity, QOF_EVENT_MODIFY, NULL);
 }
 
 /********************************************************************\
@@ -645,7 +773,7 @@ xaccAccountInsertSubAccount (Account *adult, Account *child)
 
   /* if a container for the children doesn't yet exist, add it */
   if (adult->children == NULL)
-    adult->children = xaccMallocAccountGroup (adult->book);
+    adult->children = xaccMallocAccountGroup (adult->inst.book);
 
   /* set back-pointer to parent */
   adult->children->parent = adult;
@@ -655,37 +783,24 @@ xaccAccountInsertSubAccount (Account *adult, Account *child)
 
   xaccGroupInsertAccount (adult->children, child);
 
-  gnc_engine_generate_event (&adult->guid, GNC_EVENT_MODIFY);
+  qof_event_gen (&adult->inst.entity, QOF_EVENT_MODIFY, NULL);
 }
 
 /********************************************************************\
 \********************************************************************/
 
-static int
-group_sort_helper (gconstpointer a, gconstpointer b)
-{
-  Account *aa = (Account *) a;
-  Account *bb = (Account *) b;
-
-  /* return > 1 if aa should come after bb */
-  return xaccAccountOrder (&aa, &bb);
-}
-
 void
 xaccGroupInsertAccount (AccountGroup *grp, Account *acc)
 {
   if (!grp || !grp->book) return;
   if (!acc) return;
 
+  ENTER("group %p, account %p named %s", grp, acc, xaccAccountGetName(acc));
   /* If the account is currently in another group, remove it there
    * first. Basically, we can't have accounts being in two places at
    * once. If old and new parents are the same, reinsertion causes
    * the sort order to be checked. */
-  if (acc->parent == grp)
-  {
-    grp->accounts = g_list_sort (grp->accounts, group_sort_helper);
-  }
-  else
+  if (acc->parent != grp)
   {
     xaccAccountBeginEdit (acc);
 
@@ -694,8 +809,9 @@ xaccGroupInsertAccount (AccountGroup *grp, Account *acc)
       xaccGroupRemoveAccount (acc->parent, acc);
 
       /* switch over between books, if needed */
-      if (grp->book != acc->book)
+      if (grp->book != acc->inst.book)
       {
+         QofCollection *col;
 // xxxxxxxxxxxxxxxxxxxxxxx
          /* hack alert -- this implementation is not exactly correct.
           * If the entity tables are not identical, then the 'from' book 
@@ -708,29 +824,31 @@ xaccGroupInsertAccount (AccountGroup *grp, Account *acc)
           *
           * Note also, we need to reparent the children to the new book as well.
           */
-         PWARN ("reparenting accounts accross books is not correctly supported\n");
+         PWARN ("reparenting accounts across books is not correctly supported\n");
 
-         gnc_engine_generate_event (&acc->guid, GNC_EVENT_DESTROY);
-         xaccRemoveEntity (acc->book->entity_table, &acc->guid);
-
-         xaccStoreEntity (grp->book->entity_table, acc, &acc->guid, GNC_ID_ACCOUNT);
-         gnc_engine_generate_event (&acc->guid, GNC_EVENT_CREATE);
+         qof_event_gen (&acc->inst.entity, QOF_EVENT_DESTROY, NULL);
+         col = qof_book_get_collection (grp->book, GNC_ID_ACCOUNT);
+         qof_collection_insert_entity (col, &acc->inst.entity);
+         qof_event_gen (&acc->inst.entity, QOF_EVENT_CREATE, NULL);
       }
     }
 
     /* set back-pointer to the account's parent */
     acc->parent = grp;
 
-    grp->accounts = g_list_insert_sorted (grp->accounts, acc,
-                                          group_sort_helper);
+    grp->accounts = g_list_append (grp->accounts, acc);
 
-    acc->core_dirty = TRUE;
+    /* Gather event data */
+    qof_event_gen (&acc->inst.entity, QOF_EVENT_ADD, NULL);
+
+    qof_instance_set_dirty(&acc->inst);
     xaccAccountCommitEdit (acc);
   }
 
   grp->saved = 0;
 
-  gnc_engine_generate_event (&acc->guid, GNC_EVENT_MODIFY);
+  qof_event_gen (&acc->inst.entity, QOF_EVENT_MODIFY, NULL);
+  LEAVE(" ");
 }
 
 /********************************************************************\
@@ -784,13 +902,12 @@ xaccGroupCopyGroup (AccountGroup *to, AccountGroup *from)
       to_acc = xaccCloneAccount (from_acc, to->book);
 
       xaccAccountBeginEdit (to_acc);
-      to->accounts = g_list_insert_sorted (to->accounts, to_acc,
-                                          group_sort_helper);
+      to->accounts = g_list_append (to->accounts, to_acc);
 
       to_acc->parent = to;
-      to_acc->core_dirty = TRUE;
+      qof_instance_set_dirty(&to_acc->inst);
 
-      /* copy child accounts too. */
+      /* Copy child accounts too. */
       if (from_acc->children)
       {
          to_acc->children = xaccMallocAccountGroup (to->book);
@@ -798,7 +915,7 @@ xaccGroupCopyGroup (AccountGroup *to, AccountGroup *from)
          xaccGroupCopyGroup (to_acc->children, from_acc->children);
       }
       xaccAccountCommitEdit (to_acc);
-      gnc_engine_generate_event (&to_acc->guid, GNC_EVENT_CREATE);
+      qof_event_gen (&to_acc->inst.entity, QOF_EVENT_CREATE, NULL);
 
       /* make sure that we have a symmetric, uniform number of 
        * begin-edits, so that subsequent GroupCommitEdit's 
@@ -844,7 +961,6 @@ xaccGroupMergeAccounts (AccountGroup *grp)
           (xaccAccountGetType(acc_a) == xaccAccountGetType(acc_b)))
       {
         AccountGroup *ga, *gb;
-        GList *lp;
 
         /* consolidate children */
         ga = (AccountGroup *) acc_a->children;
@@ -858,14 +974,14 @@ xaccGroupMergeAccounts (AccountGroup *grp)
             gb->parent = acc_a;
             acc_b->children = NULL;
 
-            gnc_engine_generate_event (&acc_a->guid, GNC_EVENT_MODIFY);
-            gnc_engine_generate_event (&acc_b->guid, GNC_EVENT_MODIFY);
+            qof_event_gen (&acc_a->inst.entity, QOF_EVENT_MODIFY, NULL);
+            qof_event_gen (&acc_b->inst.entity, QOF_EVENT_MODIFY, NULL);
           }
           else
           {
             xaccGroupConcatGroup (ga, gb);
             acc_b->children = NULL;
-            gnc_engine_generate_event (&acc_b->guid, GNC_EVENT_MODIFY);
+            qof_event_gen (&acc_b->inst.entity, QOF_EVENT_MODIFY, NULL);
           }
         }
 
@@ -873,27 +989,14 @@ xaccGroupMergeAccounts (AccountGroup *grp)
         xaccGroupMergeAccounts (ga);
 
         /* consolidate transactions */
-        lp = acc_b->splits;
-        
-        for (lp = acc_b->splits; lp; lp = lp->next)
-        {
-          Split *split = lp->data;
-
-          gnc_engine_generate_event (&xaccSplitGetAccount(split)->guid,
-                                     GNC_EVENT_MODIFY);
-          split->acc = NULL;
-          xaccAccountInsertSplit (acc_a, split);
-        }
-
-        g_list_free(acc_b->splits);
-        acc_b->splits = NULL;
+        while (acc_b->splits)
+          xaccSplitSetAccount (acc_b->splits->data, acc_a);
 
         /* move back one before removal */
         node_b = node_b->prev;
 
-        /* remove from list -- node_a is ok, it's before node_b */
-        grp->accounts = g_list_remove (grp->accounts, acc_b);
-
+        /* The destroy function will remove from list -- node_a is ok,
+	 * it's before node_b */
         xaccAccountBeginEdit (acc_b);
         xaccAccountDestroy (acc_b);
         break;
@@ -906,7 +1009,7 @@ xaccGroupMergeAccounts (AccountGroup *grp)
 \********************************************************************/
 
 int     
-xaccGroupGetNumAccounts (AccountGroup *grp)
+xaccGroupGetNumAccounts (const AccountGroup *grp)
 {
    if (!grp) return 0;
 
@@ -914,7 +1017,7 @@ xaccGroupGetNumAccounts (AccountGroup *grp)
 }
 
 Account *
-xaccGroupGetAccount (AccountGroup *grp, int i)
+xaccGroupGetAccount (const AccountGroup *grp, int i)
 {
    if (!grp) return NULL;
 
@@ -922,7 +1025,7 @@ xaccGroupGetAccount (AccountGroup *grp, int i)
 }
 
 Account *
-xaccGroupGetParentAccount (AccountGroup * grp)
+xaccGroupGetParentAccount (const AccountGroup * grp)
 {
   if (!grp) return NULL;
 
@@ -933,7 +1036,7 @@ xaccGroupGetParentAccount (AccountGroup * grp)
 \********************************************************************/
 
 int     
-xaccGroupGetDepth (AccountGroup *grp)
+xaccGroupGetDepth (const AccountGroup *grp)
 {
   GList *node;
   int depth = 0;
@@ -958,7 +1061,6 @@ xaccGroupGetDepth (AccountGroup *grp)
 
 /********************************************************************\
 \********************************************************************/
-
 void
 xaccSplitsBeginStagedTransactionTraversals (GList *splits)
 {
@@ -975,10 +1077,10 @@ xaccSplitsBeginStagedTransactionTraversals (GList *splits)
 }
 
 void
-xaccAccountBeginStagedTransactionTraversals (Account *account)
+xaccAccountBeginStagedTransactionTraversals (const Account *account)
 {
-  if (account == NULL) return;
-  xaccSplitsBeginStagedTransactionTraversals (account->splits);
+  if (account)
+      xaccSplitsBeginStagedTransactionTraversals (account->splits);
 }
 
 gboolean
@@ -1028,41 +1130,30 @@ xaccGroupBeginStagedTransactionTraversals (AccountGroup *grp)
 }
 
 int
-xaccAccountStagedTransactionTraversal (Account *acc,
+xaccAccountStagedTransactionTraversal (const Account *acc,
                                        unsigned int stage,
                                        int (*callback)(Transaction *t,
                                                        void *cb_data),
                                        void *cb_data)
 {
+  GList *lp;
   if (!acc) return 0;
 
-  if (callback)
+  for(lp = acc->splits; lp; lp = lp->next)
   {
-    GList *lp;
-    for(lp = acc->splits; lp; lp = lp->next)
+    Split *s = (Split *) lp->data;
+    Transaction *trans = s->parent;   
+    if (trans && (trans->marker < stage))
     {
-      Split *s = (Split *) lp->data;
-      Transaction *trans = s->parent;   
-      if (trans && (trans->marker < stage))
+      trans->marker = stage;
+      if (callback)
       {
         int retval;
-        trans->marker = stage;
         retval = callback(trans, cb_data);
         if (retval) return retval;
       }
     }
   }
-  else
-  {
-    GList *lp;
-    for(lp = acc->splits; lp; lp = lp->next)
-    {
-      Split *s = (Split *) lp->data;
-      Transaction *trans = s->parent;      
-      if (trans && (trans->marker < stage))
-        trans->marker = stage;
-    }
-  }
 
   return 0;
 }
@@ -1099,90 +1190,15 @@ xaccGroupStagedTransactionTraversal (AccountGroup *grp,
 /********************************************************************\
 \********************************************************************/
 
-struct group_visit_data
-{
-    gboolean (*proc)(Transaction *t, void *data);
-    void *up_data;
-    GHashTable *visit_table;
-};
-
-static gboolean
-xaccGroupVisitUnvisitedTransactions_thunk(Transaction *trn,
-                                          void *data)
-{
-    gpointer test_trn;
-    struct group_visit_data *grdata = (struct group_visit_data*)data;
-    gboolean result;
-
-    test_trn = g_hash_table_lookup(grdata->visit_table, trn);
-
-    if(!test_trn)
-    {
-        g_hash_table_insert(grdata->visit_table, trn, "");
-
-        result = grdata->proc(trn, grdata->up_data);
-    }
-    else
-      result = TRUE;
-
-    return result;
-}
-
-gboolean
-xaccGroupVisitUnvisitedTransactions (AccountGroup *g,
-                                     gboolean (*proc)(Transaction *t,
-                                                      void *data),
-                                     void *data,
-                                     GHashTable *visited_txns)
-{
-  gboolean keep_going = TRUE;
-  GList *list;
-  GList *node;
-  struct group_visit_data grdata;
-  
-  if (!g) return(FALSE);
-  if (!proc) return(FALSE);
-  if (!visited_txns) return(FALSE);
-
-  list = xaccGroupGetSubAccounts (g);
-
-  grdata.proc = proc;
-  grdata.up_data = data;
-  grdata.visit_table = visited_txns;
-  
-  for (node = list; node && keep_going; node = node->next)
-  {
-    Account *account = node->data;
-    
-    keep_going = xaccAccountForEachTransaction(
-        account, xaccGroupVisitUnvisitedTransactions_thunk, (void*)&grdata);
-  }
-
-  g_list_free (list);
-
-  return(keep_going);
-}
-
-gboolean
+int
 xaccGroupForEachTransaction (AccountGroup *g,
-                             gboolean (*proc)(Transaction *t, void *data),
+                             int (*proc)(Transaction *t, void *data),
                              void *data)
 {
-  GHashTable *visited_txns = NULL;
-  gboolean result = FALSE;
+  if (!g || !proc) return 0;
 
-  if (!g) return(FALSE);
-  if (!proc) return(FALSE);
-
-  visited_txns = guid_hash_table_new();
-  if (visited_txns)
-    result = xaccGroupVisitUnvisitedTransactions(g, proc, data, visited_txns);
-
-  /* cleanup */
-  if (visited_txns)
-    g_hash_table_destroy(visited_txns);  
-
-  return(result);
+  xaccGroupBeginStagedTransactionTraversals (g);
+  return xaccGroupStagedTransactionTraversal (g, 42, proc, data);
 }
 
 /********************************************************************\
@@ -1196,8 +1212,7 @@ xaccGroupMapAccounts (AccountGroup *grp,
   GList *result = NULL;
   GList *node;
 
-  if (!grp) return(NULL);
-  if (!thunk) return(NULL);
+  if (!grp || !thunk) return NULL;
 
   for (node = grp->accounts; node; node = node->next)
   {
@@ -1221,8 +1236,7 @@ xaccGroupForEachAccount (AccountGroup *grp,
 {
   GList *node;
 
-  if (!grp) return(NULL);
-  if (!thunk) return(NULL);
+  if (!grp || !thunk) return(NULL);
 
   for (node = grp->accounts; node; node = node->next)
   {
@@ -1243,4 +1257,62 @@ xaccGroupForEachAccount (AccountGroup *grp,
   return(NULL);
 }
 
+/* ============================================================== */
+
+QofBackend *
+xaccGroupGetBackend (const AccountGroup *grp)
+{
+  grp = xaccGroupGetRoot (grp);
+  if (!grp || !grp->book) return NULL;
+  return qof_book_get_backend(grp->book);
+}
+
+/* ============================================================== */
+/* gncObject function implementation and registration */
+
+static void 
+group_book_begin (QofBook *book)
+{
+  xaccSetAccountGroup (book, xaccMallocAccountGroup(book));
+}
+
+static void 
+group_book_end (QofBook *book)
+{
+  xaccSetAccountGroup (book, NULL);
+}
+
+static gboolean
+group_is_dirty (QofCollection *col)
+{
+  return xaccGroupNotSaved(xaccCollGetAccountGroup(col));
+}
+
+static void
+group_mark_clean(QofCollection *col)
+{
+  xaccGroupMarkSaved(xaccCollGetAccountGroup(col));
+}
+
+static QofObject group_object_def = 
+{
+  interface_version: QOF_OBJECT_VERSION,
+  e_type:            GNC_ID_GROUP,
+  type_label:        "AccountGroup",
+  create:            (gpointer)xaccMallocAccountGroup,
+  book_begin:        group_book_begin,
+  book_end:          group_book_end,
+  is_dirty:          group_is_dirty,
+  mark_clean:        group_mark_clean,
+  foreach:           NULL,
+  printable:         NULL,
+  version_cmp:       NULL,
+};
+
+gboolean 
+xaccGroupRegister (void)
+{
+  return qof_object_register (&group_object_def);
+}
+
 /* ========================= END OF FILE ======================== */
