@@ -1,6 +1,6 @@
 (define-module (g-wrapped gw-engine-spec))
 (debug-set! maxdepth 100000)
-(debug-set! stack    2000000)
+(debug-set! stack    200000)
 
 (use-modules (g-wrap))
 (use-modules (g-wrap simple-type))
@@ -21,22 +21,21 @@
  ws
  (lambda (wrapset client-wrapset)
    (list
+    "#include <config.h>\n"
     "#include <glib.h>\n"
-    "#include <guid.h>\n"
-    "#include <Query.h>\n"
-    "#include <Backend.h>\n"
+    "#include <qof.h>\n"
     "#include <Group.h>\n"
-    "#include <gnc-book.h>\n"
-    "#include <gnc-session.h>\n"
-    "#include <gnc-session-scm.h>\n"
-    "#include <gnc-engine-util.h>\n"
-    "#include <gnc-event.h>\n"
-    "#include <gnc-lot.h>\n"
-    "#include <date.h>\n"
-    "#include <engine-helpers.h>\n"
-    "#include <gnc-engine.h>\n"
+    "#include <Query.h>\n"
+    "#include <gnc-budget.h>\n"
     "#include <gnc-commodity.h>\n"
-    "#include <gnc-numeric.h>\n")))
+    "#include <gnc-engine.h>\n"
+    "#include <gnc-filepath-utils.h>\n"
+    "#include <gnc-pricedb.h>\n"
+    "#include <gnc-lot.h>\n"
+    "#include <gnc-session-scm.h>\n"
+    "#include <gnc-hooks-scm.h>\n"
+    "#include <engine-helpers.h>\n"
+    "#include <SX-book.h>\n")))
 
 (gw:wrapset-add-cs-initializers!
  ws
@@ -92,61 +91,64 @@
 
 (gw:wrap-as-wct ws '<gnc:GList*> "GList*" "const GList*")
 
-(gw:wrap-as-wct ws '<gnc:id-type> "GNCIdType" "GNCIdTypeConst")
+(gw:wrap-as-wct ws '<gnc:id-type> "QofIdType" "QofIdTypeConst")
 (gw:wrap-as-wct ws '<gnc:Account*> "Account*" "const Account*")
 (gw:wrap-as-wct ws '<gnc:Account**> "Account**" "const Account**")
 (gw:wrap-as-wct ws '<gnc:AccountGroup*> "AccountGroup*" "const AccountGroup*")
-(gw:wrap-as-wct ws '<gnc:Book*> "GNCBook*" "const GNCBook*")
+(gw:wrap-as-wct ws '<gnc:Book*> "QofBook*" "const QofBook*")
 (gw:wrap-as-wct ws '<gnc:Lot*> "GNCLot*" "const GNCLot*")
-(gw:wrap-as-wct ws '<gnc:Session*> "GNCSession*" "const GNCSession**")
+(gw:wrap-as-wct ws '<gnc:Session*> "QofSession*" "const QofSession**")
 (gw:wrap-as-wct ws '<gnc:Split*> "Split*" "const Split*")
 (gw:wrap-as-wct ws '<gnc:Transaction*> "Transaction*" "const Transaction*")  
 (gw:wrap-as-wct ws '<gnc:commodity*> "gnc_commodity*" "const gnc_commodity*")
+(gw:wrap-as-wct ws '<gnc:commodity-namespace*>
+                "gnc_commodity_namespace*" 
+                "const gnc_commodity_namespace*")
 (gw:wrap-as-wct ws '<gnc:commodity-table*>
                 "gnc_commodity_table*" 
                 "const gnc_commodity_table*")
 
 (gw:wrap-as-wct ws '<gnc:Query*> "Query *" "const Query *")
 
-(let ((wt (gw:wrap-enumeration ws '<gnc:event-type> "GNCEngineEventType")))
+(let ((wt (gw:wrap-enumeration ws '<gnc:event-type> "QofEventId")))
 
-  (gw:enum-add-value! wt "GNC_EVENT_NONE" 'gnc-event-none)
-  (gw:enum-add-value! wt "GNC_EVENT_CREATE" 'gnc-event-create)
-  (gw:enum-add-value! wt "GNC_EVENT_MODIFY" 'gnc-event-modify)
-  (gw:enum-add-value! wt "GNC_EVENT_DESTROY" 'gnc-event-destroy)
-  (gw:enum-add-value! wt "GNC_EVENT_ALL" 'gnc-event-all))
+  (gw:enum-add-value! wt "QOF_EVENT_NONE" 'qof-event-none)
+  (gw:enum-add-value! wt "QOF_EVENT_CREATE" 'qof-event-create)
+  (gw:enum-add-value! wt "QOF_EVENT_MODIFY" 'qof-event-modify)
+  (gw:enum-add-value! wt "QOF_EVENT_DESTROY" 'qof-event-destroy)
+  (gw:enum-add-value! wt "QOF_EVENT_ALL" 'qof-event-all))
 
-(let ((wt (gw:wrap-enumeration ws '<gnc:query-op> "QueryOp")))
+(let ((wt (gw:wrap-enumeration ws '<gnc:query-op> "QofQueryOp")))
 
-  (gw:enum-add-value! wt "QUERY_AND" 'query-and)
-  (gw:enum-add-value! wt "QUERY_OR" 'query-or)
-  (gw:enum-add-value! wt "QUERY_NAND" 'query-nand)
-  (gw:enum-add-value! wt "QUERY_NOR" 'query-nor)
-  (gw:enum-add-value! wt "QUERY_XOR" 'query-xor))
+  (gw:enum-add-value! wt "QOF_QUERY_AND" 'query-and)
+  (gw:enum-add-value! wt "QOF_QUERY_OR" 'query-or)
+  (gw:enum-add-value! wt "QOF_QUERY_NAND" 'query-nand)
+  (gw:enum-add-value! wt "QOF_QUERY_NOR" 'query-nor)
+  (gw:enum-add-value! wt "QOF_QUERY_XOR" 'query-xor))
 
-(let ((wt (gw:wrap-enumeration ws '<gnc:query-compare-how> "query_compare_t")))
-  (gw:enum-add-value! wt "COMPARE_LT" 'query-compare-lt)
-  (gw:enum-add-value! wt "COMPARE_LTE" 'query-compare-lte)
-  (gw:enum-add-value! wt "COMPARE_EQUAL" 'query-compare-equal)
-  (gw:enum-add-value! wt "COMPARE_GT" 'query-compare-gt)
-  (gw:enum-add-value! wt "COMPARE_GTE" 'query-compare-gte)
-  (gw:enum-add-value! wt "COMPARE_NEQ" 'query-compare-neq)
+(let ((wt (gw:wrap-enumeration ws '<gnc:query-compare-how> "QofQueryCompare")))
+  (gw:enum-add-value! wt "QOF_COMPARE_LT" 'query-compare-lt)
+  (gw:enum-add-value! wt "QOF_COMPARE_LTE" 'query-compare-lte)
+  (gw:enum-add-value! wt "QOF_COMPARE_EQUAL" 'query-compare-equal)
+  (gw:enum-add-value! wt "QOF_COMPARE_GT" 'query-compare-gt)
+  (gw:enum-add-value! wt "QOF_COMPARE_GTE" 'query-compare-gte)
+  (gw:enum-add-value! wt "QOF_COMPARE_NEQ" 'query-compare-neq)
   #t)
 
-(let ((wt (gw:wrap-enumeration ws '<gnc:string-match-how> "string_match_t")))
-  (gw:enum-add-value! wt "STRING_MATCH_NORMAL" 'string-match-normal)
-  (gw:enum-add-value! wt "STRING_MATCH_CASEINSENSITIVE" 'string-match-caseinsensitive)
+(let ((wt (gw:wrap-enumeration ws '<gnc:string-match-how> "QofStringMatch")))
+  (gw:enum-add-value! wt "QOF_STRING_MATCH_NORMAL" 'string-match-normal)
+  (gw:enum-add-value! wt "QOF_STRING_MATCH_CASEINSENSITIVE" 'string-match-caseinsensitive)
   #t)
 
-(let ((wt (gw:wrap-enumeration ws '<gnc:date-match-how> "date_match_t")))
-  (gw:enum-add-value! wt "DATE_MATCH_NORMAL" 'date-match-normal)
-  (gw:enum-add-value! wt "DATE_MATCH_ROUNDED" 'date-match-rounded)
+(let ((wt (gw:wrap-enumeration ws '<gnc:date-match-how> "QofDateMatch")))
+  (gw:enum-add-value! wt "QOF_DATE_MATCH_NORMAL" 'date-match-normal)
+  (gw:enum-add-value! wt "QOF_DATE_MATCH_DAY" 'date-match-rounded)
   #t)
 
-(let ((wt (gw:wrap-enumeration ws '<gnc:numeric-match-how> "numeric_match_t")))
-  (gw:enum-add-value! wt "NUMERIC_MATCH_ANY" 'amt-sgn-match-either)
-  (gw:enum-add-value! wt "NUMERIC_MATCH_CREDIT" 'amt-sgn-match-credit)
-  (gw:enum-add-value! wt "NUMERIC_MATCH_DEBIT" 'amt-sgn-match-debit)
+(let ((wt (gw:wrap-enumeration ws '<gnc:numeric-match-how> "QofNumericMatch")))
+  (gw:enum-add-value! wt "QOF_NUMERIC_MATCH_ANY" 'amt-sgn-match-either)
+  (gw:enum-add-value! wt "QOF_NUMERIC_MATCH_CREDIT" 'amt-sgn-match-credit)
+  (gw:enum-add-value! wt "QOF_NUMERIC_MATCH_DEBIT" 'amt-sgn-match-debit)
   #t)
 
 (let ((wt (gw:wrap-enumeration ws '<gnc:cleared-match-how> "cleared_match_t")))
@@ -157,16 +159,17 @@
   (gw:enum-add-value! wt "CLEARED_VOIDED" 'cleared-match-voided)
   #t)
 
-(let ((wt (gw:wrap-enumeration ws '<gnc:guid-match-how> "guid_match_t")))
-  (gw:enum-add-value! wt "GUID_MATCH_ALL" 'guid-match-all)
-  (gw:enum-add-value! wt "GUID_MATCH_ANY" 'guid-match-any)
-  (gw:enum-add-value! wt "GUID_MATCH_NULL" 'guid-match-null)
-  (gw:enum-add-value! wt "GUID_MATCH_NONE" 'guid-match-none)
+(let ((wt (gw:wrap-enumeration ws '<gnc:guid-match-how> "QofGuidMatch")))
+  (gw:enum-add-value! wt "QOF_GUID_MATCH_ALL" 'guid-match-all)
+  (gw:enum-add-value! wt "QOF_GUID_MATCH_ANY" 'guid-match-any)
+  (gw:enum-add-value! wt "QOF_GUID_MATCH_NULL" 'guid-match-null)
+  (gw:enum-add-value! wt "QOF_GUID_MATCH_NONE" 'guid-match-none)
+  (gw:enum-add-value! wt "QOF_GUID_MATCH_LIST_ANY" 'guid-match-list-any)
   #t)
 
-(let ((wt (gw:wrap-enumeration ws '<gnc:char-match-how> "char_match_t")))
-  (gw:enum-add-value! wt "CHAR_MATCH_ANY" 'char-match-any)
-  (gw:enum-add-value! wt "CHAR_MATCH_NONE" 'char-match-none)
+(let ((wt (gw:wrap-enumeration ws '<gnc:char-match-how> "QofCharMatch")))
+  (gw:enum-add-value! wt "QOF_CHAR_MATCH_ANY" 'char-match-any)
+  (gw:enum-add-value! wt "QOF_CHAR_MATCH_NONE" 'char-match-none)
   #t)
 
 (let ((wt (gw:wrap-enumeration ws '<gnc:query-txn-match-t> "query_txn_match_t")))
@@ -174,7 +177,7 @@
   (gw:enum-add-value! wt "QUERY_TXN_MATCH_ANY" 'query-txn-match-any)
   #t)
 
-(let ((wt (gw:wrap-enumeration ws '<gnc:kvp-value-t> "kvp_value_t")))
+(let ((wt (gw:wrap-enumeration ws '<gnc:kvp-value-t> "KvpValueType")))
   (gw:enum-add-value! wt "KVP_TYPE_GINT64" 'kvp-type-gint64)
   (gw:enum-add-value! wt "KVP_TYPE_DOUBLE" 'kvp-type-double)
   (gw:enum-add-value! wt "KVP_TYPE_NUMERIC" 'kvp-type-numeric)
@@ -210,7 +213,7 @@
   (gw:enum-add-value! we "CREDITLINE" 'credit-line)
   #t)
 
-(let ((we (gw:wrap-enumeration ws '<gnc:BackendError> "GNCBackendError")))
+(let ((we (gw:wrap-enumeration ws '<gnc:BackendError> "QofBackendError")))
 
   (gw:enum-add-value! we "ERR_BACKEND_NO_ERR" 'no-err)
   (gw:enum-add-value! we "ERR_BACKEND_NO_BACKEND" 'no-backend)
@@ -239,20 +242,14 @@
   
   ;;     (gw:enum-add-value! we "ERR_SQL_MISSING_DATA" 'sql-missing-data)
 
-  ;;     (gw:enum-add-value! we "ERR_RPC_HOST_UNK" 'rpc_host_unk)
-  ;;     (gw:enum-add-value! we "ERR_RPC_CANT_BIND" 'rpc_cant_bind)
-  ;;     (gw:enum-add-value! we "ERR_RPC_CANT_ACCEPT" 'rpc_cant_accept)
-  ;;     (gw:enum-add-value! we "ERR_RPC_NO_CONNECTION" 'rpc_no_connection)
-  ;;     (gw:enum-add-value! we "ERR_RPC_BAD_VERSION" 'rpc_bad_version)
-  ;;     (gw:enum-add-value! we "ERR_RPC_FAILED" 'rpc_failed)
-  ;;     (gw:enum-add-value! we "ERR_RPC_NOT_ADDED" 'rpc_not_added)
   #t)
 
 ;
-; Definitions from GNCId.h
+; Definitions from gnc-engine.h
 ;
 (gw:wrap-value ws 'gnc:id-account '<gnc:id-type> "GNC_ID_ACCOUNT")
 (gw:wrap-value ws 'gnc:id-book '<gnc:id-type> "GNC_ID_BOOK")
+(gw:wrap-value ws 'gnc:id-budget '<gnc:id-type> "GNC_ID_BUDGET")
 (gw:wrap-value ws 'gnc:id-lot '<gnc:id-type> "GNC_ID_LOT")
 (gw:wrap-value ws 'gnc:id-price '<gnc:id-type> "GNC_ID_PRICE")
 (gw:wrap-value ws 'gnc:id-split '<gnc:id-type> "GNC_ID_SPLIT")
@@ -385,6 +382,15 @@
  '((<gnc:Split*> s))
  "Return split's value.")
 
+
+(gw:wrap-function
+ ws
+ 'gnc:account-separator-string
+ '(<gw:mchars> callee-owned const)
+ "gnc_get_account_separator_string"
+ '()
+ "Returns a string with the user-selected account separator")
+
 (gw:wrap-function
  ws
  'gnc:split-get-account
@@ -456,7 +462,7 @@ its account")
  'gnc:split-get-corr-account-full-name-internal
  '(<gw:gchars> caller-owned)
  "xaccSplitGetCorrAccountFullName"
- '((<gnc:Split*> sa) (<gw:char> separator))
+ '((<gnc:Split*> sa))
  "Find the split on the other side of the transaction, and return the 
 name of its account.  Don't use directly, use 
 gnc:split-get-corr-account-full-name in src/scm/report-utilities.scm")
@@ -561,7 +567,7 @@ value.  It is meant to be a short descriptive phrase.")
 (gw:wrap-function
  ws
  'gnc:transaction-get-void-reason
- '(<gw:mchars> callee-owned)
+ '(<gw:mchars> callee-owned const)
  "xaccTransGetVoidReason"
  '((<gnc:Transaction*> transaction))
  "return a string indicating reason for voiding")
@@ -726,35 +732,20 @@ number of nanoseconds.")
  "Insert the split s into account a. If the split already belongs
 to another account, it will be removed from that account first.")
 
-(gw:wrap-function
- ws
- 'gnc:account-fix-split-date-order
- '<gw:void>
- "xaccAccountFixSplitDateOrder"
- '((<gnc:Account*> a) (<gnc:Split*> s))
- "Check to see if split s is in proper sorted date order with respect
-to the other splits in account a.")
-
-(gw:wrap-function
- ws
- 'gnc:trans-fix-split-date-order
- '<gw:void>
- "xaccTransFixSplitDateOrder"
- '((<gnc:Transaction*> t))
- "Check to see if all of the splits in transaction t are in proper
-date order.")
-
-(gw:wrap-function
- ws
- 'gnc:account-order
- '<gw:int>
- "xaccAccountOrder"
- '((<gnc:Account**> a1) (<gnc:Account**> a2))
- "Defines a sorting order on accounts.  Returns -1 if a1 is \"less
-than\" the second, +1 if the a1 is \"greater than\" the second, and 0
-if they are equal.  To determine the sort order, the account codes are
-compared, and if these are equal, then account types, and, if these
-are equal, then account names.")
+;; (gw:wrap-function
+;;  ws
+;;  'gnc:account-order
+;;  '<gw:int>
+;;  "xaccAccountOrder_gwrap"
+;;  '(
+;;    (<gnc:Account**> a1)
+;;    (<gnc:Account**> a2)
+;;    )
+;;  "Defines a sorting order on accounts.  Returns -1 if a1 is \"less
+;; than\" the second, +1 if the a1 is \"greater than\" the second, and 0
+;; if they are equal.  To determine the sort order, the account codes are
+;; compared, and if these are equal, then account types, and, if these
+;; are equal, then account names.")
 
 (gw:wrap-function
  ws
@@ -918,22 +909,6 @@ Expenses) are given numeric codes in corresponding ``number ranges.''")
  '((<gnc:Account*> a))
  "Get the tax payer name source set on the account.")
 
-(gw:wrap-function
- ws
- 'gnc:account-get-price-src
- '(<gw:mchars> callee-owned const)
- "xaccAccountGetPriceSrc"
- '((<gnc:Account*> a))
- "Get the account's price source, if any.")
-
-(gw:wrap-function
- ws
- 'gnc:account-get-quote-tz
- '(<gw:mchars> callee-owned const)
- "xaccAccountGetQuoteTZ"
- '((<gnc:Account*> a))
- "Get the quote source's timezone, if any.")
-
 (gw:wrap-function
  ws
  'gnc:account-get-children
@@ -1029,7 +1004,7 @@ children to this account.")
  '<gnc:Book*>
  "xaccGroupGetBook"
  '((<gnc:AccountGroup*> g))
- "Return the GNCBook of group g.")
+ "Return the QofBook of group g.")
 
 (gw:wrap-function
  ws
@@ -1082,8 +1057,7 @@ group")
  '<gnc:Account*>
  "xaccGetAccountFromFullName"
  '((<gnc:AccountGroup*> g)
-   ((<gw:mchars> caller-owned const) name)
-   (<gw:char> separator))
+   ((<gw:mchars> caller-owned const) name))
  "Return account named name in group g.  full path with separators.")
 
 (gw:wrap-function
@@ -1114,7 +1088,7 @@ group")
  ws
  'gnc:group-get-subaccounts
  '(gw:glist-of <gnc:Account*> caller-owned)
- "xaccGroupGetSubAccounts"
+ "xaccGroupGetSubAccountsSorted"
  '((<gnc:AccountGroup*> g))
  "Return a list containing all of the accounts, including
 subaccounts, in the account group. The returned array should be freed
@@ -1123,8 +1097,8 @@ when no longer needed.")
 (gw:wrap-function
  ws
  'gnc:group-get-account-list
- '(gw:glist-of <gnc:Account*> callee-owned)
- "xaccGroupGetAccountList"
+ '(gw:glist-of <gnc:Account*> caller-owned)
+ "xaccGroupGetAccountListSorted"
  '((<gnc:AccountGroup*> g))
  "Return a list containing the immediate children of g.")
 
@@ -1327,6 +1301,27 @@ when no longer needed.")
    (<gnc:commodity*> commodity) (<gnc:time-pair> t))
  "Returns the price(s) nearest to t in any currency available.")
 
+
+(gw:wrap-function
+ws
+'gnc:pricedb-lookup-latest-before
+'<gnc:Price*>
+"gnc_pricedb_lookup_latest_before"
+'((<gnc:PriceDB*> db)
+  (<gnc:commodity*> commodity) (<gnc:commodity*> currency)
+  (<gnc:time-pair> t))
+"Returns the latest price quote <= t. Unref price when finished with it.")
+
+(gw:wrap-function
+ws
+'gnc:pricedb-lookup-latest-before-any-currency
+'(gw:glist-of <gnc:Price*> caller-owned)
+"gnc_pricedb_lookup_latest_before_any_currency"
+'((<gnc:PriceDB*> db)
+  (<gnc:commodity*> commodity) (<gnc:time-pair> t))
+"Returns the latest price quote(s) <= t in any currency available.")
+
+
 (gw:wrap-function
  ws
  'gnc:pricedb-get-prices
@@ -1379,21 +1374,35 @@ when no longer needed.")
  "convert balance in commodity balance_commodity to new_currency using nearest price
 to time t.")
 
+
+(gw:wrap-function
+ ws
+ 'gnc:pricedb-convert-balance-latest-before
+ '<gnc:numeric>
+ "gnc_pricedb_convert_balance_latest_before"
+ '((<gnc:PriceDB*> db)
+   (<gnc:numeric> balance)
+   (<gnc:commodity*> balance_commodity) (<gnc:commodity*> new_currency)
+   (<gnc:time-pair> t))
+ "convert balance in commodity balance_commodity to new_currency using latest price
+prior to time t.")
+
+
 ;;===========
-;; GNCSession
+;; QofSession
 
 (gw:wrap-function
  ws
  'gnc:session-new
  '<gnc:Session*>
- "gnc_session_new" '()
+ "qof_session_new" '()
  "Create a new session.")
 
 (gw:wrap-function
  ws
  'gnc:session-destroy
  '<gw:void>
- "gnc_session_destroy"
+ "qof_session_destroy"
  '((<gnc:Session*> session))
  "Destroy the given session.")
 
@@ -1401,7 +1410,7 @@ to time t.")
  ws
  'gnc:session-get-book
  '<gnc:Book*>
- "gnc_session_get_book"
+ "qof_session_get_book"
  '((<gnc:Session*> session))
  "Get the book of the given session.")
 
@@ -1409,7 +1418,7 @@ to time t.")
  ws
  'gnc:session-begin
  '<gw:void>
- "gnc_session_begin"
+ "qof_session_begin"
  '((<gnc:Session*> session)
    ((<gw:mchars> caller-owned const) id)
    (<gw:bool> ignore-lock?)
@@ -1432,14 +1441,6 @@ to time t.")
  '((<gnc:Session*> session))
  "Save the data in the session.")
 
-(gw:wrap-function
- ws
- 'gnc:session-export
- '<gw:bool>
- "gnc_session_scm_export"
- '((<gnc:Session*> tmp_session) (<gnc:Session*> real_session))
- "Export the accounts in the session.")
-
 (gw:wrap-function
  ws
  'gnc:session-set-callback
@@ -1454,7 +1455,7 @@ argument between 0 and 100 (inclusive).")
  ws
  'gnc:session-end
  '<gw:void>
- "gnc_session_end"
+ "qof_session_end"
  '((<gnc:Session*> session))
  "Indicate you're finished with the session.")
 
@@ -1462,15 +1463,23 @@ argument between 0 and 100 (inclusive).")
  ws
  'gnc:book-get-group
  '<gnc:AccountGroup*>
- "gnc_book_get_group"
+ "xaccGetAccountGroup"
  '((<gnc:Book*> book))
  "Get the book's account group.")
 
+(gw:wrap-function
+ ws
+ 'gnc:book-get-template-group
+ '<gnc:AccountGroup*>
+ "gnc_book_get_template_group"
+ '((<gnc:Book*> book))
+ "Get the book's template account group.")
+
 (gw:wrap-function
  ws
  'gnc:book-get-commodity-table
  '<gnc:commodity-table*>
- "gnc_book_get_commodity_table"
+ "gnc_commodity_table_get_table"
  '((<gnc:Book*> book))
  "Get the book's commodity table.")
 
@@ -1478,7 +1487,7 @@ argument between 0 and 100 (inclusive).")
  ws
  'gnc:book-get-pricedb
  '<gnc:PriceDB*>
- "gnc_book_get_pricedb"
+ "gnc_pricedb_get_db"
  '((<gnc:Book*> book))
  "Get the book's pricedb.")
 
@@ -1486,7 +1495,7 @@ argument between 0 and 100 (inclusive).")
  ws
  'gnc:book-kvp-changed
  '<gw:void>
- "gnc_book_kvp_changed"
+ "qof_book_kvp_changed"
  '((<gnc:Book*> book))
  "Set the flag that the Book's kvp changed.")
 
@@ -1494,7 +1503,7 @@ argument between 0 and 100 (inclusive).")
  ws
  'gnc:session-get-error
  '<gnc:BackendError>
- "gnc_session_get_error"
+ "qof_session_get_error"
  '((<gnc:Session*> session))
  "Check for a pending error.")
 
@@ -1502,7 +1511,7 @@ argument between 0 and 100 (inclusive).")
  ws
  'gnc:session-get-url
  '(<gw:mchars> callee-owned const)
- "gnc_session_get_url"
+ "qof_session_get_url"
  '((<gnc:Session*> session))
  "Return the URL of the opened session.")
 
@@ -1510,7 +1519,7 @@ argument between 0 and 100 (inclusive).")
  ws
  'gnc:session-pop-error
  '<gnc:BackendError>
- "gnc_session_pop_error"
+ "qof_session_pop_error"
  '((<gnc:Session*> session))
  "Remove an error, if any, from the error stack.")
 
@@ -1518,7 +1527,7 @@ argument between 0 and 100 (inclusive).")
  ws
  'gnc:set-log-level-global
  '<gw:void>
- "gnc_set_log_level_global"
+ "qof_log_set_level_registered"
  '((<gw:int> level))
  "Set the logging level for all modules to level.")
 
@@ -1681,14 +1690,6 @@ of having a parent transaction with which one is working...")
  '((<gnc:Split*> s) (<gw:char> value))
  "Set reconcile state for split entry")
 
-(gw:wrap-function
- ws
- 'd-gnc:split-set-share-price
- '<gw:void>
- "DxaccSplitSetSharePrice"
- '((<gnc:Split*> s) (<gw:double> value))
- "Set share price for split entry")
-
 (gw:wrap-function
  ws
  'gnc:split-set-share-price
@@ -1727,7 +1728,7 @@ of having a parent transaction with which one is working...")
  ws
  'gnc:query-create
  '<gnc:Query*>
- "gncQueryCreate"
+ "qof_query_create"
  '()
  "Create a new (empty) Query structure.")
 
@@ -1735,7 +1736,7 @@ of having a parent transaction with which one is working...")
  ws
  'gnc:query-create-for
  '<gnc:Query*>
- "gncQueryCreateFor"
+ "qof_query_create_for"
  '((<gnc:id-type> obj-type))
  "Create a new (empty) Query structure to search for the supplied type.")
 
@@ -1743,7 +1744,7 @@ of having a parent transaction with which one is working...")
  ws
  'gnc:query-search-for
  '<gw:void>
- "gncQuerySearchFor"
+ "qof_query_search_for"
  '((<gnc:Query*> q) (<gnc:id-type> obj-type))
  "Set the object-type to search for.")
 
@@ -1791,7 +1792,7 @@ of having a parent transaction with which one is working...")
  ws
  'gnc:query-purge-terms
  '<gw:void>
- "gncQueryPurgeTerms"
+ "qof_query_purge_terms"
  '((<gnc:Query*> q) ((gw:gslist-of <gnc:id-type> caller-owned) param-path))
  "Remove query terms of a particular parameter-path.")
 
@@ -1987,11 +1988,11 @@ of having a parent transaction with which one is working...")
  ws
  'gnc:query-set-sort-order
  '<gw:void>
- "xaccQuerySetSortOrder"
+ "qof_query_set_sort_order"
  '((<gnc:Query*> q)
-   ((gw:glist-of <gnc:id-type> callee-owned) primary)
-   ((gw:glist-of <gnc:id-type> callee-owned) secondary)
-   ((gw:glist-of <gnc:id-type> callee-owned) tertiary))
+   ((gw:gslist-of <gnc:id-type> callee-owned) primary)
+   ((gw:gslist-of <gnc:id-type> callee-owned) secondary)
+   ((gw:gslist-of <gnc:id-type> callee-owned) tertiary))
  "set sort order.")
 
 (gw:wrap-function
@@ -2009,7 +2010,7 @@ of having a parent transaction with which one is working...")
  ws
  'gnc:query-set-max-results
  '<gw:void>
- "gncQuerySetMaxResults"
+ "qof_query_set_max_results"
  '((<gnc:Query*> q) (<gw:int> n))
  "Set the max number of results to be returned by a query.")
 
@@ -2038,7 +2039,8 @@ of having a parent transaction with which one is working...")
  'gnc:commodity-create
  '<gnc:commodity*>
  "gnc_commodity_new"
- '(((<gw:mchars> caller-owned const) fullname)
+ '((<gnc:Book*> book)
+   ((<gw:mchars> caller-owned const) fullname)
    ((<gw:mchars> caller-owned const) namespace)
    ((<gw:mchars> caller-owned const) mnemonic)
    ((<gw:mchars> caller-owned const) exchange-code)
@@ -2089,9 +2091,9 @@ of having a parent transaction with which one is working...")
  ws
  'gnc:commodity-get-exchange-code
  '(<gw:mchars> callee-owned const)
- "gnc_commodity_get_exchange_code"
+ "gnc_commodity_get_cusip"
  '((<gnc:commodity*> comm))
- "Get the exchange code (ISO numeric code)")
+ "Get the cusip (exchange specific data, not the stock ticker)")
 
 (gw:wrap-function
  ws
@@ -2101,6 +2103,14 @@ of having a parent transaction with which one is working...")
  '((<gnc:commodity*> comm))
  "Get the number of smallest transactional units per unit of the currency")
 
+(gw:wrap-function
+ ws
+ 'gnc:commodity-is-currency?
+ '<gw:bool>
+ "gnc_commodity_is_currency"
+ '((<gnc:commodity*> comm))
+ "return true if the commodity is an ISO4217 currency")
+
 (gw:wrap-function
  ws
  'gnc:commodity-equiv?
@@ -2166,10 +2176,11 @@ of having a parent transaction with which one is working...")
 (gw:wrap-function
  ws
  'gnc:commodity-table-add-namespace
- '<gw:void>
+ '<gnc:commodity-namespace*>
  "gnc_commodity_table_add_namespace"
  '((<gnc:commodity-table*> table)
-   ((<gw:mchars> caller-owned const) namespace))
+   ((<gw:mchars> caller-owned const) namespace)
+   (<gnc:Book*> book))
  "Add a new namespace to the commodity table")
 
 (gw:wrap-function
@@ -2188,14 +2199,23 @@ of having a parent transaction with which one is working...")
  "gnc_commodity_table_get_commodities"
  '((<gnc:commodity-table*> table)
    ((<gw:mchars> caller-owned const) namespace))
- "Return a list of all the namespaces in the table.")
+ "Return a list of all the commodities in a given namespace in the table.")
+
+(gw:wrap-function
+ ws
+ 'gnc:commodity-table-get-quotable-commodities
+ '(gw:glist-of <gnc:commodity*> caller-owned)
+ "gnc_commodity_table_get_quotable_commodities"
+ '((<gnc:commodity-table*> table))
+ "Return a list of all the quotable commodities in a given namespace in the table.")
 
 (gw:wrap-function
  ws
  'gnc:commodity-table-add-default-data
  '<gw:bool>
  "gnc_commodity_table_add_default_data"
- '((<gnc:commodity-table*> table))
+ '((<gnc:commodity-table*> table)
+   (<gnc:Book*> book))
  "Add default commodities to the commodity table.")
 
 ;;=========
@@ -2426,15 +2446,7 @@ of having a parent transaction with which one is working...")
  '((<gnc:numeric> arg))
  "Convert gnc_numeric to a printable string")
 
-(gw:wrap-function
- ws
- 'gnc:run-rpc-server
- '<gw:void>
- "gnc_run_rpc_server"
- '()
- "Run the RPC Server") 
-
-;; src/engine/date.h
+;; gnc-date.h
 
 (gw:wrap-function
  ws
@@ -2445,6 +2457,26 @@ of having a parent transaction with which one is working...")
  "Convert a timepair on a certain day (localtime) to\
 the timepair representing midday on that day")
 
+;;
+;; gnc-filepath-utils.h
+;;
+
+(gw:wrap-function
+ ws
+ 'gnc:build-dotgnucash-path
+ '(<gw:mchars> caller-owned)
+ "gnc_build_dotgnucash_path"
+ '(((<gw:mchars> caller-owned) filename))
+ "Convert a relative path name into a full path name in the .gnucash directory")
+
+(gw:wrap-function
+ ws
+ 'gnc:build-book-path
+ '(<gw:mchars> caller-owned)
+ "gnc_build_book_path"
+ '(((<gw:mchars> caller-owned) filename))
+ "Convert a relative path name into a full path name in the .gnucash/books directory")
+
 ;;
 ;; gnc-lot.h
 ;;
@@ -2475,16 +2507,161 @@ the timepair representing midday on that day")
 
 (gw:wrap-function
  ws
- 'gnc:engine-suspend-events
+ 'qof:event-suspend
  '<gw:void>
- "gnc_engine_suspend_events"
+ "qof_event_suspend"
  '()
  "Suspend all engine events.") 
 
 (gw:wrap-function
  ws
- 'gnc:engine-resume-events
+ 'qof:event-resume
  '<gw:void>
- "gnc_engine_resume_events"
+ "qof_event_resume"
  '()
  "Resume engine event generation.") 
+
+(gw:wrap-function
+ ws
+ 'gnc:quote-source-set-fq-installed
+ '<gw:void>
+ "gnc_quote_source_set_fq_installed"
+ '(((gw:glist-of (<gw:mchars> callee-owned) callee-owned) choices))
+ "Takes a list of installed Finance::Quote souces and records it internally.")
+
+
+;; Budget functions
+
+(gw:wrap-as-wct ws '<gnc:Budget*> "GncBudget *" "const GncBudget *")
+
+(gw:wrap-function
+ ws
+ 'gnc:budget-get-guid
+ '<gnc:guid-scm>
+ "gnc_budget_return_guid"
+ '((<gnc:Budget*> budget))
+ "Gets the guid of the budget")
+
+
+(gw:wrap-function
+ ws
+ 'gnc:budget-lookup
+ '<gnc:Budget*>
+ "gnc_budget_lookup_direct"
+ '((<gnc:guid-scm> guid)
+   (<gnc:Book*> book))
+ "Lookup a budget from its GUID.")
+
+
+(gw:wrap-function
+ ws
+ 'gnc:budget-get-default
+ '<gnc:Budget*>
+ "gnc_budget_get_default"
+ '((<gnc:Book*> book))
+ "Get the default budget for the book.")
+
+
+(gw:wrap-function
+ ws
+ 'gnc:budget-get-name
+ '(<gw:mchars> callee-owned const)
+ "gnc_budget_get_name"
+ '((<gnc:Budget*> budget))
+ "Get the brief name for the budget.")
+
+(gw:wrap-function
+ ws
+ 'gnc:budget-get-num-periods
+ '<gw:unsigned-int>
+ "gnc_budget_get_num_periods"
+ '((<gnc:Budget*> budget))
+ "Get the number of periods in a budget.")
+
+(gw:wrap-function
+ ws
+ 'gnc:budget-get-account-period-value
+ '<gnc:numeric>
+ "gnc_budget_get_account_period_value"
+ '((<gnc:Budget*> budget)
+   (<gnc:Account*> acct)
+   (<gw:unsigned-int> period_num)
+   )
+ "Get the budgeted value for the given account and budget period.")
+
+(gw:wrap-function
+ ws
+ 'gnc:budget-get-account-period-actual-value
+ '<gnc:numeric>
+ "gnc_budget_get_account_period_actual_value"
+ '((<gnc:Budget*> budget)
+   (<gnc:Account*> acct)
+   (<gw:unsigned-int> period_num)
+   )
+ "Get the actual account value for the given account and budget period.")
+
+(gw:wrap-function
+ ws
+ 'gnc:budget-get-period-start-date
+ '<gnc:time-pair>
+ "gnc_budget_get_period_start_date"
+ '((<gnc:Budget*> budget)
+   (<gw:unsigned-int> period_num)
+   )
+ "Get the date that the given period begins.")
+
+;;
+;; gnc-hooks-scm.h
+;;   (and gnc-hooks.h)
+;;
+(gw:wrap-function
+ ws
+ 'gnc:hook-define
+ '(<gw:mchars> caller-owned)
+ "gnc_hook_create"
+ '(((<gw:mchars> caller-owned) name) (<gw:int> how) ((<gw:mchars> caller-owned) desc))
+ "Define (create) a new hook")
+
+(gw:wrap-function
+ ws
+ 'gnc:hook-get-description
+ '(<gw:mchars> callee-owned)
+ "gnc_hook_get_description"
+ '(((<gw:mchars> caller-owned) hook))
+ "Get the description of a hook")
+
+(gw:wrap-function
+ ws
+ 'gnc:hook-add-dangler
+ '<gw:void>
+ "gnc_hook_add_scm_dangler"
+ '(((<gw:mchars> caller-owned) hook) (<gw:scm> procedure))
+ "Add a hook dangler to an existing hook")
+
+(gw:wrap-function
+ ws
+ 'gnc:hook-remove-dangler
+ '<gw:void>
+ "gnc_hook_del_scm_dangler"
+ '(((<gw:mchars> caller-owned) hook) (<gw:scm> procedure))
+ "Remove a hook dangler from an existing hook")
+
+(gw:wrap-function
+ ws
+ 'gnc:hook-run-danglers-real
+ '<gw:void>
+ "gnc_hook_run"
+ '(((<gw:mchars> caller-owned) name) (<gnc:Session*> arg))
+ "Run the danglers on a hook.")
+
+; Now wrap all the 'known' hooks
+(gw:wrap-value ws 'gnc:*new-book-hook*
+	       '(<gw:mchars> callee-owned) "HOOK_NEW_BOOK")
+(gw:wrap-value ws 'gnc:*report-hook*
+	       '(<gw:mchars> callee-owned) "HOOK_REPORT")
+(gw:wrap-value ws 'gnc:*save-options-hook*
+	       '(<gw:mchars> callee-owned) "HOOK_SAVE_OPTIONS")
+(gw:wrap-value ws 'gnc:*book-opened-hook*
+	       '(<gw:mchars> callee-owned) "HOOK_BOOK_OPENED")
+(gw:wrap-value ws 'gnc:*book-closed-hook*
+	       '(<gw:mchars> callee-owned) "HOOK_BOOK_CLOSED")
