@@ -39,25 +39,25 @@
 ( "Brunei Dollar" "dollar" "cent" "ISO4217" "BND" "096" 100 100 )
 ( "Bulgarian Lev" "lev" "stotinki" "ISO4217" "BGL" "100" 100 100 ) ;; replaced 1999
 ( "Bulgarian Lev" "lev" "stotinki" "ISO4217" "BGN" "975" 100 100 )
-( "Burundi Franc" "franc" "centime" "ISO4217" "BIF" "108" 100 100 )
-( "CFA Franc BEAC" "franc" "centime" "ISO4217" "XAF" "950" 100 100 )
-( "CFA Franc BCEAO" "franc" "centime" "ISO4217" "XOF" "952" 100 100 )
+( "Burundi Franc" "franc" "centime" "ISO4217" "BIF" "108" 1 1 )
+( "CFA Franc BEAC" "franc" "centime" "ISO4217" "XAF" "950" 1 1 )
+( "CFA Franc BCEAO" "franc" "centime" "ISO4217" "XOF" "952" 1 1 )
 ( "CFP Franc Pacifique" "franc" "centime" "ISO4217" "XPF" "953" 100 100 )
 ( "Cambodia Riel" "riel" "sen" "ISO4217" "KHR" "116" 100 100 )
 ( "Canadian Dollar" "dollar" "cent" "ISO4217" "CAD" "124" 100 100 )
 ( "Cape Verde Escudo" "escudo" "centavo" "ISO4217" "CVE" "132" 100 100 )
 ( "Cayman Islands Dollar" "dollar" "cent"  "ISO4217" "KYD" "136" 100 100 )
-( "Chilean Peso" "peso" "centavo" "ISO4217" "CLP" "152" 100 100 )
+( "Chilean Peso" "peso" "centavo" "ISO4217" "CLP" "152" 1 1 )
 ( "Chinese Yuan Renminbi" "renminbi" "fen" "ISO4217" "CNY" "156" 100 100 )
 ( "Colombian Peso" "peso" "centavo" "ISO4217" "COP" "170" 100 100 )
-( "Comoros Franc" "franc" "centime" "ISO4217" "KMF" "174" 100 100  )
+( "Comoros Franc" "franc" "centime" "ISO4217" "KMF" "174" 1 1  )
 ( "Costa Rican Colon" "colon" "centimo" "ISO4217" "CRC" "188" 100 100 )
 ( "Croatian Kuna" "kuna" "lipa" "ISO4217" "HRK" "191" 100 100 )
 ( "Cuban Peso" "peso" "centavo" "ISO4217" "CUP" "192" 100 100 )
 ( "Cyprus Pound" "pound" "pence"  "ISO4217" "CYP" "196" 100 100 )
 ( "Czech Koruna" "koruna" "haleru" "ISO4217" "CZK" "203" 100 100  )
 ( "Danish Krone" "krone" "ore" "ISO4217" "DKK" "208" 100 100 )
-( "Djibouti Franc" "franc" "centime" "ISO4217" "DJF" "262" 100 100 )
+( "Djibouti Franc" "franc" "centime" "ISO4217" "DJF" "262" 1 1 )
 ( "Dominican Peso" "peso" "centavo"  "ISO4217" "DOP" "214" 100 100 )
 ( "Dutch Guilder" "guilder" "cent" "ISO4217" "NLG" "528" 100 100 )
 ( "East Caribbean Dollar" "dollar" "cent" "ISO4217" "XCD" "951" 100 100 )
@@ -120,7 +120,7 @@
 ( "Maltese Lira" "lira" "cent"  "ISO4217" "MTL" "470" 100 100 )
 ( "Mauritanian Ouguiya" "ouguiya" "khoum"  "ISO4217" "MRO" "478" 5 5)
 ( "Mauritius Rupee" "rupee" "cent"  "ISO4217" "MUR" "480" 100 100 )
-( "Mexican Peso" "peso" "centavo" "ISO4217" "MXP" "484" 100 100 )
+( "Mexican Peso" "peso" "centavo" "ISO4217" "MXN" "484" 100 100 ) ;;sicne Jan 1993
 ( "Mongolian Tugrik" "tugrik" "mongo" "ISO4217" "MNT" "496" 100 100 )
 ( "Moroccan Dirham" "dirham" "centime"  "ISO4217" "MAD" "504" 100 100 )
 ( "Mozambique Metical" "metical" "centavo"  "ISO4217" "MZM" "508" 100 100 )
@@ -158,13 +158,14 @@
 ( "Solomon Islands Dollar" "dollar" "cent"  "ISO4217" "SBD" "090" 100 100 )
 ( "Somali Shilling" "shilling" "centisimi" "ISO4217" "SOS" "706" 100 100 )
 ( "South African Rand" "rand" "cent" "ISO4217" "ZAR" "710" 100 100 )
-( "South Korean Won" "won" "chon"  "ISO4217" "KRW" "410" 100 100 )
+( "South Korean Won" "won" "chon"  "ISO4217" "KRW" "410" 1 1 )
 ( "Spanish Peseta" "peseta" "centimo"  "ISO4217" "ESP" "724" 100 100 ) ;; through 1998
 ( "Sri Lanka Rupee" "rupee" "cent"  "ISO4217" "LKR" "144" 100 100 )
 ( "St. Helena Pound" "pound" "pence"  "ISO4217" "SHP"  "654" 100 100 )
 ( "Sudanese Dinar" "dinar" "piastre"  "ISO4217" "SDD" "736" 100 100 ) ;; 1992 on
 ( "Sudanese Pound" "pound" "piastre"  "ISO4217" "SDP" "736" 100 100 ) ;; into 1992
-( "Suriname Guilder" "guilder" "cent"  "ISO4217" "SRG" "740" 100 100 )
+( "Suriname Guilder" "guilder" "cent"  "ISO4217" "SRG" "740" 100 100 ) ;; only until 2003-12-31
+( "Suriname Dollar" "dollar" "cent"  "ISO4217" "SRD" "968" 100 100) ;; Since 2004-01-01
 ( "Swaziland Lilangeni" "lilangeni" "cent"  "ISO4217" "SZL" "748" 100 100 )
 ( "Swedish Krona" "krona" "ore"  "ISO4217" "SEK" "752" 100 100 )
 ( "Swiss Franc" "franc" "centime" "ISO4217" "CHF" "756" 100 100 )
@@ -177,7 +178,7 @@
 ( "Tongan Pa'anga" "Pa'anga" "seniti" "ISO4217" "TOP" "776" 100 100 )
 ( "Trinidad and Tobago Dollar" "dollar" "cent" "ISO4217" "TTD" "780" 100 100 )
 ( "Tunisian Dinar" "dinar" "milleme" "ISO4217" "TND" "788" 1000 1000 )
-( "Turkish Lira" "lira" "kuru" "ISO4217" "TRL" "792" 100 100)
+( "Turkish Lira" "lira" "kuru" "ISO4217" "TRL" "792" 1 1)
 ( "Turkmenistan Manat" "manat" "tenga" "ISO4217" "TMM" "795" 100 100 )
 ( "US Dollar" "dollar" "cent" "ISO4217" "USD" "840" 100 100 )
 ( "Uganda Shilling" "shilling" "cent"  "ISO4217" "UGX" "800" 100 100  )
@@ -185,7 +186,7 @@
 ( "United Arab Emirates Dirham" "dirham" "fil" "ISO4217" "AED" "784" 100 100 )
 ( "Uruguayan Peso" "peso" "centesimo" "ISO4217" "UYU" "858" 100 100 )
 ( "Uzbekistani Sum" "som" "tiyin" "ISO4217" "UZS" "860" 100 100 )
-( "Vanuatu Vatu" "vatu" "centime" "ISO4217" "VUV" "548" 100 100 )
+( "Vanuatu Vatu" "vatu" "centime" "ISO4217" "VUV" "548" 1 1 )
 ( "Venezuelan Bolivar" "bolivar" "centimo" "ISO4217" "VEB" "862" 100 100 )
 ( "Vietnamese Dong" "dong" "hao" "ISO4217" "VND" "704" 100 100 )
 ( "Yugoslav Dinar" "dinar" "para"  "ISO4217" "YUM"  "890" 100 100)
