@@ -6,12 +6,16 @@
 
 #include <stdio.h>
 #include <ltdl.h>
-#include <guile/gh.h>
+#include <libguile.h>
 
 #include "gnc-module.h"
 
+#ifndef lt_ptr
+# define lt_ptr lt_ptr_t
+#endif
+
 static void
-guile_main(int argc, char ** argv)
+guile_main(void *closure, int argc, char ** argv)
 {
   lt_dlhandle handle;
 
@@ -40,8 +44,9 @@ guile_main(int argc, char ** argv)
 }
 
 int
-main(int argc, char ** argv) {
-  gh_enter(argc, argv, guile_main);
+main(int argc, char ** argv)
+{
+  scm_boot_guile(argc, argv, guile_main, NULL);
   return 0;
 }
 
