@@ -16,21 +16,23 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include "config.h"
 
-#include <gnome.h>
+#include <gtk/gtk.h>
 
 #include "dialog-utils.h"
 #include "gnc-ui-util.h"
-#include "gnc-engine-util.h"
+#include "qof.h"
 #include "gnc-component-manager.h"
-#include "messages.h"
 #include "gnc-query-list.h"
 #include "search-param.h"
+#include "QueryCore.h"
+#include "QueryNew.h"
+#include "QueryObject.h"
 
 /* Signal codes */
 enum
@@ -40,11 +42,17 @@ enum
   LAST_SIGNAL
 };
 
-struct _GNCQueryListPriv {
-  QueryAccess	get_guid;
-  gint		component_id;
+typedef struct _GNCQueryListPriv  GNCQueryListPriv;
+
+struct _GNCQueryListPriv 
+{
+  const QofParam * get_guid;
+  gint	          component_id;
 };
 
+#define GNC_QUERY_LIST_GET_PRIVATE(o)  \
+   (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNC_TYPE_QUERY_LIST, GNCQueryListPriv))
+
 /* Impossible to get at runtime. Assume this is a reasonable number */
 #define ARROW_SIZE      14
 #define VSCROLLBAR_SLOP 40
@@ -80,20 +88,21 @@ gnc_query_list_get_type (void)
 
   if (!gnc_query_list_type)
   {
-    static const GtkTypeInfo gnc_query_list_info =
-    {
-      "GNCQueryList",
-      sizeof (GNCQueryList),
-      sizeof (GNCQueryListClass),
-      (GtkClassInitFunc) gnc_query_list_class_init,
-      (GtkObjectInitFunc) gnc_query_list_init,
-      /* reserved_1 */ NULL,
-      /* reserved_2 */ NULL,
-      (GtkClassInitFunc) NULL
+    GTypeInfo type_info = {
+      sizeof(GNCQueryListClass),        /* class_size */
+      NULL,   				/* base_init */
+      NULL,				/* base_finalize */
+      (GClassInitFunc)gnc_query_list_class_init,
+      NULL,				/* class_finalize */
+      NULL,				/* class_data */
+      sizeof (GNCQueryList),		/* */
+      0,				/* n_preallocs */
+      (GInstanceInitFunc)gnc_query_list_init,
     };
 
-    gnc_query_list_type = gtk_type_unique(GTK_TYPE_CLIST,
-					      &gnc_query_list_info);
+    gnc_query_list_type = g_type_register_static(GTK_TYPE_CLIST,	
+						 "GNCQueryList",
+						 &type_info, 0);
   }
 
   return gnc_query_list_type;
@@ -111,6 +120,8 @@ gnc_query_list_get_type (void)
 void
 gnc_query_list_construct (GNCQueryList *list, GList *param_list, Query *query)
 {
+  GNCQueryListPriv *priv;
+
   g_return_if_fail(list);
   g_return_if_fail(param_list);
   g_return_if_fail(query);
@@ -121,9 +132,9 @@ gnc_query_list_construct (GNCQueryList *list, GList *param_list, Query *query)
   list->column_params = param_list;
 
   /* cache the function to get the guid of this query type */
-  list->priv->get_guid =
-    gncQueryObjectGetParameterGetter (gncQueryGetSearchFor (query),
-				      QUERY_PARAM_GUID);
+  priv = GNC_QUERY_LIST_GET_PRIVATE(list);
+  priv->get_guid =
+    qof_class_get_parameter (qof_query_get_search_for(query), QOF_PARAM_GUID);
 
   /* Initialize the CList */
   gnc_query_list_init_clist(list);
@@ -134,11 +145,15 @@ GtkWidget *
 gnc_query_list_new(GList *param_list, Query *query)
 {
   GNCQueryList *list;
+  gint columns;
 
   g_return_val_if_fail(param_list, NULL);
   g_return_val_if_fail(query, NULL);
 
-  list = GNC_QUERY_LIST(gtk_type_new(gnc_query_list_get_type()));
+  columns = g_list_length(param_list);
+  list = GNC_QUERY_LIST(g_object_new(gnc_query_list_get_type(),
+				     "n_columns", columns,
+				     NULL));
 
   gnc_query_list_construct(list, param_list, query);
 
@@ -213,6 +228,8 @@ gnc_query_list_refresh_handler (GHashTable *changes, gpointer user_data)
 static void
 gnc_query_list_init (GNCQueryList *list)
 {
+  GNCQueryListPriv *priv;
+
   list->query = NULL;
   list->no_toggle = FALSE;
   list->always_unselect = FALSE;
@@ -230,8 +247,8 @@ gnc_query_list_init (GNCQueryList *list)
   list->numeric_abs = FALSE;
   list->numeric_inv_sort = FALSE;
 
-  list->priv = g_new0(GNCQueryListPriv, 1);
-  list->priv->component_id =
+  priv = GNC_QUERY_LIST_GET_PRIVATE(list);
+  priv->component_id =
     gnc_register_gui_component ("gnc-query-list-cm-class",
 				gnc_query_list_refresh_handler,
 				NULL, list);
@@ -258,8 +275,7 @@ gnc_query_list_init_clist (GNCQueryList *list)
     titles[i] = (gchar *)param->title;
   }
 
-  /* construct the clist */
-  gtk_clist_construct (clist, list->num_columns, titles);
+  gtk_clist_column_titles_show (clist);
   gtk_clist_set_shadow_type (clist, GTK_SHADOW_IN);
 
   /* build all the column titles */
@@ -278,15 +294,16 @@ gnc_query_list_init_clist (GNCQueryList *list)
       gtk_clist_set_column_resizeable (clist, i, FALSE);
   }
 
-  gtk_signal_connect (GTK_OBJECT (clist), "click_column",
-		      GTK_SIGNAL_FUNC(gnc_query_list_click_column_cb),
-		      NULL);
-  gtk_signal_connect (GTK_OBJECT (clist), "size_allocate",
-		      GTK_SIGNAL_FUNC(gnc_query_list_size_allocate_cb),
-		      NULL);
+  g_signal_connect (clist, "click_column",
+		    G_CALLBACK(gnc_query_list_click_column_cb),
+		    NULL);
+  g_signal_connect (clist, "size_allocate",
+		    G_CALLBACK(gnc_query_list_size_allocate_cb),
+		    NULL);
 
   style = gtk_widget_get_style (GTK_WIDGET(list));
 
+#if 0
   {
     GdkFont *font = NULL;
     gint width;
@@ -305,6 +322,7 @@ gnc_query_list_init_clist (GNCQueryList *list)
       }
     }
   }
+#endif
   g_free(titles);
 }
 
@@ -323,29 +341,29 @@ gnc_query_list_class_init (GNCQueryListClass *klass)
 
   parent_class = gtk_type_class(GTK_TYPE_CLIST);
 
+  g_type_class_add_private(klass, sizeof(GNCQueryListPriv));
+
   query_list_signals[LINE_TOGGLED] =
-    gtk_signal_new("line_toggled",
-		   GTK_RUN_FIRST,
-		   object_class->type,
-		   GTK_SIGNAL_OFFSET(GNCQueryListClass,
-				     line_toggled),
-		   gtk_marshal_NONE__POINTER,
-		   GTK_TYPE_NONE, 1,
-		   GTK_TYPE_POINTER);
+    g_signal_new("line_toggled",
+		 G_TYPE_FROM_CLASS (object_class),
+		 G_SIGNAL_RUN_FIRST,
+		 G_STRUCT_OFFSET(GNCQueryListClass, line_toggled),
+		 NULL, NULL,
+		 g_cclosure_marshal_VOID__POINTER,
+		 G_TYPE_NONE,
+		 1,
+		 G_TYPE_POINTER);
 
   query_list_signals[DOUBLE_CLICK_ENTRY] =
-    gtk_signal_new("double_click_entry",
-		   GTK_RUN_FIRST,
-		   object_class->type,
-		   GTK_SIGNAL_OFFSET(GNCQueryListClass,
-				     double_click_entry),
-		   gtk_marshal_NONE__POINTER,
-		   GTK_TYPE_NONE, 1,
-		   GTK_TYPE_POINTER);
-
-  gtk_object_class_add_signals(object_class,
-			       query_list_signals,
-			       LAST_SIGNAL);
+    g_signal_new("double_click_entry",
+		 G_TYPE_FROM_CLASS (object_class),
+		 G_SIGNAL_RUN_FIRST,
+		 G_STRUCT_OFFSET(GNCQueryListClass, double_click_entry),
+		 NULL, NULL,
+		 g_cclosure_marshal_VOID__POINTER,
+		 G_TYPE_NONE,
+		 1,
+		 G_TYPE_POINTER);
 
   object_class->destroy = gnc_query_list_destroy;
 
@@ -371,7 +389,7 @@ gnc_query_list_toggle (GNCQueryList *list)
   entry = gtk_clist_get_row_data (GTK_CLIST(list), row);
   list->current_entry = entry;
 
-  gtk_signal_emit (GTK_OBJECT (list), query_list_signals[LINE_TOGGLED], entry);
+  g_signal_emit (list, query_list_signals[LINE_TOGGLED], 0, entry);
 
   update_booleans (list, row);
 }
@@ -399,8 +417,7 @@ gnc_query_list_select_row (GtkCList *clist, gint row, gint column,
 
     entry = gtk_clist_get_row_data (clist, row);
 
-    gtk_signal_emit(GTK_OBJECT(list),
-		    query_list_signals[DOUBLE_CLICK_ENTRY], entry);
+    g_signal_emit(list, query_list_signals[DOUBLE_CLICK_ENTRY], 0, entry);
   }
 }
 
@@ -429,8 +446,7 @@ gnc_query_list_unselect_row (GtkCList *clist, gint row, gint column,
 
     entry = gtk_clist_get_row_data (clist, row);
 
-    gtk_signal_emit (GTK_OBJECT(list),
-		     query_list_signals[DOUBLE_CLICK_ENTRY], entry);
+    g_signal_emit (list, query_list_signals[DOUBLE_CLICK_ENTRY], 0, entry);
   }
 }
 
@@ -438,13 +454,12 @@ static void
 gnc_query_list_destroy (GtkObject *object)
 {
   GNCQueryList *list = GNC_QUERY_LIST(object);
+  GNCQueryListPriv *priv;
 
-  if (list->priv && list->priv->component_id >= 0)
-    gnc_unregister_gui_component (list->priv->component_id);
-  if (list->priv)
-  {
-    g_free (list->priv);
-    list->priv = NULL;
+  priv = GNC_QUERY_LIST_GET_PRIVATE(list);
+  if (priv->component_id > 0) {
+    gnc_unregister_gui_component (priv->component_id);
+    priv->component_id = 0;
   }
   if (list->query)
   {
@@ -487,7 +502,7 @@ gnc_query_list_get_needed_height (GNCQueryList *list, gint num_rows)
 
   /* sync with gtkclist.c */
   title_height = (clist->column_title_area.height +
-                  (GTK_WIDGET(list)->style->klass->ythickness +
+                  (GTK_WIDGET(list)->style->ythickness +
                    GTK_CONTAINER(list)->border_width) * 2);
   list_height = (clist->row_height * num_rows) + (num_rows + 1);
 
@@ -758,13 +773,15 @@ gnc_query_list_click_column_cb(GtkWidget *w, gint column, gpointer data)
 static void
 gnc_query_list_fill(GNCQueryList *list)
 {
+  GNCQueryListPriv *priv;
   gchar *strings[list->num_columns + 1];
   GList *entries, *item;
   const GUID *guid;
   gint i;
 
   /* Clear all watches */
-  gnc_gui_component_clear_watches (list->priv->component_id);
+  priv = GNC_QUERY_LIST_GET_PRIVATE(list);
+  gnc_gui_component_clear_watches (priv->component_id);
 
   /* Reverse the list now because 'append()' takes too long */
   entries = gncQueryRun(list->query);
@@ -773,6 +790,8 @@ gnc_query_list_fill(GNCQueryList *list)
   {
     GList *node;
     gint row;
+    const QofParam *gup;
+    QofParam *qp= NULL;
 
     for (i = 0, node = list->column_params; node; node = node->next)
     {
@@ -780,33 +799,35 @@ gnc_query_list_fill(GNCQueryList *list)
       GSList *converters = gnc_search_param_get_converters (param);
       const char *type = gnc_search_param_get_param_type (param);
       gpointer res = item->data;
-      QueryAccess fcn = NULL;
 
       /* if this is a boolean, ignore it now -- we'll use a checkmark later */
       if (!safe_strcmp (type, QUERYCORE_BOOLEAN)) {
-	strings[i++] = g_strdup("");
-	continue;
+        strings[i++] = g_strdup("");
+        continue;
       }
 
       /* Do all the object conversions */
-      for (; converters; converters = converters->next) {
-	fcn = converters->data;
-
-	if (converters->next)
-	  res = fcn (res);
+      for (; converters; converters = converters->next) 
+      {
+         qp = converters->data;
+         if (converters->next)
+         {
+            res = (qp->param_getfcn)(res, qp);
+         }
       }
 
       /* Now convert this to a text value for the row */
       if (!safe_strcmp(type, QUERYCORE_DEBCRED) ||
-	  !safe_strcmp(type, QUERYCORE_NUMERIC))
+          !safe_strcmp(type, QUERYCORE_NUMERIC))
       {
-	gnc_numeric (*nfcn)(gpointer) = (gnc_numeric(*)(gpointer))fcn;
-	gnc_numeric value = nfcn(res);
-	if (list->numeric_abs)
-	  value = gnc_numeric_abs (value);
-	strings[i++] = g_strdup(xaccPrintAmount(value,gnc_default_print_info(FALSE)));
+        gnc_numeric (*nfcn)(gpointer, QofParam *) = 
+                (gnc_numeric(*)(gpointer, QofParam *))(qp->param_getfcn);
+        gnc_numeric value = nfcn(res, qp);
+        if (list->numeric_abs)
+          value = gnc_numeric_abs (value);
+        strings[i++] = g_strdup(xaccPrintAmount(value,gnc_default_print_info(FALSE)));
       } else
-	strings[i++] = gncQueryCoreToString (type, res, fcn);
+        strings[i++] = gncQueryCoreToString (type, res, qp);
     }
 
     row = gtk_clist_append (GTK_CLIST(list), (gchar **) strings);
@@ -815,16 +836,17 @@ gnc_query_list_fill(GNCQueryList *list)
     /* Free up our strings */
     for (i = 0; i < list->num_columns; i++) {
       if (strings[i])
-	g_free (strings[i]);
+        g_free (strings[i]);
     }
 
     /* Now update any checkmarks */
     update_booleans (list, row);
 
     /* and set a watcher on this item */
-    guid = (const GUID*)((list->priv->get_guid)(item->data));
-    gnc_gui_component_watch_entity (list->priv->component_id, guid,
-				    GNC_EVENT_MODIFY | GNC_EVENT_DESTROY);
+    gup = priv->get_guid;
+    guid = (const GUID*)((gup->param_getfcn)(item->data, gup));
+    gnc_gui_component_watch_entity (priv->component_id, guid,
+				    QOF_EVENT_MODIFY | QOF_EVENT_DESTROY);
 
     list->num_entries++;
   }
