@@ -1,7 +1,8 @@
 /********************************************************************\
  * druid-stock-split.c -- stock split druid for GnuCash             *
  * Copyright (C) 2001 Gnumatic, Inc.                                *
- * Author: Dave Peticolas <dave@krondo.com>                         *
+ * Copyright (c) 2001 Dave Peticolas <dave@krondo.com>              *
+ * Copyright (c) 2006 David Hampton <hampton@employees.org>         *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -17,36 +18,35 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include "config.h"
 
 #include <gnome.h>
-#include <libgnomeui/gnome-window-icon.h>
+#include <glib/gi18n.h>
 
 #include "Group.h"
 #include "Transaction.h"
 #include "dialog-utils.h"
+#include "druid-stock-split.h"
 #include "druid-utils.h"
-#include "global-options.h"
-#include "gnc-account-tree.h"
 #include "gnc-amount-edit.h"
+#include "gnc-book.h"
 #include "gnc-component-manager.h"
 #include "gnc-currency-edit.h"
 #include "gnc-date-edit.h"
-#include "gnc-engine-util.h"
+#include "qof.h"
 #include "gnc-exp-parser.h"
 #include "gnc-gui-query.h"
+#include "gnc-tree-view-account.h"
 #include "gnc-ui.h"
 #include "gnc-ui-util.h"
-#include "messages.h"
 
 
 #define DRUID_STOCK_SPLIT_CM_CLASS "druid-stock-split"
 
-
 /** structures *********************************************************/
 typedef struct
 {
@@ -55,7 +55,7 @@ typedef struct
 
   /* account page data */
   GtkWidget * account_list;
-  GUID account;
+  Account   * acct;
 
   /* info page data */
   GtkWidget * date_edit;
@@ -72,9 +72,32 @@ typedef struct
 } StockSplitInfo;
 
 
-/** implementations ****************************************************/
-static void
-window_destroy_cb (GtkObject *object, gpointer data)
+/** declarations *******************************************************/
+void     gnc_stock_split_druid_window_destroy_cb (GtkObject *object, gpointer data);
+gboolean gnc_stock_split_druid_account_next      (GnomeDruidPage *druidpage,
+						  gpointer arg1,
+						  gpointer user_data);
+void     gnc_stock_split_druid_details_prepare   (GnomeDruidPage *druidpage,
+						  gpointer arg1,
+						  gpointer user_data);
+gboolean gnc_stock_split_druid_details_next      (GnomeDruidPage *druidpage,
+						  gpointer arg1,
+						  gpointer user_data);
+void     gnc_stock_split_druid_cash_prepare      (GnomeDruidPage *druidpage,
+						  gpointer arg1,
+						  gpointer user_data);
+gboolean gnc_stock_split_druid_cash_next         (GnomeDruidPage *druidpage,
+						  gpointer arg1,
+						  gpointer user_data);
+void     gnc_stock_split_druid_finish            (GnomeDruidPage *druidpage,
+						  gpointer arg1,
+						  gpointer user_data);
+void     gnc_stock_split_druid_cancel_druid      (GnomeDruid *druid,
+						  gpointer user_data);
+
+/******* implementations ***********************************************/
+void
+gnc_stock_split_druid_window_destroy_cb (GtkObject *object, gpointer data)
 {
   StockSplitInfo *info = data;
 
@@ -97,25 +120,22 @@ fill_account_list (StockSplitInfo *info, Account *account)
 
   gtk_clist_clear (clist);
 
-  accounts = xaccGroupGetSubAccounts (gnc_get_current_group ());
+  accounts = xaccGroupGetSubAccountsSorted (gnc_get_current_group ());
   for (node = accounts; node; node = node->next)
   {
     Account *account = node->data;
     GNCPrintAmountInfo print_info;
     const gnc_commodity *commodity;
-    GNCAccountType account_type;
     gnc_numeric balance;
     char *strings[4];
     gint row;
 
-    account_type = xaccAccountGetType (account);
-    if (account_type != STOCK &&
-        account_type != MUTUAL)
-      continue;
+    if (!xaccAccountIsPriced(account))
+        continue;
 
     balance = xaccAccountGetBalance (account);
     if (gnc_numeric_zero_p (balance))
-      continue;
+        continue;
 
     if (xaccAccountGetPlaceholder (account))
 	continue;
@@ -124,8 +144,7 @@ fill_account_list (StockSplitInfo *info, Account *account)
 
     print_info = gnc_account_print_info (account, FALSE);
 
-    strings[0] = xaccAccountGetFullName (account,
-                                         gnc_get_account_separator ());
+    strings[0] = xaccAccountGetFullName (account);
     strings[1] = (char *) gnc_commodity_get_mnemonic (commodity);
     strings[2] = (char *) xaccPrintAmount (balance, print_info);
     strings[3] = NULL;
@@ -138,6 +157,7 @@ fill_account_list (StockSplitInfo *info, Account *account)
 
     rows++;
   }
+  g_list_free(accounts);
 
   {
     gint row = 0;
@@ -170,16 +190,24 @@ clist_select_row (GtkCList *clist,
 
   account = gtk_clist_get_row_data (clist, row);
 
-  info->account = *xaccAccountGetGUID (account);
+  /* Happens when the first row is inserted, before the row data can be added */
+  if (account == NULL)
+    return;
+
+  info->acct = account;
 }
 
 static void
 refresh_details_page (StockSplitInfo *info)
 {
   GNCPrintAmountInfo print_info;
+  gnc_commodity *commodity, *currency;
   Account *account;
+  QofBook *book;
+  GNCPriceDB *db;
+  GList *prices;
 
-  account = xaccAccountLookup (&info->account, gnc_get_current_book ());
+  account = info->acct;
 
   g_return_if_fail (account != NULL);
 
@@ -190,22 +218,33 @@ refresh_details_page (StockSplitInfo *info)
   gnc_amount_edit_set_fraction (GNC_AMOUNT_EDIT (info->distribution_edit),
                                 xaccAccountGetCommoditySCU (account));
 
+  commodity = xaccAccountGetCommodity (account);
+  book = xaccAccountGetBook (account);
+  db = gnc_book_get_pricedb(book);
+
+  prices = gnc_pricedb_lookup_latest_any_currency(db, commodity);
+  if (prices) {
+    /* Use the first existing price */
+    currency = gnc_price_get_currency(prices->data);
+  } else {
+    /* Take a wild guess. */
+    currency = gnc_default_currency ();
+ }
+  gnc_price_list_destroy(prices);
+
   gnc_currency_edit_set_currency
     (GNC_CURRENCY_EDIT (info->price_currency_edit),
-     xaccAccountGetCommodity (account));
+     currency);
 }
 
-static gboolean
-account_next (GnomeDruidPage *druidpage,
-              gpointer arg1,
-              gpointer user_data)
+gboolean
+gnc_stock_split_druid_account_next (GnomeDruidPage *druidpage,
+				    gpointer arg1,
+				    gpointer user_data)
 {
   StockSplitInfo *info = user_data;
-  Account *account;
 
-  account = xaccAccountLookup (&info->account, gnc_get_current_book ());
-
-  g_return_val_if_fail (account != NULL, TRUE);
+  g_return_val_if_fail (info->acct != NULL, TRUE);
 
   refresh_details_page (info);
 
@@ -224,14 +263,14 @@ gnc_parse_error_dialog (StockSplitInfo *info, const char *error_string)
   if (error_string == NULL)
     error_string = "";
 
-  gnc_error_dialog_parented (GTK_WINDOW (info->window),
-			     "%s.\n\n%s: %s.",
-			     error_string, _("Error"),
-			     parse_error_string);
+  gnc_error_dialog (info->window,
+		    "%s.\n\n%s: %s.",
+		    error_string, _("Error"),
+		    parse_error_string);
 }
 
-static void
-details_prepare (GnomeDruidPage *druidpage,
+void
+gnc_stock_split_druid_details_prepare (GnomeDruidPage *druidpage,
 		 gpointer arg1,
 		 gpointer user_data)
 {
@@ -240,10 +279,10 @@ details_prepare (GnomeDruidPage *druidpage,
   gtk_widget_grab_focus(info->distribution_edit);
 }
 
-static gboolean
-details_next (GnomeDruidPage *druidpage,
-              gpointer arg1,
-              gpointer user_data)
+gboolean
+gnc_stock_split_druid_details_next (GnomeDruidPage *druidpage,
+				    gpointer arg1,
+				    gpointer user_data)
 {
   StockSplitInfo *info = user_data;
   gnc_numeric amount;
@@ -261,14 +300,14 @@ details_next (GnomeDruidPage *druidpage,
   if (gnc_numeric_zero_p (amount))
   {
     const char *message = _("You must enter a distribution amount.");
-    gnc_error_dialog_parented (GTK_WINDOW (info->window), message);
+    gnc_error_dialog (info->window, message);
     return TRUE;
   }
 
   if (!gnc_amount_edit_evaluate (GNC_AMOUNT_EDIT (info->price_edit)))
   {
     gnc_parse_error_dialog (info,
-                            _("You must either enter a valid price\n"
+                            _("You must either enter a valid price "
                               "or leave it blank."));
     return TRUE;
   }
@@ -278,35 +317,36 @@ details_next (GnomeDruidPage *druidpage,
   if (gnc_numeric_negative_p (amount))
   {
     const char *message = _("The price must be positive.");
-    gnc_error_dialog_parented (GTK_WINDOW (info->window), message);
+    gnc_error_dialog (info->window, message);
     return TRUE;
   }
 
   return FALSE;
 }
 
-static void
-cash_prepare (GnomeDruidPage *druidpage,
-              gpointer arg1,
-              gpointer user_data)
+void
+gnc_stock_split_druid_cash_prepare (GnomeDruidPage *druidpage,
+				    gpointer arg1,
+				    gpointer user_data)
 {
   StockSplitInfo *info = user_data;
+  GtkTreeSelection *selection;
 
-  gnc_account_tree_refresh (GNC_ACCOUNT_TREE (info->income_tree));
-  gnc_account_tree_expand_all (GNC_ACCOUNT_TREE (info->income_tree));
-  gtk_clist_select_row (GTK_CLIST (info->income_tree), 0, 0);
+  gtk_tree_view_expand_all (GTK_TREE_VIEW(info->income_tree));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(info->income_tree));
+  gtk_tree_selection_unselect_all (selection);
 
-  gnc_account_tree_refresh (GNC_ACCOUNT_TREE (info->asset_tree));
-  gnc_account_tree_expand_all (GNC_ACCOUNT_TREE (info->asset_tree));
-  gtk_clist_select_row (GTK_CLIST (info->asset_tree), 0, 0);
+  gtk_tree_view_expand_all (GTK_TREE_VIEW(info->asset_tree));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(info->asset_tree));
+  gtk_tree_selection_unselect_all (selection);
 
   gtk_widget_grab_focus(info->cash_edit);
 }
 
-static gboolean
-cash_next (GnomeDruidPage *druidpage,
-           gpointer arg1,
-           gpointer user_data)
+gboolean
+gnc_stock_split_druid_cash_next (GnomeDruidPage *druidpage,
+				 gpointer arg1,
+				 gpointer user_data)
 {
   StockSplitInfo *info = user_data;
   gnc_numeric amount;
@@ -314,7 +354,7 @@ cash_next (GnomeDruidPage *druidpage,
   if (!gnc_amount_edit_evaluate (GNC_AMOUNT_EDIT (info->cash_edit)))
   {
     gnc_parse_error_dialog (info,
-                            _("You must either enter a valid cash amount\n"
+                            _("You must either enter a valid cash amount "
                               "or leave it blank."));
     return TRUE;
   }
@@ -324,7 +364,7 @@ cash_next (GnomeDruidPage *druidpage,
   if (gnc_numeric_negative_p (amount))
   {
     const char *message = _("The cash distribution must be positive.");
-    gnc_error_dialog_parented (GTK_WINDOW (info->window), message);
+    gnc_error_dialog (info->window, message);
     return TRUE;
   }
 
@@ -332,23 +372,21 @@ cash_next (GnomeDruidPage *druidpage,
   {
     Account *account;
 
-    account = gnc_account_tree_get_current_account
-      (GNC_ACCOUNT_TREE (info->income_tree));
+    account = gnc_tree_view_account_get_selected_account (GNC_TREE_VIEW_ACCOUNT(info->income_tree));
     if (!account)
     {
-      const char *message = _("You must select an income account\n"
+      const char *message = _("You must select an income account "
                               "for the cash distribution.");
-      gnc_error_dialog_parented (GTK_WINDOW (info->window), message);
+      gnc_error_dialog (info->window, message);
       return TRUE;
     }
 
-    account = gnc_account_tree_get_current_account
-      (GNC_ACCOUNT_TREE (info->asset_tree));
+    account = gnc_tree_view_account_get_selected_account (GNC_TREE_VIEW_ACCOUNT(info->asset_tree));
     if (!account)
     {
-      const char *message = _("You must select an asset account\n"
+      const char *message = _("You must select an asset account "
                               "for the cash distribution.");
-      gnc_error_dialog_parented (GTK_WINDOW (info->window), message);
+      gnc_error_dialog (info->window, message);
       return TRUE;
     }
   }
@@ -356,10 +394,10 @@ cash_next (GnomeDruidPage *druidpage,
   return FALSE;
 }
 
-static void
-stock_split_finish (GnomeDruidPage *druidpage,
-                    gpointer arg1,
-                    gpointer user_data)
+void
+gnc_stock_split_druid_finish (GnomeDruidPage *druidpage,
+			      gpointer arg1,
+			      gpointer user_data)
 {
   StockSplitInfo *info = user_data;
   GList *account_commits;
@@ -371,7 +409,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
   Split *split;
   time_t date;
 
-  account = xaccAccountLookup (&info->account, gnc_get_current_book ());
+  account = info->acct;
   g_return_if_fail (account != NULL);
 
   amount = gnc_amount_edit_get_amount
@@ -387,7 +425,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
   xaccTransSetCurrency (trans, gnc_default_currency ());
 
   date = gnc_date_edit_get_date (GNC_DATE_EDIT (info->date_edit));
-  xaccTransSetDateSecs (trans, date);
+  xaccTransSetDatePostedSecs (trans, date);
 
   {
     const char *description;
@@ -438,8 +476,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
     pdb = gnc_book_get_pricedb (book);
 
     if (!gnc_pricedb_add_price (pdb, price))
-      gnc_error_dialog_parented (GTK_WINDOW (info->window),
-                                 _("Error adding price."));
+      gnc_error_dialog (info->window, _("Error adding price."));
 
     gnc_price_unref (price);
   }
@@ -452,8 +489,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
     memo = gtk_entry_get_text (GTK_ENTRY (info->memo_entry));
 
     /* asset split */
-    account = gnc_account_tree_get_current_account
-      (GNC_ACCOUNT_TREE (info->asset_tree));
+    account = gnc_tree_view_account_get_selected_account (GNC_TREE_VIEW_ACCOUNT(info->asset_tree));
 
     split = xaccMallocSplit (gnc_get_current_book ());
 
@@ -471,8 +507,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
 
 
     /* income split */
-    account = gnc_account_tree_get_current_account
-      (GNC_ACCOUNT_TREE (info->income_tree));
+    account = gnc_tree_view_account_get_selected_account (GNC_TREE_VIEW_ACCOUNT(info->income_tree));
 
     split = xaccMallocSplit (gnc_get_current_book ());
 
@@ -500,36 +535,51 @@ stock_split_finish (GnomeDruidPage *druidpage,
   gnc_close_gui_component_by_data (DRUID_STOCK_SPLIT_CM_CLASS, info);
 }
 
-static void
-druid_cancel (GnomeDruid *druid, gpointer user_data)
+void
+gnc_stock_split_druid_cancel_druid (GnomeDruid *druid, gpointer user_data)
 {
   StockSplitInfo *info = user_data;
 
   gnc_close_gui_component_by_data (DRUID_STOCK_SPLIT_CM_CLASS, info);
 }
 
+static gboolean
+gnc_stock_split_druid_view_filter_income (Account  *account,
+					  gpointer  data)
+{
+  GNCAccountType type;
+
+  type = xaccAccountGetType(account);
+  return (type == INCOME);
+}
+
+static gboolean
+gnc_stock_split_druid_view_filter_asset (Account  *account,
+					 gpointer  data)
+{
+  GNCAccountType type;
+
+  type = xaccAccountGetType(account);
+  return ((type == BANK) || (type == CASH) || (type == ASSET));
+}
+
 static void
 gnc_stock_split_druid_create (StockSplitInfo *info)
 {
-  GtkWidget *page;
   GladeXML *xml;
 
   xml = gnc_glade_xml_new ("stocks.glade", "Stock Split Druid");
-
   info->window = glade_xml_get_widget (xml, "Stock Split Druid");
-
   info->druid = glade_xml_get_widget (xml, "stock_split_druid");
 
-  gtk_signal_connect (GTK_OBJECT (info->window), "destroy",
-                      GTK_SIGNAL_FUNC (window_destroy_cb), info);
+  glade_xml_signal_autoconnect_full(xml, gnc_glade_autoconnect_full_func, info);
 
-  gtk_signal_connect (GTK_OBJECT (info->druid), "cancel",
-                      GTK_SIGNAL_FUNC (druid_cancel), info);
+  /* libglade2 is broken. It should read these from the glade file. */
+  gnc_druid_set_colors (GNOME_DRUID(info->druid));
+  gnc_druid_set_watermark_images (GNOME_DRUID(info->druid),
+				  "stock_split_title.png",
+				  "stock_split_watermark.png");
 
-  gnc_druid_set_title_image (GNOME_DRUID(info->druid),
-                             "stock_split_title.png");
-  gnc_druid_set_watermark_image (GNOME_DRUID(info->druid),
-                                 "stock_split_watermark.png");
 
   /* account list */
   {
@@ -541,13 +591,8 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
 
     gtk_clist_set_selection_mode (clist, GTK_SELECTION_BROWSE);
 
-    gtk_signal_connect (GTK_OBJECT (clist), "select_row",
-                        GTK_SIGNAL_FUNC (clist_select_row), info);
-
-    page = glade_xml_get_widget (xml, "account_page");
-
-    gtk_signal_connect (GTK_OBJECT (page), "next",
-                        GTK_SIGNAL_FUNC (account_next), info);
+    g_signal_connect (clist, "select_row",
+		      G_CALLBACK (clist_select_row), info);
   }
 
   /* info widgets */
@@ -556,6 +601,7 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
     GtkWidget *amount;
     GtkWidget *date;
     GtkWidget *ce;
+    GtkWidget *label;
 
     info->description_entry = glade_xml_get_widget (xml, "description_entry");
 
@@ -563,11 +609,15 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
     date = gnc_date_edit_new(time(NULL), FALSE, FALSE);
     gtk_box_pack_start (GTK_BOX (box), date, TRUE, TRUE, 0);
     info->date_edit = date;
+    label = glade_xml_get_widget (xml, "date_label");
+    gnc_date_make_mnemonic_target (GNC_DATE_EDIT(date), label);
 
     box = glade_xml_get_widget (xml, "distribution_box");
     amount = gnc_amount_edit_new ();
     gtk_box_pack_start (GTK_BOX (box), amount, TRUE, TRUE, 0);
     info->distribution_edit = amount;
+    label = glade_xml_get_widget (xml, "distribution_label");
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), amount);
 
     box = glade_xml_get_widget (xml, "price_box");
     amount = gnc_amount_edit_new ();
@@ -576,89 +626,67 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
     gnc_amount_edit_set_evaluate_on_enter (GNC_AMOUNT_EDIT (amount), TRUE);
     gtk_box_pack_start (GTK_BOX (box), amount, TRUE, TRUE, 0);
     info->price_edit = amount;
+    label = glade_xml_get_widget (xml, "price_label");
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), amount);
 
     box = glade_xml_get_widget (xml, "price_currency_box");
     ce = gnc_currency_edit_new ();
     gtk_box_pack_start (GTK_BOX (box), ce, TRUE, TRUE, 0);
     info->price_currency_edit = ce;
-
-    page = glade_xml_get_widget (xml, "details_page");
-
-    gtk_signal_connect (GTK_OBJECT (page), "prepare",
-                        GTK_SIGNAL_FUNC (details_prepare), info);
-    gtk_signal_connect (GTK_OBJECT (page), "next",
-                        GTK_SIGNAL_FUNC (details_next), info);
+    label = glade_xml_get_widget (xml, "currency_label");
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), ce);
   }
 
   /* Cash in Lieu page */
   {
-    AccountViewInfo view_info;
-    GNCAccountType type;
     GtkWidget *box;
     GtkWidget *tree;
     GtkWidget *amount;
+    GtkWidget *label;
     GtkWidget *scroll;
 
     box = glade_xml_get_widget (xml, "cash_box");
     amount = gnc_amount_edit_new ();
     gtk_box_pack_start (GTK_BOX (box), amount, TRUE, TRUE, 0);
     info->cash_edit = amount;
+    label = glade_xml_get_widget (xml, "cash_label");
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), amount);
 
     info->memo_entry = glade_xml_get_widget (xml, "memo_entry");
 
     /* income tree */
-    tree = gnc_account_tree_new ();
+    tree = GTK_WIDGET(gnc_tree_view_account_new (FALSE));
     info->income_tree = tree;
-    gtk_clist_column_titles_hide (GTK_CLIST (tree));
-    gtk_clist_set_selection_mode (GTK_CLIST (tree), GTK_SELECTION_BROWSE);
-    gnc_account_tree_hide_all_but_name (GNC_ACCOUNT_TREE (tree));
-
-    gnc_account_tree_get_view_info (GNC_ACCOUNT_TREE (tree), &view_info);
-
-    for (type = 0; type < NUM_ACCOUNT_TYPES; type++)
-      view_info.include_type[type] = (type == INCOME);
-
-    gnc_account_tree_set_view_info (GNC_ACCOUNT_TREE (tree), &view_info);
+    gnc_tree_view_account_set_filter (GNC_TREE_VIEW_ACCOUNT (tree),
+				      gnc_stock_split_druid_view_filter_income,
+				      NULL, /* user data */
+				      NULL  /* destroy callback */);
 
     gtk_widget_show (tree);
 
+    label = glade_xml_get_widget (xml, "income_label");
+    gtk_label_set_mnemonic_widget (GTK_LABEL(label), tree);
+
     scroll = glade_xml_get_widget (xml, "income_scroll");
     gtk_container_add (GTK_CONTAINER (scroll), tree);
 
 
     /* asset tree */
-    tree = gnc_account_tree_new ();
+    tree = GTK_WIDGET(gnc_tree_view_account_new (FALSE));
     info->asset_tree = tree;
-    gtk_clist_column_titles_hide (GTK_CLIST (tree));
-    gtk_clist_set_selection_mode (GTK_CLIST (tree), GTK_SELECTION_BROWSE);
-    gnc_account_tree_hide_all_but_name (GNC_ACCOUNT_TREE (tree));
-
-    gnc_account_tree_get_view_info (GNC_ACCOUNT_TREE (tree), &view_info);
-
-    for (type = 0; type < NUM_ACCOUNT_TYPES; type++)
-      view_info.include_type[type] =
-        (type == BANK) || (type == CASH) || (type == ASSET);
-
-    gnc_account_tree_set_view_info (GNC_ACCOUNT_TREE (tree), &view_info);
+    gnc_tree_view_account_set_filter (GNC_TREE_VIEW_ACCOUNT (tree),
+				      gnc_stock_split_druid_view_filter_asset,
+				      NULL /* user data */,
+				      NULL /* destroy callback */);
 
     gtk_widget_show (tree);
 
+    label = glade_xml_get_widget (xml, "asset_label");
+    gtk_label_set_mnemonic_widget (GTK_LABEL(label), tree);
+
     scroll = glade_xml_get_widget (xml, "asset_scroll");
     gtk_container_add (GTK_CONTAINER (scroll), tree);
-
-    page = glade_xml_get_widget (xml, "cash_page");
-
-    gtk_signal_connect (GTK_OBJECT (page), "prepare",
-                        GTK_SIGNAL_FUNC (cash_prepare), info);
-
-    gtk_signal_connect (GTK_OBJECT (page), "next",
-                        GTK_SIGNAL_FUNC (cash_next), info);
   }
-
-  page = glade_xml_get_widget (xml, "finish_page");
-
-  gtk_signal_connect (GTK_OBJECT (page), "finish",
-                      GTK_SIGNAL_FUNC (stock_split_finish), info);
 }
 
 static void
@@ -666,24 +694,18 @@ refresh_handler (GHashTable *changes, gpointer user_data)
 {
   StockSplitInfo *info = user_data;
   Account *old_account;
-  Account *new_account;
-  GNCIdType id_type;
   GtkWidget *page;
   GladeXML *xml;
 
-  id_type = xaccGUIDType (&info->account, gnc_get_current_book ());
-  old_account = xaccAccountLookup (&info->account, gnc_get_current_book ());
+  old_account = info->acct;
 
-  if (fill_account_list (info, old_account) == 0)
+  if (fill_account_list (info, info->acct) == 0)
   {
     gnc_close_gui_component_by_data (DRUID_STOCK_SPLIT_CM_CLASS, info);
     return;
   }
 
-  new_account = xaccAccountLookup (&info->account, gnc_get_current_book ());
-
-  if (!safe_strcmp (id_type, GNC_ID_NULL) || old_account == new_account)
-    return;
+  if (NULL == info->acct || old_account == info->acct) return;
 
   xml = glade_get_widget_tree (info->window);
   page = glade_xml_get_widget (xml, "account_page");
@@ -703,18 +725,19 @@ close_handler (gpointer user_data)
  * gnc_stock_split_dialog                                           *
  *   opens up a window to record a stock split                      *
  *                                                                  * 
- * Args:   initial - the initial account to use                     *
+ * Args:   parent  - the parent ofthis window                       *
+ *         initial - the initial account to use                     *
  * Return: nothing                                                  *
 \********************************************************************/
 void
-gnc_stock_split_dialog (Account * initial)
+gnc_stock_split_dialog (GtkWidget *parent, Account * initial)
 {
   StockSplitInfo *info;
   gint component_id;
 
   info = g_new0 (StockSplitInfo, 1);
 
-  info->account = *xaccGUIDNULL ();
+  info->acct = NULL;
 
   gnc_stock_split_druid_create (info);
 
@@ -724,16 +747,15 @@ gnc_stock_split_dialog (Account * initial)
 
   gnc_gui_component_watch_entity_type (component_id,
                                        GNC_ID_ACCOUNT,
-                                       GNC_EVENT_MODIFY | GNC_EVENT_DESTROY);
+                                       QOF_EVENT_MODIFY | QOF_EVENT_DESTROY);
 
   if (fill_account_list (info, initial) == 0)
   {
-    gnc_warning_dialog (_("You don't have any stock accounts with balances!"));
+    gnc_warning_dialog (parent, _("You don't have any stock accounts with balances!"));
     gnc_close_gui_component_by_data (DRUID_STOCK_SPLIT_CM_CLASS, info);
     return;
   }
 
-  gnome_window_icon_set_from_default(GTK_WINDOW(info->window));
   gtk_widget_show_all (info->window);
 
   gnc_window_adjust_for_screen (GTK_WINDOW(info->window));
