@@ -3,6 +3,7 @@
  * Copyright (C) 2002 Christian Stimming                            *
  * Copyright (C) 2004 Bernd Wagner (changes for                     *
  *                     online transaction templates)                *
+ * Copyright (c) 2006 David Hampton <hampton@employees.org>         *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -18,8 +19,8 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -27,6 +28,7 @@
 #endif
 
 #include <gnome.h>
+#include <glib/gi18n.h>
 #include <aqbanking/version.h>
 #include <aqbanking/account.h>
 #include <aqbanking/jobsingletransfer.h>
@@ -41,6 +43,7 @@
 #endif
 #include <iconv.h>
 
+#include "gtk-compat.h"
 #include "dialog-utils.h"
 #include "gnc-ui.h"
 #include "gnc-amount-edit.h"
@@ -53,6 +56,14 @@
 #  include <ktoblzcheck.h>
 #endif
 
+#define TEMPLATE_LABEL "template"
+
+typedef enum {
+	TEMPLATE_NAME,
+	TEMPLATE_POINTER,
+	TEMPLATE_NUM_COLUMNS
+} TemplateListColumns;
+
 /* -------------------------------------- */
 /* Data structure */
 /* -------------------------------------- */
@@ -82,14 +93,9 @@ struct _trans_data
   /* Recipient's bank name (may be filled in automatically sometime later) */
   GtkWidget *recp_bankname_label;
 
-  /* The template choosing GtkList */
-  GtkWidget *template_gtk_list;
-
-  /* The selected template in the list */
-  GtkWidget *selected_template;
-  
-  /* GList of GNCTransTempl */
-  GList *templ;
+  /* The template choosing GtkTreeView/GtkListStore */
+  GtkTreeView *template_gtktreeview;
+  GtkListStore *template_list_store;
 
   /* Flag, if template list has been changed */
   gboolean templ_changed;
@@ -119,18 +125,37 @@ void gnc_hbci_dialog_delete(HBCITransDialog *td)
   if (td->hbci_trans)
     AB_Transaction_free (td->hbci_trans);
 
-  td->selected_template = NULL;
-    
-  gtk_widget_destroy (GTK_WIDGET (td->dialog));
+  if (td->dialog)
+    gtk_widget_destroy (GTK_WIDGET (td->dialog));
 #if HAVE_KTOBLZCHECK_H
   AccountNumberCheck_delete(td->blzcheck);
 #endif    
 }
 
+static gboolean
+get_templ_helper (GtkTreeModel *model,
+		  GtkTreePath *path,
+		  GtkTreeIter *iter,
+		  gpointer data)
+{
+  GList **list = data;
+  GNCTransTempl *templ;
+
+  gtk_tree_model_get (model, iter,
+		      TEMPLATE_POINTER, &templ,
+		      -1);
+  *list = g_list_append(*list, templ);
+  return FALSE; /* continue */
+}
+
 GList *gnc_hbci_dialog_get_templ(const HBCITransDialog *td)
 {
+  GList *list = NULL;
+
   g_assert(td);
-  return td->templ;
+  gtk_tree_model_foreach(GTK_TREE_MODEL(td->template_list_store),
+			 get_templ_helper, &list);
+ return list;
 }
 GtkWidget *gnc_hbci_dialog_get_parent(const HBCITransDialog *td)
 {
@@ -174,9 +199,7 @@ gboolean
 check_ktoblzcheck(GtkWidget *parent, const HBCITransDialog *td, 
 		  const AB_TRANSACTION *trans);
 
-void on_template_list_select_child(GtkList  *list, GtkWidget  *widget, gpointer  user_data);
-void on_template_list_selection_changed(GtkList *list, gpointer  user_data);
-void on_template_list_unselect_child(GtkList  *list, GtkWidget  *widget, gpointer  user_data);
+void on_template_list_selection_changed(GtkTreeSelection *selection, gpointer user_data);
 
 void template_selection_cb(GtkButton *b, gpointer user_data);
 void add_template_cb(GtkButton *b, gpointer user_data);
@@ -202,17 +225,17 @@ void blz_changed_cb(GtkEditable *e, gpointer user_data);
 static void fill_template_list_func(gpointer data, gpointer user_data)
 {
   GNCTransTempl *templ = data;
-  GtkList *list = user_data;
-  GtkWidget *item;
-
+  GtkListStore *list_store = user_data;
+  GtkTreeIter iter;
+  
   g_assert(templ);
-  g_assert(list);
+  g_assert(list_store);
   
-  item = gtk_list_item_new_with_label(gnc_trans_templ_get_name(templ));
-  g_assert(item);
-  
-  gtk_object_set_user_data(GTK_OBJECT(item), templ);
-  gtk_container_add(GTK_CONTAINER(list), item );
+  gtk_list_store_append(list_store, &iter);
+  gtk_list_store_set(list_store, &iter,
+		     TEMPLATE_NAME, gnc_trans_templ_get_name(templ),
+		     TEMPLATE_POINTER, templ,
+		     -1);
 }
 
 HBCITransDialog *
@@ -225,11 +248,13 @@ gnc_hbci_dialog_new (GtkWidget *parent,
   GladeXML *xml;
   const char *hbci_bankid, *hbci_bankname;
   HBCITransDialog *td;
+  GtkTreeSelection *selection;
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
 
   td = g_new0(HBCITransDialog, 1);
   
   td->parent = parent;
-  td->templ = templates;
   td->trans_type = trans_type;
   g_assert (h_acc);
   hbci_bankid = AB_Account_GetBankCode(h_acc);
@@ -243,7 +268,8 @@ gnc_hbci_dialog_new (GtkWidget *parent,
   td->dialog = glade_xml_get_widget (xml, "HBCI_trans_dialog");
 
   if (parent)
-    gnome_dialog_set_parent (GNOME_DIALOG (td->dialog), GTK_WINDOW (parent));
+    gtk_window_set_transient_for (GTK_WINDOW (td->dialog), 
+				  GTK_WINDOW (parent));
   
   {
     GtkWidget *heading_label;
@@ -311,7 +337,7 @@ gnc_hbci_dialog_new (GtkWidget *parent,
     g_assert
       ((exec_later_button = glade_xml_get_widget (xml, "exec_later_button")) != NULL);
     g_assert
-      ((td->template_gtk_list = glade_xml_get_widget (xml, "template_list")) != NULL);
+      ((td->template_gtktreeview = GTK_TREE_VIEW(glade_xml_get_widget (xml, "template_list"))) != NULL);
     g_assert
       ((add_templ_button = glade_xml_get_widget (xml, "add_templ_button")) != NULL);
     g_assert
@@ -366,7 +392,7 @@ gnc_hbci_dialog_new (GtkWidget *parent,
     }
     
     /* Make this button insensitive since it's still unimplemented. */
-    gtk_widget_set_sensitive (GTK_WIDGET (exec_later_button), FALSE);
+    gtk_widget_destroy (exec_later_button);
     
     /* Fill in the values from the objects */
     gtk_label_set_text (GTK_LABEL (orig_name_label), 
@@ -381,51 +407,49 @@ gnc_hbci_dialog_new (GtkWidget *parent,
 			hbci_bankid);
 
     /* fill list for choosing a transaction template */
-    g_list_foreach(td->templ, fill_template_list_func, 
-		    GTK_LIST (td->template_gtk_list));
+     gtk_tree_view_set_headers_visible(td->template_gtktreeview, FALSE);
+    td->template_list_store = gtk_list_store_new(TEMPLATE_NUM_COLUMNS,
+						 G_TYPE_STRING,
+						 G_TYPE_POINTER);
+    gtk_tree_view_set_model(td->template_gtktreeview,
+			    GTK_TREE_MODEL(td->template_list_store));
+    g_list_foreach(templates, fill_template_list_func, td->template_list_store);
+
+    renderer = gtk_cell_renderer_text_new();
+    column = gtk_tree_view_column_new_with_attributes ("Template Name",
+                                                      renderer,
+                                                      "text", TEMPLATE_NAME,
+                                                      NULL);
+    gtk_tree_view_append_column(td->template_gtktreeview, column);
 
-    td->selected_template = NULL;
     td->templ_changed = FALSE;
     
     /* Connect signals */
-/*    gnc_option_menu_init_w_signal (td->template_option, 
-				   GTK_SIGNAL_FUNC(template_selection_cb),
-				   td);   */
-    gtk_signal_connect (GTK_OBJECT (td->template_gtk_list), "select_child",
-                      GTK_SIGNAL_FUNC (on_template_list_select_child),
+    selection = gtk_tree_view_get_selection(td->template_gtktreeview);
+    g_signal_connect (selection, "changed",
+                      G_CALLBACK (on_template_list_selection_changed),
                       td);
                       
-    gtk_signal_connect(GTK_OBJECT (add_templ_button), "clicked",
-		       GTK_SIGNAL_FUNC(add_template_cb), td);
-
-    gtk_signal_connect (GTK_OBJECT (moveup_templ_button), "clicked",
-                      GTK_SIGNAL_FUNC (moveup_template_cb),
+    g_signal_connect (add_templ_button, "clicked",
+		      G_CALLBACK(add_template_cb), td);
+    g_signal_connect (moveup_templ_button, "clicked",
+                      G_CALLBACK (moveup_template_cb),
                       td);
                       
-    gtk_signal_connect (GTK_OBJECT (movedown_templ_button), "clicked",
-                      GTK_SIGNAL_FUNC (movedown_template_cb),
+    g_signal_connect (movedown_templ_button, "clicked",
+                      G_CALLBACK (movedown_template_cb),
                       td);
 
-     gtk_signal_connect (GTK_OBJECT (sort_templ_button), "clicked",
-                      GTK_SIGNAL_FUNC (sort_template_cb),
+     g_signal_connect (sort_templ_button, "clicked",
+                      G_CALLBACK (sort_template_cb),
                       td);
 
-     gtk_signal_connect (GTK_OBJECT (del_templ_button), "clicked",
-                      GTK_SIGNAL_FUNC (del_template_cb),
+     g_signal_connect (del_templ_button, "clicked",
+                      G_CALLBACK (del_template_cb),
                       td);
 
-
-    gtk_signal_connect(GTK_OBJECT (td->recp_bankcode_entry), "changed",
-		       GTK_SIGNAL_FUNC(blz_changed_cb), td);
-
-    /* Default button */
-    gnome_dialog_set_default (GNOME_DIALOG (td->dialog), 0);
-    
-    gtk_widget_grab_focus (td->recp_name_entry);
-
-    /* Hide on close instead of destroy since we still need the values
-       from the boxes. */
-    gnome_dialog_close_hides (GNOME_DIALOG (td->dialog), TRUE);
+    g_signal_connect (td->recp_bankcode_entry, "changed",
+		      G_CALLBACK(blz_changed_cb), td);
 
   } /* GtkWidget declarations/definitions */
   
@@ -440,7 +464,7 @@ gnc_hbci_dialog_new (GtkWidget *parent,
 int gnc_hbci_dialog_run_until_ok(HBCITransDialog *td, 
 				 const AB_ACCOUNT *h_acc)
 {
-  int result;
+  gint result;
   int max_purpose_lines;
   gboolean values_ok;
 
@@ -448,7 +472,7 @@ int gnc_hbci_dialog_run_until_ok(HBCITransDialog *td,
     AB_JOB *job = AB_JobSingleTransfer_new((AB_ACCOUNT *)h_acc);
     if (AB_Job_CheckAvailability(job)) {
       printf("gnc_hbci_trans_dialog_enqueue: Oops, job not available. Aborting.\n");
-      return -1;
+      return GTK_RESPONSE_CANCEL;
     }
 #if ((AQBANKING_VERSION_MAJOR > 1) || \
      ((AQBANKING_VERSION_MAJOR == 1) && \
@@ -479,16 +503,21 @@ int gnc_hbci_dialog_run_until_ok(HBCITransDialog *td,
     gtk_widget_show_all (td->dialog); 
 
     /* Now run the dialog until it gets closed by a button press. */
-    result = gnome_dialog_run (GNOME_DIALOG (td->dialog));
+    result = gtk_dialog_run (GTK_DIALOG (td->dialog));
     /* printf("hbci_trans: result button was %d.\n", result); */
 
     /* The dialog gets hidden anyway as soon as any button is pressed. */
     gtk_widget_hide_all (td->dialog);
 
-    /* Was cancel pressed or dialog closed? 0 == execute now, 1 ==
-       scheduled for later execution (currently unimplemented) */
-    if ((result != 0) && (result != 1)) {
-      return -1;
+    /* Was cancel pressed or dialog closed?
+     *  GNC_RESPONSE_NOW == execute now
+     *  GNC_RESPONSE_LATER == scheduled for later execution (currently unimplemented)
+     *  GTK_RESPONSE_CANCEL == cancel
+     *  GTK_RESPONSE_DELETE_EVENT == window destroyed */
+    if ((result != GNC_RESPONSE_NOW) && (result != GNC_RESPONSE_LATER)) {
+      gtk_widget_destroy (GTK_WIDGET (td->dialog));
+      td->dialog = NULL;
+      return GTK_RESPONSE_CANCEL;
     }
 
     /* Now fill in the values from the entry fields into a new
@@ -500,19 +529,20 @@ int gnc_hbci_dialog_run_until_ok(HBCITransDialog *td,
       AB_VALUE_toReadableString (AB_TRANSACTION_value (trans)));*/
     if (AB_Value_GetValue (AB_Transaction_GetValue (td->hbci_trans)) == 0.0) {
       gtk_widget_show_all (td->dialog); 
-      values_ok = !gnc_verify_dialog_parented
-	(GTK_WIDGET (td->dialog),
+      values_ok = !gnc_verify_dialog
+	(td->dialog,
 	 TRUE,
 	 "%s",
-	 _("The amount is zero or the amount field could not be \n"
-	   "interpreted correctly. You might have mixed up decimal \n"
-	   "point and comma, compared to your locale settings. \n"
+	 _("The amount is zero or the amount field could not be "
+	   "interpreted correctly. You might have mixed up decimal "
+	   "point and comma, compared to your locale settings. "
+	   "This does not result in a valid online transfer job. \n"
 	   "\n"
-	   "This does not result in a valid online transfer job.\n"
 	   "Do you want to enter the job again?"));
       if (values_ok) {
 	AB_Transaction_free (td->hbci_trans);
-	return -1;
+	td->hbci_trans = NULL;
+	return GTK_RESPONSE_CANCEL;
       }
       continue;
     } /* check Transaction_value */
@@ -521,17 +551,18 @@ int gnc_hbci_dialog_run_until_ok(HBCITransDialog *td,
       char *purpose = gnc_hbci_getpurpose (td->hbci_trans);
       if (strlen(purpose) == 0) {
 	gtk_widget_show_all (td->dialog); 
-	values_ok = !gnc_verify_dialog_parented
+	values_ok = !gnc_verify_dialog
 	  (GTK_WIDGET (td->dialog),
 	   TRUE,
 	   "%s",
-	   _("You did not enter any transaction purpose. A purpose is \n"
+	   _("You did not enter any transaction purpose. A purpose is "
 	     "required for an online transfer.\n"
 	     "\n"
 	     "Do you want to enter the job again?"));
 	if (values_ok) {
 	  AB_Transaction_free (td->hbci_trans);
-	  return -1;
+	  td->hbci_trans = NULL;
+	  return GTK_RESPONSE_CANCEL;
 	}
 	continue;
       } /* check Transaction_purpose */
@@ -560,13 +591,14 @@ hbci_trans_fill_values(const AB_ACCOUNT *h_acc, HBCITransDialog *td)
   AB_TRANSACTION *trans = AB_Transaction_new();
   gchar *tmpchar;
 
-  /* FIXME: The internal source encoding is hard-coded so far. This
-     needs to be fixed for the gnome2 version; the source encoding is
-     then probably utf-8 as well. iconv is also used in
+  /* The internal source encoding is returned by
+     gnc_hbci_book_encoding(), which is hard-coded so far. This needs
+     to be fixed for the gnome2 version; the source encoding is then
+     probably utf-8 as well. iconv is also used in
      gnc_AB_BANKING_interactors() in hbci-interaction.c. */
-  iconv_t gnc_iconv_handler =
-    iconv_open(gnc_hbci_AQBANKING_encoding(), gnc_hbci_book_encoding());
-  g_assert(gnc_iconv_handler != (iconv_t)(-1));
+  GIConv gnc_iconv_handler =
+    g_iconv_open(gnc_hbci_AQBANKING_encoding(), gnc_hbci_book_encoding());
+  g_assert(gnc_iconv_handler != (GIConv)(-1));
 	
   /* OpenHBCI newer than 0.9.8: use account's bankCode values
    * instead of the bank's ones since this is what some banks
@@ -632,7 +664,7 @@ hbci_trans_fill_values(const AB_ACCOUNT *h_acc, HBCITransDialog *td)
     AB_Transaction_SetTextKey (trans, 51);
   }
 
-  iconv_close(gnc_iconv_handler);
+  g_iconv_close(gnc_iconv_handler);
   return trans;
 }
 
@@ -655,12 +687,12 @@ check_ktoblzcheck(GtkWidget *parent, const HBCITransDialog *td,
   switch (blzresult) {
   case 2:
     gtk_widget_show_all (parent); 
-    values_ok = gnc_verify_dialog_parented
+    values_ok = gnc_verify_dialog
       (parent,
        TRUE,
-       _("The internal check of the destination account number '%s' \n"
-	 "at the specified bank with bank code '%s' failed. This means \n"
-	 "the account number might contain an error. Should the online \n"
+       _("The internal check of the destination account number '%s' "
+	 "at the specified bank with bank code '%s' failed. This means "
+	 "the account number might contain an error. Should the online "
 	 "transfer job be sent with this account number anyway?"),
        AB_Transaction_GetRemoteAccountNumber (trans),
        AB_Transaction_GetRemoteBankCode (trans));
@@ -760,14 +792,14 @@ gnc_hbci_trans_dialog_execute(HBCITransDialog *td, AB_BANKING *api,
     /* AB_BANKING_executeOutbox failed. */
     if ((AB_Job_GetStatus (job) == AB_Job_StatusPending) ||
 	(AB_Job_GetStatus (job) == AB_Job_StatusError)) 
-      successful = !gnc_verify_dialog_parented
+      successful = !gnc_verify_dialog
 	(td->parent, 
 	 FALSE,
 	 "%s",
-	 _("The job was sent to the bank successfully, but the \n"
-	   "bank is refusing to execute the job. Please check \n"
-	   "the log window for the exact error message of the \n"
-	   "bank. The line with the error message contains a \n"
+	 _("The job was sent to the bank successfully, but the "
+	   "bank is refusing to execute the job. Please check "
+	   "the log window for the exact error message of the "
+	   "bank. The line with the error message contains a "
 	   "code number that is greater than 9000.\n"
 	   "\n"
 	   "Do you want to enter the job again?"));
@@ -794,16 +826,19 @@ static void fill_entry(const char *str, GtkWidget *entry) {
 
 
 void
-on_template_list_select_child          (GtkList         *list,
-                                        GtkWidget       *widget,
-                                        gpointer         user_data)
+on_template_list_selection_changed (GtkTreeSelection *selection,
+				    gpointer          user_data)
 {
   HBCITransDialog *td = user_data;
-  GNCTransTempl *templ = gtk_object_get_user_data (GTK_OBJECT(widget)) ;
+  GNCTransTempl *templ;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
 
   g_assert(td);
 
-  td->selected_template = widget;
+  if (!gtk_tree_selection_get_selected(selection, &model, &iter))
+    return;
+  gtk_tree_model_get(model, &iter, TEMPLATE_POINTER, &templ, -1);
 
       fill_entry(gnc_trans_templ_get_recp_name(templ), td->recp_name_entry);
       fill_entry(gnc_trans_templ_get_recp_account(templ), td->recp_account_entry);
@@ -817,28 +852,6 @@ on_template_list_select_child          (GtkList         *list,
 }
 
 
-void
-on_template_list_selection_changed     (GtkList         *list,
-                                        gpointer         user_data)
-{
-
-}
-
-
-void
-on_template_list_unselect_child        (GtkList         *list,
-                                        GtkWidget       *widget,
-                                        gpointer         user_data)
-{
-  HBCITransDialog *td = user_data;
-  g_assert(td);
-
-  td->selected_template = NULL;
-
-}
-
-
-
 void blz_changed_cb(GtkEditable *e, gpointer user_data)
 {
 #if HAVE_KTOBLZCHECK_H
@@ -852,8 +865,19 @@ void blz_changed_cb(GtkEditable *e, gpointer user_data)
   
   if (record) {
     const char *bankname = AccountNumberCheck_Record_bankName (record);
+    GError *error = NULL;
+    gchar *utf8_bankname = g_convert (bankname, strlen(bankname), 
+				      "UTF-8", "ISO-8859-15",
+				      NULL, NULL, &error);
+    if (error != NULL) {
+      printf ("Error convertion bankname \"%s\" to UTF-8\n", bankname);
+      g_error_free (error);
+      utf8_bankname = g_strdup (bankname);
+    }
     gtk_label_set_text (GTK_LABEL (td->recp_bankname_label),
-			(strlen(bankname)>0 ? bankname : _("(unknown)")));
+			(strlen(utf8_bankname)>0 ? 
+			 utf8_bankname : _("(unknown)")));
+    g_free (utf8_bankname);
     gtk_widget_show_all (td->recp_bankname_label);
 
     /*printf("blz_changed_cb: KtoBlzCheck said check is bank is '%s' at '%s'.\n",
@@ -870,68 +894,62 @@ void blz_changed_cb(GtkEditable *e, gpointer user_data)
 
 /* -------------------------------------- */
 /* -------------------------------------- */
-/* Copied from window-help.c */
-static void
-goto_string_cb(char * string, gpointer data)
-{
-  if(!data) return;
-  if(!string) {
-    *(char **)data = NULL;
-  }
-  else {
-    *(char **)data = g_strdup(string);
-  }
-}
+
 void add_template_cb(GtkButton *b,
 		     gpointer user_data)
 {
   HBCITransDialog *td = user_data;
   GtkWidget *dlg;
-  char *name;
+  const gchar *name;
   int retval = -1;
-  GNCTransTempl *t;
-  gint index;
+  GladeXML *xml;
+  GtkTreeSelection *selection;
+  GtkTreeModel *model;
+  GtkTreeIter cur_iter, new_iter;
+  GtkWidget *entry;
+
   g_assert(td);
 
-  dlg = gnome_request_dialog(FALSE,
-			     _("Enter name for new template:"), "", 250,
-			     &goto_string_cb, &name, GTK_WINDOW(td->parent));
-  retval = gnome_dialog_run_and_close(GNOME_DIALOG(dlg));
+  xml = gnc_glade_xml_new ("hbci.glade", "HBCI_template_name_dialog");
 
-  if ((retval == 0) && name && (strlen(name) > 0)) {
-    GNCTransTempl *r;
-    /*printf("add_template_cb: adding template '%s'\n", name);*/
-    r = gnc_trans_templ_new_full
-      (name, 
-       gtk_entry_get_text (GTK_ENTRY (td->recp_name_entry)),
-       gtk_entry_get_text (GTK_ENTRY (td->recp_account_entry)),
-       gtk_entry_get_text (GTK_ENTRY (td->recp_bankcode_entry)),
-       gnc_amount_edit_get_amount (GNC_AMOUNT_EDIT (td->amount_edit)),
-       gtk_entry_get_text (GTK_ENTRY (td->purpose_entry)),
-       gtk_entry_get_text (GTK_ENTRY (td->purpose_cont_entry)));
+  dlg = glade_xml_get_widget (xml, "HBCI_template_name_dialog");
+  entry = glade_xml_get_widget (xml, "template_name");
+  gtk_entry_set_text(GTK_ENTRY(entry),
+		     gtk_entry_get_text(GTK_ENTRY(td->recp_name_entry)));
 
-  if (td->selected_template) {
-    t = gtk_object_get_user_data(GTK_OBJECT(td->selected_template));
+  retval = gtk_dialog_run(GTK_DIALOG(dlg));
 
-    index = 1+gtk_list_child_position(GTK_LIST(td->template_gtk_list), td->selected_template);
+  if (retval == GTK_RESPONSE_OK) {
+    name = gtk_entry_get_text(GTK_ENTRY(entry));
+
+    if (name && (strlen(name) > 0)) {
+      GNCTransTempl *r;
+      /*printf("add_template_cb: adding template '%s'\n", name);*/
+      r = gnc_trans_templ_new_full
+	(name, 
+	 gtk_entry_get_text (GTK_ENTRY (td->recp_name_entry)),
+	 gtk_entry_get_text (GTK_ENTRY (td->recp_account_entry)),
+	 gtk_entry_get_text (GTK_ENTRY (td->recp_bankcode_entry)),
+	 gnc_amount_edit_get_amount (GNC_AMOUNT_EDIT (td->amount_edit)),
+	 gtk_entry_get_text (GTK_ENTRY (td->purpose_entry)),
+	 gtk_entry_get_text (GTK_ENTRY (td->purpose_cont_entry)));
+
+      selection = gtk_tree_view_get_selection(td->template_gtktreeview);
+      if (gtk_tree_selection_get_selected(selection, &model, &cur_iter)) {
+	gtk_list_store_insert_after(td->template_list_store,
+				    &new_iter, &cur_iter);
+      } else {
+	gtk_list_store_append(GTK_LIST_STORE(model), &new_iter);
+      }
+      gtk_list_store_set(td->template_list_store, &new_iter,
+			 TEMPLATE_NAME, name,
+			 TEMPLATE_POINTER, r,
+			 -1);
+      td->templ_changed = TRUE;
     }
-  else index = 0;
-  
-  td->templ = g_list_insert(td->templ, r, index);
-    
-  td->templ_changed = TRUE;
-
-  gtk_list_clear_items(GTK_LIST(td->template_gtk_list), 0, -1);
-
-  /* fill list for choosing a transaction template */
-  g_list_foreach(td->templ, fill_template_list_func,
-		    GTK_LIST (td->template_gtk_list));
-
-  gtk_list_select_item(GTK_LIST(td->template_gtk_list), index);
-
-  /* the show_all is necessary since otherwise the new item doesn't show up */
-  gtk_widget_show_all (GTK_WIDGET ( GTK_LIST (td->template_gtk_list)));
   }
+  gtk_widget_destroy(dlg);
+  //  g_object_unref(xml);
 }
 
 
@@ -940,31 +958,26 @@ moveup_template_cb(GtkButton       *button,
                   gpointer         user_data)
 {
   HBCITransDialog *td = user_data;
-  GNCTransTempl *t;
-  gint index;
+  GtkTreeSelection *selection;
+  GtkTreeModel *model;
+  GtkTreeIter iter, prev_iter;
+  GtkTreePath *prev_path;
+
   g_assert(td);
 
-  if (td->selected_template) {
-    t = gtk_object_get_user_data(GTK_OBJECT(td->selected_template));
-
-    index = gtk_list_child_position(GTK_LIST(td->template_gtk_list), td->selected_template);
-
-    if (index > 0) {
-      td->templ =  g_list_remove( td->templ, t);
-      td->templ =  g_list_insert( td->templ, t, index-1);
+  selection = gtk_tree_view_get_selection(td->template_gtktreeview);
+  if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+    return;
 
+  prev_path = gtk_tree_model_get_path(model, &iter);
+  if (gtk_tree_path_prev(prev_path)) {
+    if (gtk_tree_model_get_iter(model, &prev_iter, prev_path)) {
+      gtk_list_store_move_before(GTK_LIST_STORE(model), &iter, &prev_iter);
       td->templ_changed = TRUE;
-      gtk_list_clear_items(GTK_LIST(td->template_gtk_list), 0, -1);
-
-      /* fill list for choosing a transaction template */
-      g_list_foreach(td->templ, fill_template_list_func,
-		    GTK_LIST (td->template_gtk_list));
-
-      gtk_list_select_item(GTK_LIST(td->template_gtk_list), index-1);
-
-      gtk_widget_show_all (GTK_WIDGET ( GTK_LIST (td->template_gtk_list)));
-      }
     }
+  }
+
+  gtk_tree_path_free(prev_path);
 }
 
 
@@ -973,41 +986,23 @@ movedown_template_cb(GtkButton       *button,
                     gpointer         user_data)
 {
   HBCITransDialog *td = user_data;
-  GNCTransTempl *t;
-  gint index;
+  GtkTreeSelection *selection;
+  GtkTreeModel *model;
+  GtkTreeIter iter, next_iter;
+
   g_assert(td);
 
-  if (td->selected_template) {
-    t = gtk_object_get_user_data(GTK_OBJECT(td->selected_template));
+  selection = gtk_tree_view_get_selection(td->template_gtktreeview);
+  if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+    return;
 
-    index = gtk_list_child_position(GTK_LIST(td->template_gtk_list), td->selected_template);
-
-    if (index < g_list_length(td->templ)-1) {
-      td->templ =  g_list_remove( td->templ, t);
-      td->templ =  g_list_insert( td->templ, t, index+1);
-
-      td->templ_changed = TRUE;
-      gtk_list_clear_items(GTK_LIST(td->template_gtk_list), 0, -1);
-
-      /* fill list for choosing a transaction template */
-      g_list_foreach(td->templ, fill_template_list_func,
-		    GTK_LIST (td->template_gtk_list));
-
-      gtk_list_select_item(GTK_LIST(td->template_gtk_list), index+1);
-
-      gtk_widget_show_all (GTK_WIDGET ( GTK_LIST (td->template_gtk_list)));
-      }
-    }
+  next_iter = iter;
+  if (gtk_tree_model_iter_next(model, &next_iter)) {
+    gtk_list_store_move_after(GTK_LIST_STORE(model), &iter, &next_iter);
+    td->templ_changed = TRUE;
+  }
 }
 
-static gint comparefunc(const gconstpointer e1,
-                 const gconstpointer e2)
-{
-  return g_strcasecmp(gnc_trans_templ_get_name((GNCTransTempl*)e1),
-        gnc_trans_templ_get_name((GNCTransTempl*)e2));
-  
-}  
-                 
 void
 sort_template_cb(GtkButton       *button,
                  gpointer         user_data)
@@ -1015,23 +1010,14 @@ sort_template_cb(GtkButton       *button,
   HBCITransDialog *td = user_data;
   g_assert(td);
 
-  if (gnc_verify_dialog_parented (td->parent,
-       FALSE, "%s", _("Do you really want to sort the list of templates?"))) {
-
-    td->templ =  g_list_sort( td->templ, comparefunc);
-  
-    td->templ_changed = TRUE;
-
-    gtk_list_clear_items(GTK_LIST(td->template_gtk_list), 0, -1);
-
-    /* fill list for choosing a transaction template */
-    g_list_foreach(td->templ, fill_template_list_func,
-		    GTK_LIST (td->template_gtk_list));
-
-    gtk_list_unselect_all ( GTK_LIST (td->template_gtk_list) );
-   
-    gtk_widget_show_all (GTK_WIDGET ( GTK_LIST (td->template_gtk_list)));
-  }
+  gtk_tree_sortable_set_sort_column_id
+    (GTK_TREE_SORTABLE(td->template_list_store),
+     TEMPLATE_NAME, GTK_SORT_ASCENDING);
+  gtk_tree_sortable_set_sort_column_id
+    (GTK_TREE_SORTABLE(td->template_list_store),
+     GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
+     GTK_SORT_ASCENDING);
+  td->templ_changed = TRUE;
 }
 
 
@@ -1041,32 +1027,18 @@ del_template_cb(GtkButton       *button,
                gpointer         user_data)
 {
   HBCITransDialog *td = user_data;
-  GNCTransTempl *t;
-  gint index;
+  GtkTreeSelection *selection;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+
   g_assert(td);
 
-  if (td->selected_template) {
+  selection = gtk_tree_view_get_selection(td->template_gtktreeview);
+  if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+    return;
 
-    t = gtk_object_get_user_data(GTK_OBJECT(td->selected_template));
-
-    index = gtk_list_child_position(GTK_LIST(td->template_gtk_list), td->selected_template);
-
-    if (gnc_verify_dialog_parented (td->parent,
-          FALSE, _("Do you really want to delete the template '%s'?"),
-          gnc_trans_templ_get_name(g_list_nth_data(td->templ, index)))) {
-      gtk_list_clear_items(GTK_LIST(td->template_gtk_list), index, index+1);
-            
-      td->templ =  g_list_remove( td->templ, t);
-      td->templ_changed = TRUE;
-
-      gnc_trans_templ_delete(t);
-
-      gtk_list_unselect_all ( GTK_LIST (td->template_gtk_list) );
-
-      gtk_widget_show_all (GTK_WIDGET ( GTK_LIST (td->template_gtk_list)));
-    
-      }
-    }
+  gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+  td->templ_changed = TRUE;
 }
 
 
@@ -1090,4 +1062,3 @@ void gnc_hbci_dialog_xfer_cb(Transaction *trans, gpointer user_data)
   }
   return;
 }
-
