@@ -16,13 +16,14 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include "config.h"
 
 #include <gnome.h>
+#include <glib/gi18n.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -40,20 +41,16 @@
 #include "gnc-ui-util.h"
 #include "gnc-ui.h"
 #include "gnc-html.h"
+//#include "import-account-matcher.h"
 #include "gnc-component-manager.h"
 
 #include <aqbanking/banking.h>
+#include <aqbanking/version.h>
 #include <gwenhywfar/stringlist.h>
+#include <gwenhywfar/version.h>
 
 /* #define DEFAULT_HBCI_VERSION 201 */
 
-typedef enum _infostate {
-  INI_UPDATE_ACCOUNTS,
-  INI_MATCH_ACCOUNTS,
-  UPDATE_ACCOUNTS,
-  MATCH_ACCOUNTS
-} Infostate;
-
 struct _hbciinitialinfo 
 {
   GtkWidget *window;
@@ -75,9 +72,6 @@ struct _hbciinitialinfo
   /* hbci_account (direct) -> gnucash_account  -- DO NOT DELETE THE KEYS! */
   GHashTable *gnc_hash;
 
-  /* Status of user's movement through the wizard */
-  Infostate state;
-
 };
 
 static gboolean
@@ -174,8 +168,7 @@ update_accountlist_acc_cb (AB_ACCOUNT *hacc, gpointer user_data)
   if (gacc == NULL)
     row_text[1] = "";
   else 
-    row_text[1] = 
-      xaccAccountGetFullName (gacc, gnc_get_account_separator ());
+    row_text[1] = xaccAccountGetFullName (gacc);
 
   /* Add this row to the list */
   row = gtk_clist_append (GTK_CLIST (info->accountlist), row_text);
@@ -215,7 +208,6 @@ update_accountlist (HBCIInitialInfo *info)
   /* Delete old hash with row_number -> hbci_account */
   delete_hash (info->hbci_hash);
   info->hbci_hash = g_hash_table_new (&g_int_hash, &g_int_equal);
-  g_hash_table_freeze (info->hbci_hash);
   
   /* Go through all HBCI accounts */
   acclist = AB_Banking_GetAccounts(banking);
@@ -229,7 +221,6 @@ update_accountlist (HBCIInitialInfo *info)
   /* printf("update_accountlist: HBCI hash has %d entries.\n", g_hash_table_size(info->hbci_hash)); */
   /* printf("update_accountlist: GNC hash has %d entries.\n", g_hash_table_size(info->gnc_hash)); */
   
-  g_hash_table_thaw (info->hbci_hash);
   gtk_clist_thaw (GTK_CLIST (info->accountlist));
 
   /* move to the old selected row */
@@ -248,14 +239,14 @@ druid_enable_next_button(HBCIInitialInfo *info)
 {
   g_assert(info);
   gnome_druid_set_buttons_sensitive (GNOME_DRUID (info->druid),
-				     TRUE, TRUE, TRUE);
+				     TRUE, TRUE, TRUE, TRUE);
 }
 static void 
 druid_disable_next_button(HBCIInitialInfo *info)
 {
   g_assert(info);
   gnome_druid_set_buttons_sensitive (GNOME_DRUID (info->druid),
-				     TRUE, FALSE, TRUE);
+				     TRUE, FALSE, TRUE, TRUE);
 }
 /*
  * end button enabling
@@ -267,7 +258,6 @@ druid_disable_next_button(HBCIInitialInfo *info)
  * GUI callbacks
  */
 
-
 static gboolean banking_has_accounts(AB_BANKING *banking)
 {
   AB_ACCOUNT_LIST2 *accl;
@@ -332,25 +322,6 @@ on_aqbutton_prepare (GnomeDruidPage *gnomedruidpage,
 }
 
 
-static gboolean 
-on_accountlist_back (GnomeDruidPage  *gnomedruidpage,
-		     gpointer         arg1,
-		     gpointer         user_data)
-{
-  HBCIInitialInfo *info = user_data;
-  g_assert(info);
-  
-  switch (info->state) {
-  case INI_MATCH_ACCOUNTS:
-  case MATCH_ACCOUNTS:
-    gnome_druid_set_page (GNOME_DRUID (info->druid), 
-			  GNOME_DRUID_PAGE (info->filepage));
-    return TRUE;
-  default:
-    return FALSE;
-  }
-}
-
 static void
 on_accountlist_prepare (GnomeDruidPage *gnomedruidpage,
 			gpointer arg1,
@@ -366,7 +337,7 @@ on_accountlist_prepare (GnomeDruidPage *gnomedruidpage,
     info->gnc_hash = gnc_hbci_new_hash_from_kvp (info->api);
   
   gnome_druid_set_buttons_sensitive (GNOME_DRUID (info->druid),
-				     FALSE, TRUE, TRUE);
+				     FALSE, TRUE, TRUE, TRUE);
 
   update_accountlist(info);
 }
@@ -396,10 +367,11 @@ on_accountlist_select_row (GtkCList *clist, gint row,
 	(strlen(AB_Account_GetCurrency (hbci_acc)) > 0)) {
       currency = gnc_commodity_table_lookup 
 	(gnc_book_get_commodity_table (gnc_get_current_book ()), 
-	 GNC_COMMODITY_NS_ISO, AB_Account_GetCurrency (hbci_acc));
+	 GNC_COMMODITY_NS_CURRENCY, AB_Account_GetCurrency (hbci_acc));
     }
 
-    gnc_acc = gnc_import_select_account(NULL, TRUE, longname, currency, BANK,
+    gnc_acc = gnc_import_select_account(info->window,
+					NULL, TRUE, longname, currency, BANK,
 					old_value, NULL);
     g_free(longname);
 
@@ -421,28 +393,17 @@ on_accountlist_select_row (GtkCList *clist, gint row,
 
 
 
-static void
-on_button_clicked (GtkButton *button,
-		   gpointer user_data)
-{
-  HBCIInitialInfo *info = user_data;
-  char *name;
-  g_assert(info->druid);
-  
-  name = gtk_widget_get_name (GTK_WIDGET (button));
-  if (strcmp (name, "aqhbci_button") == 0) {
-/*     info->state = ADD_BANK; */
-    /* gnome_druid_set_page (GNOME_DRUID (info->druid), 
-       GNOME_DRUID_PAGE (info->bankpage)); */
-  } else if (strcmp (name, "updatelist_button") == 0) {
-    info->state = UPDATE_ACCOUNTS;
-    /* Nothing else to do. */
-  } else {
-    printf("on_button_clicked: Oops, unknown button: %s\n",
-	   name);
-  }
-}
-
+#if (AQBANKING_VERSION_MAJOR > 1) || \
+  ((AQBANKING_VERSION_MAJOR == 1) && \
+    (AQBANKING_VERSION_MINOR == 9) && \
+     (AQBANKING_VERSION_PATCHLEVEL > 0))
+/* The wizard choice has changed with aqbanking >= 1.9.1; we don't
+   need to specify a "backend" for the wizard anymore but instead
+   there is only one wizard for all backends now. */
+# define AQBANKING_WIZARD_ALLBACKENDS
+#else
+# undef AQBANKING_WIZARD_ALLBACKENDS
+#endif
 
 static void
 on_aqhbci_button (GtkButton *button,
@@ -451,8 +412,6 @@ on_aqhbci_button (GtkButton *button,
   HBCIInitialInfo *info = user_data;
   GWEN_BUFFER *buf;
   int res;
-  GWEN_PLUGIN_DESCRIPTION_LIST2 *pluginlist;
-  const char *backend_name_nc;
   char *backend_name;
 
   /* This is the point where we look for and start an external
@@ -469,6 +428,12 @@ on_aqhbci_button (GtkButton *button,
   AB_BANKING *banking = info->api;
   g_assert(info->druid);
 
+  /* See note above about wizard choice. */
+#ifndef AQBANKING_WIZARD_ALLBACKENDS
+  {
+  GWEN_PLUGIN_DESCRIPTION_LIST2 *pluginlist;
+  const char *backend_name_nc;
+
   /* Get list of all backends, active or inactive */
   pluginlist = AB_Banking_GetProviderDescrs (banking);
 
@@ -501,10 +466,11 @@ on_aqhbci_button (GtkButton *button,
       }
       GWEN_PluginDescription_List2Iterator_free(pluginlist_it);
 
-      x = gnc_choose_radio_option_dialog_parented 
+      x = gnc_choose_radio_option_dialog
 	(GTK_WIDGET(info->window),
 	 _("Choose AqBanking Backend"),
 	 _("Please choose an AqBanking backend to be configured"),
+	 _("Co_nfigure"),
 	 0,
 	 radio_list);
       g_list_free(radio_list);
@@ -512,7 +478,13 @@ on_aqhbci_button (GtkButton *button,
       /* User pressed cancel in choice dialog */
       if (x == -1) {
 	GWEN_PluginDescription_List2_freeAll(pluginlist);
+#if ((GWENHYWFAR_VERSION_MAJOR < 1) || \
+     ((GWENHYWFAR_VERSION_MAJOR == 1) && \
+      ((GWENHYWFAR_VERSION_MINOR < 98))))
+	/* Memory cleanup needed for gwenhywfar<1.98.x but not for
+	   gwenhywfar>=1.98.x */
 	GWEN_PluginDescription_List2_free(pluginlist);
+#endif
 	return;
       }
 
@@ -531,7 +503,18 @@ on_aqhbci_button (GtkButton *button,
      be freed */
   backend_name = g_strdup (backend_name_nc);
   GWEN_PluginDescription_List2_freeAll (pluginlist);
+#if ((GWENHYWFAR_VERSION_MAJOR < 1) || \
+     ((GWENHYWFAR_VERSION_MAJOR == 1) && \
+      ((GWENHYWFAR_VERSION_MINOR < 98))))
+  /* Memory cleanup needed for gwenhywfar<1.98.x but not for
+     gwenhywfar>=1.98.x */
   GWEN_PluginDescription_List2_free (pluginlist);
+#endif
+  }
+
+#else /* AQBANKING_WIZARD_ALLBACKENDS */
+  backend_name = g_strdup("");
+#endif /* AQBANKING_WIZARD_ALLBACKENDS */
 
   /* ***** */
 
@@ -559,8 +542,14 @@ on_aqhbci_button (GtkButton *button,
     /* Call the qt wizard. See the note above about why this approach
        is chosen. */
 
-    /* In gtk2, this would be g_spawn_async or similar. */
+    /* Reset existing mapping tables */
     AB_Banking_Fini (info->api);
+    delete_hash (info->hbci_hash);
+    info->hbci_hash = NULL;
+    if (info->gnc_hash != NULL)
+      g_hash_table_destroy (info->gnc_hash);
+    info->gnc_hash = NULL;
+    /* In gtk2, this would be g_spawn_async or similar. */
     {
       pid_t pid;
       pid = fork();
@@ -590,7 +579,9 @@ on_aqhbci_button (GtkButton *button,
     }
 
     if (res == 0) {
+#ifndef AQBANKING_WIZARD_ALLBACKENDS
       res = AB_Banking_ActivateProvider(banking, backend_name);
+#endif
       if ((res == 0) || (res == AB_ERROR_FOUND))
 	druid_enable_next_button(info);
       else {
@@ -600,24 +591,24 @@ on_aqhbci_button (GtkButton *button,
     }
     else {
       printf("on_aqhbci_button: Oops, aqhbci wizard return nonzero value: %d. The called program was \"%s\".\n", res, wizard_path);
-      gnc_error_dialog_parented
-	(GTK_WINDOW(info->window),
+      gnc_error_dialog
+	(info->window,
        /* Each of the %s is the name of the backend, e.g. "aqhbci". */
-	 _("The external program \"%s Setup Wizard\" returned a nonzero \n"
-	   "exit code which means it has not been finished successfully. \n"
-	   "The further HBCI setup can only be finished if the %s \n"
-	   "Setup Wizard is run successfully. Please try to start and \n"
+	 _("The external program \"%s Setup Wizard\" returned a nonzero "
+	   "exit code which means it has not been finished successfully. "
+	   "The further HBCI setup can only be finished if the %s "
+	   "Setup Wizard is run successfully. Please try to start and "
 	   "successfully finish the %s Setup Wizard program again."),
 	 backend_name, backend_name, backend_name);
       druid_disable_next_button(info);
     }
   } else {
     printf("on_aqhbci_button: Oops, no aqhbci setup wizard found.");
-    gnc_error_dialog_parented
-      (GTK_WINDOW(info->window), 
+    gnc_error_dialog
+      (info->window,
        /* Each of the %s is the name of the backend, e.g. "aqhbci". */
        _("The external program \"%s Setup Wizard\" has not been found. \n\n"
-	 "The package aqbanking is supposed to install the program \n"
+	 "The package aqbanking is supposed to install the program "
 	 "\"%s-qt3-wizard\". Please check your installation of aqbanking."),
        backend_name, backend_name);
     druid_disable_next_button(info);
@@ -646,9 +637,9 @@ void gnc_hbci_initial_druid (void)
   gnc_druid_set_colors (GNOME_DRUID (info->druid));
   
   glade_xml_signal_connect_data (xml, "on_finish", 
-				 GTK_SIGNAL_FUNC (on_finish), info);
+				 G_CALLBACK (on_finish), info);
   glade_xml_signal_connect_data (xml, "on_cancel", 
-				 GTK_SIGNAL_FUNC (on_cancel), info);
+				 G_CALLBACK (on_cancel), info);
   
   info->api = gnc_AB_BANKING_new_currentbook(info->window, &(info->interactor));
   g_assert(info->api);
@@ -657,43 +648,25 @@ void gnc_hbci_initial_druid (void)
     /* Page with config file entry widget */
     page = glade_xml_get_widget(xml, "configfile_page");
     info->filepage = page;
-    gtk_signal_connect (GTK_OBJECT (page), "prepare",
-			GTK_SIGNAL_FUNC (on_aqbutton_prepare), info);
-    gtk_signal_connect (GTK_OBJECT 
-			(glade_xml_get_widget (xml, "aqhbci_button")), 
-			"clicked",
-			GTK_SIGNAL_FUNC (on_aqhbci_button), info);
+    g_signal_connect (page, "prepare",
+		      G_CALLBACK (on_aqbutton_prepare), info);
+    g_signal_connect (glade_xml_get_widget (xml, "aqhbci_button"),
+		      "clicked",
+		      G_CALLBACK (on_aqhbci_button), info);
   }
   {
     page = glade_xml_get_widget(xml, "account_match_page");
     info->accountpage = page;
     info->accountlist = glade_xml_get_widget(xml, "account_page_list");
-    gtk_signal_connect (GTK_OBJECT (info->accountlist), "select_row",
-			GTK_SIGNAL_FUNC (on_accountlist_select_row), info);
-    gtk_signal_connect (GTK_OBJECT 
-			(glade_xml_get_widget (xml, "aqhbci_again_button")), 
-			"clicked",
-			GTK_SIGNAL_FUNC (on_aqhbci_button), info);
-    gtk_signal_connect (GTK_OBJECT 
-			(glade_xml_get_widget (xml, "updatelist_button")), 
-			"clicked",
-			GTK_SIGNAL_FUNC (on_button_clicked), info);
-    gtk_signal_connect (GTK_OBJECT (page), "prepare", 
-			GTK_SIGNAL_FUNC (on_accountlist_prepare), info);
-    gtk_signal_connect (GTK_OBJECT (page), "back", 
-			GTK_SIGNAL_FUNC (on_accountlist_back), info);
+    g_signal_connect (info->accountlist, "select_row",
+		      G_CALLBACK (on_accountlist_select_row), info);
+    g_signal_connect (page, "prepare", 
+		      G_CALLBACK (on_accountlist_prepare), info);
   }
 
 
-  /*gtk_signal_connect (GTK_OBJECT(dialog), "destroy",*/
-  /*                   GTK_SIGNAL_FUNC(gnc_hierarchy_destroy_cb), NULL);*/
+  /*g_signal_connect (dialog, "destroy",*/
+  /*                  G_CALLBACK(gnc_hierarchy_destroy_cb), NULL);*/
 
   gtk_widget_show_all (info->window);
-  
-}
-
-SCM  scm_hbci_initial_druid (void)
-{
-  gnc_hbci_initial_druid();
-  return SCM_EOL;
 }
