@@ -15,35 +15,36 @@
  *                                                                         *
  *   You should have received a copy of the GNU Lesser General Public      *
  *   License along with this library; if not, write to the Free Software   *
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
- *   MA  02111-1307  USA                                                   *
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,            *
+ *   MA  02110-1301  USA                                                   *
  *                                                                         *
  ***************************************************************************/
 
 #include "config.h"
 
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <locale.h>
+#include <iconv.h>
+#include <aqbanking/banking.h>
+#include <gwenhywfar/bio_buffer.h>
+#include <gwenhywfar/xml.h>
+
 #include "hbci-interaction.h"
 #include "hbci-interactionP.h"
 
-#include <aqbanking/banking.h>
 #include "dialog-utils.h"
 #include "druid-utils.h"
 #include "gnc-ui-util.h"
 #include "gnc-ui.h"
-#include "global-options.h"
+#include "gnc-gconf-utils.h"
 
 #include "dialog-pass.h"
 #include "gnc-hbci-utils.h"
 
-#include <gwenhywfar/bio_buffer.h>
-#include <gwenhywfar/xml.h>
-
-#include <iconv.h>
-
-#define PREF_TAB_ONLINE_BANKING N_("Online Banking & Importing")
+#define GCONF_SECTION_CONNECTION GCONF_SECTION "/connection_dialog"
 
 gchar *gnc__extractText(const char *text);
 
@@ -59,15 +60,14 @@ GNCInteractor *gnc_AB_BANKING_interactors (AB_BANKING *api, GtkWidget *parent)
      then probably utf-8 as well. iconv is also used in
      gnc_hbci_descr_tognc() in gnc-hbci-utils.c. */
   data->gnc_iconv_handler = 
-    iconv_open(gnc_hbci_book_encoding(), gnc_hbci_AQBANKING_encoding());
-  g_assert(data->gnc_iconv_handler != (iconv_t)(-1));
+    g_iconv_open(gnc_hbci_book_encoding(), gnc_hbci_AQBANKING_encoding());
+  g_assert(data->gnc_iconv_handler != (GIConv)(-1));
   data->keepAlive = TRUE;
   data->cache_pin = 
-    gnc_lookup_boolean_option(PREF_TAB_ONLINE_BANKING,
-			      "HBCI Remember PIN in memory",
-                              FALSE);
+    gnc_gconf_get_bool(GCONF_SECTION, KEY_REMEMBER_PIN, NULL);
   data->showbox_id = 1;
   data->showbox_hash = g_hash_table_new(NULL, NULL); 
+  data->min_loglevel = AB_Banking_LogLevelVerbous;
 
   /* set HBCI_Interactor */
   gnc_hbci_add_callbacks(api, data);
@@ -79,17 +79,19 @@ void GNCInteractor_delete(GNCInteractor *data)
   if (data == NULL)
     return;
   if (data->dialog != NULL) {
-    gnc_set_boolean_option ("__gui", "hbci_close_on_finish",
-			    gtk_toggle_button_get_active 
-			    (GTK_TOGGLE_BUTTON (data->close_checkbutton)));
-    gtk_object_unref (GTK_OBJECT (data->dialog));
+    gnc_gconf_set_bool(GCONF_SECTION, KEY_CLOSE_ON_FINISH,
+		       gtk_toggle_button_get_active 
+		       (GTK_TOGGLE_BUTTON (data->close_checkbutton)),
+		       NULL);
+    gnc_save_window_size(GCONF_SECTION_CONNECTION, GTK_WINDOW (data->dialog));
+    g_object_unref (G_OBJECT (data->dialog));
     gtk_widget_destroy (data->dialog);
   }
   
   data->dialog = NULL;
 
   g_hash_table_destroy(data->showbox_hash);
-  iconv_close(data->gnc_iconv_handler);
+  g_iconv_close(data->gnc_iconv_handler);
 
   g_free (data);
 }
@@ -143,9 +145,7 @@ gboolean GNCInteractor_aborted(const GNCInteractor *i)
 void GNCInteractor_show_nodelete(GNCInteractor *i)
 {
   gboolean cache_pin = 
-    gnc_lookup_boolean_option(PREF_TAB_ONLINE_BANKING,
-			      "HBCI Remember PIN in memory",
-			      FALSE);
+    gnc_gconf_get_bool(GCONF_SECTION, KEY_REMEMBER_PIN, NULL);
   g_assert(i);
   /* Show widgets */
   gtk_widget_show_all (i->dialog);
@@ -163,7 +163,9 @@ void GNCInteractor_show(GNCInteractor *i)
   g_assert(i);
   GNCInteractor_show_nodelete(i);
   /* Clear log window. */
-  gtk_editable_delete_text (GTK_EDITABLE (i->log_text), 0, -1);
+  gtk_text_buffer_set_text
+    (gtk_text_view_get_buffer (GTK_TEXT_VIEW (i->log_text) ),
+     "", 0);
 }
 
 
@@ -173,9 +175,11 @@ void GNCInteractor_hide(GNCInteractor *i)
   if (gtk_toggle_button_get_active 
       (GTK_TOGGLE_BUTTON (i->close_checkbutton)))
     gtk_widget_hide_all (i->dialog);
-  gnc_set_boolean_option ("__gui", "hbci_close_on_finish",
-			  gtk_toggle_button_get_active 
-			  (GTK_TOGGLE_BUTTON (i->close_checkbutton)));
+  gnc_gconf_set_bool(GCONF_SECTION, KEY_CLOSE_ON_FINISH,
+		     gtk_toggle_button_get_active 
+		     (GTK_TOGGLE_BUTTON (i->close_checkbutton)),
+		     NULL);
+  gnc_save_window_size(GCONF_SECTION_CONNECTION, GTK_WINDOW (i->dialog));
 }
 
 gboolean GNCInteractor_get_cache_valid(const GNCInteractor *i)
@@ -204,7 +208,7 @@ void GNCInteractor_reparent (GNCInteractor *i, GtkWidget *new_parent)
 	gtk_widget_reparent (GTK_WIDGET (i->dialog), new_parent);
 	else
 	gtk_widget_set_parent (GTK_WIDGET (i->dialog), new_parent);*/
-      gnome_dialog_set_parent (GNOME_DIALOG (i->dialog), 
+      gtk_window_set_transient_for (GTK_WINDOW (i->dialog), 
 			       GTK_WINDOW (new_parent));
     }
 }
@@ -212,7 +216,17 @@ void GNCInteractor_reparent (GNCInteractor *i, GtkWidget *new_parent)
 gboolean GNCInteractor_hadErrors (const GNCInteractor *i)
 {
   g_assert (i);
-  return i->msgBoxError != 0;
+  return (i->msgBoxError != 0);
+}
+
+gboolean GNCInteractor_errorsLogged (const GNCInteractor *i)
+{
+  g_assert (i);
+  /* Note: Unfortunately this does not mean at all that there actually
+     has been any error. Old aqbanking versions had some debugging
+     messages set at "error" level, and there can also be errors when
+     closing connection that don't affect the job result at all. */
+  return (i->min_loglevel < AB_Banking_LogLevelNotice);
 }
 
 /* ************************************************************ 
@@ -337,13 +351,28 @@ static int inputBoxCB(AB_BANKING *ab,
     if (!retval)
       break;
     
+    g_assert(passwd);
     if (strlen(passwd) < (unsigned int)minsize) {
       gboolean retval;
       char *msg = 
-	g_strdup_printf (  _("The PIN needs to be at least %d characters \n"
+	g_strdup_printf (  _("The PIN needs to be at least %d characters "
 			     "long. Do you want to try again?"),
 			   minsize);
-      retval = gnc_verify_dialog_parented (GTK_WIDGET (data->parent), 
+      retval = gnc_verify_dialog (GTK_WIDGET (data->parent), 
+					   TRUE,
+					   msg);
+      g_free (msg);
+      if (!retval)
+	break;
+    }
+    else if (strlen(passwd) >= (unsigned int)maxLen) {
+      gboolean retval;
+      char *msg = 
+	g_strdup_printf (  _("You entered %ld characters, but the PIN must "
+			     "be no longer than %d characters. "
+			     "Do you want to try again?"),
+			   (long)strlen(passwd), maxLen);
+      retval = gnc_verify_dialog (GTK_WIDGET (data->parent), 
 					   TRUE,
 					   msg);
       g_free (msg);
@@ -351,7 +380,7 @@ static int inputBoxCB(AB_BANKING *ab,
 	break;
     }
     else {
-      g_assert (maxLen > strlen(passwd));
+      g_assert (maxLen > strlen(passwd)); /* assertion was queried above */
       strcpy(resultbuffer, passwd);
       g_free (memset (passwd, 0, strlen (passwd)));
       g_free(title);
@@ -407,10 +436,24 @@ static int getTanCB(AB_BANKING *ab,
     if (strlen(passwd) < (unsigned int)minsize) {
       gboolean retval;
       char *msg = 
-	g_strdup_printf (  _("This TAN needs to be at least %d characters \n"
+	g_strdup_printf (  _("This TAN needs to be at least %d characters "
 			     "long. Do you want to try again?"),
 			   minsize);
-      retval = gnc_verify_dialog_parented (GTK_WIDGET (data->parent), 
+      retval = gnc_verify_dialog (GTK_WIDGET (data->parent), 
+					   TRUE,
+					   msg);
+      g_free (msg);
+      if (!retval)
+	break;
+    }
+    else if (strlen(passwd) >= (unsigned int)maxLen) {
+      gboolean retval;
+      char *msg = 
+	g_strdup_printf (  _("You entered %ld characters, but the TAN must "
+			     "be no longer than %d characters. "
+			     "Do you want to try again?"),
+			   (long)strlen(passwd), maxLen);
+      retval = gnc_verify_dialog (GTK_WIDGET (data->parent), 
 					   TRUE,
 					   msg);
       g_free (msg);
@@ -418,7 +461,7 @@ static int getTanCB(AB_BANKING *ab,
 	break;
     }
     else {
-      g_assert (maxLen > strlen(passwd));
+      g_assert (maxLen > strlen(passwd)); /* assertion was queried above */
       strcpy(resultbuffer, passwd);
 
       g_free (memset (passwd, 0, strlen (passwd)));
@@ -441,11 +484,14 @@ static int getTanCB(AB_BANKING *ab,
 static int keepAlive(void *user_data)
 {
   GNCInteractor *data = user_data;
+  GMainContext *context;
+
   g_assert(data);
   /*fprintf(stdout, "my-keepAlive: returning 1\n");*/
 
   /* Let the widgets be redrawn */
-  while (g_main_iteration (FALSE));
+  context = g_main_context_default();
+  while (g_main_context_iteration(context, FALSE));
 
   return data->keepAlive;
 }
@@ -474,14 +520,14 @@ hideBoxCB(AB_BANKING *ab, GWEN_TYPE_UINT32 id)
   g_assert(data);
 
   if (id > 0) {
-    dialog = g_hash_table_lookup(data->showbox_hash, (gpointer)id);
+    dialog = g_hash_table_lookup(data->showbox_hash, GUINT_TO_POINTER(id));
   } else {
     dialog = data->showbox_last;
   }
   if (dialog) {
-    gnome_dialog_close (GNOME_DIALOG (dialog));
+    gtk_widget_hide (dialog);
     gtk_widget_destroy (dialog);
-    g_hash_table_remove(data->showbox_hash, (gpointer)id);
+    g_hash_table_remove(data->showbox_hash, GUINT_TO_POINTER(id));
   }
 }
 
@@ -503,14 +549,21 @@ showBoxCB(AB_BANKING *ab, GWEN_TYPE_UINT32 flags,
   title = gnc_hbci_utf8ToLatin1(data, utf8title);
 
   /* Create new dialog */
-  dialog = gnome_ok_dialog_parented (text, GTK_WINDOW (data->parent));
+  dialog = gtk_message_dialog_new(GTK_WINDOW(data->parent),
+				  0,
+				  GTK_MESSAGE_INFO,
+				  GTK_BUTTONS_OK,
+				  "%s", text);
+
   if (title && (strlen(title) > 0))
     gtk_window_set_title (GTK_WINDOW (dialog), title);
-  gnome_dialog_close_hides (GNOME_DIALOG(dialog), TRUE);
+
+  g_signal_connect(G_OBJECT(dialog), "response",
+		   (GCallback)gtk_widget_hide, NULL);
   gtk_widget_show_all (dialog);
 
   result = data->showbox_id;
-  g_hash_table_insert(data->showbox_hash, (gpointer)result, dialog);
+  g_hash_table_insert(data->showbox_hash, GUINT_TO_POINTER(result), dialog);
   data->showbox_id++;
   data->showbox_last = dialog;
 
@@ -542,34 +595,35 @@ static int messageBoxCB(AB_BANKING *ab, GWEN_TYPE_UINT32 flags,
   b2text = gnc_hbci_utf8ToLatin1(data, b2);
   b3text = gnc_hbci_utf8ToLatin1(data, b3);
 
-  dialog = gnome_dialog_new (title, 
-			     b1 ? b1text : NULL,
-			     b2 ? b2text : NULL,
-			     b3 ? b3text : NULL,
-			     NULL);
-  gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (data->parent));
-  gnome_dialog_set_close (GNOME_DIALOG (dialog), TRUE);
+  dialog = gtk_dialog_new_with_buttons (title, 
+					GTK_WINDOW (data->parent),
+					GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					b1 ? b1text : NULL,
+					1,
+					b2 ? b2text : NULL,
+					2,
+					b3 ? b3text : NULL,
+					3,
+					NULL);
+  /* Add the label, and show everything we've added to the dialog. */
   label = gtk_label_new (text);
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label, TRUE, TRUE, 0);
-  gtk_widget_show (label);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
+		     label);
+  gtk_widget_show_all (dialog);
 
-  result = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
-  if (result<0 || result>2) {
+  result = gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
+  if (result<1 || result>3) {
     printf("messageBoxCB: Bad result %d", result);
-    g_free(title);
-    g_free(text);
-    g_free(b1text);
-    g_free(b2text);
-    g_free(b3text);
-    return 0;
+    result = 0;
   }
   g_free(title);
   g_free(text);
   g_free(b1text);
   g_free(b2text);
   g_free(b3text);
-  return result+1;
+  return result;
 }
 
 
@@ -599,8 +653,8 @@ static GWEN_TYPE_UINT32 progressStartCB(AB_BANKING *ab, const char *utf8title,
   /*   GNCInteractor_add_log_text (data, text); */
 
   /* Set progress bar */
-  gtk_progress_set_percentage (GTK_PROGRESS (data->action_progress), 
-			       0.0);
+  gtk_widget_set_sensitive (data->action_progress, TRUE);
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(data->action_progress), 0.0);
   data->action_max = total;
   GNCInteractor_setRunning(data);
   /* printf("progressStartCB: Action \"%s\" started, total %d.\n",
@@ -609,6 +663,9 @@ static GWEN_TYPE_UINT32 progressStartCB(AB_BANKING *ab, const char *utf8title,
   /* Show the dialog */
   GNCInteractor_show(data);
 
+  /* Initialize loglevel caching */
+  data->min_loglevel = AB_Banking_LogLevelVerbous;
+
   g_free(title);
   g_free(text);
   return progress_id;
@@ -631,8 +688,8 @@ static int progressAdvanceCB(AB_BANKING *ab, GWEN_TYPE_UINT32 id,
     /* printf("progressLogCB: Progress set to %d out of %f.\n", 
        progress, data->action_max); */
     if (progress <= data->action_max) 
-      gtk_progress_set_percentage (GTK_PROGRESS (data->action_progress), 
-				   progress/data->action_max);
+      gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (data->action_progress),
+				     progress/data->action_max);
   }
 
   return !keepAlive(data);
@@ -658,6 +715,10 @@ static int progressLogCB(AB_BANKING *ab, GWEN_TYPE_UINT32 id,
   /* printf("progressLogCB: Logging msg: %s\n", text); */
   GNCInteractor_add_log_text (data, text);
 
+  /* Cache loglevel */
+  if (level < data->min_loglevel)
+    data->min_loglevel = level;
+
   g_free(text);
   return !keepAlive(data);
 }
@@ -687,16 +748,17 @@ static int progressEndCB(AB_BANKING *ab, GWEN_TYPE_UINT32 id)
 int debug_pmonitor = FALSE;
 void GNCInteractor_add_log_text (GNCInteractor *data, const char *msg)
 {
-  int pos;
+  GtkTextBuffer *tb;
+  GtkTextView *tv;
   g_assert(data);
-  
-  pos = gtk_text_get_length (GTK_TEXT (data->log_text));
-  gtk_editable_insert_text (GTK_EDITABLE (data->log_text),
-			    msg, strlen (msg),
-			    &pos);
-  gtk_editable_insert_text (GTK_EDITABLE (data->log_text),
-			    "\n", 1,
-			    &pos);
+
+  tv = GTK_TEXT_VIEW (data->log_text);
+  tb = gtk_text_view_get_buffer (tv);
+  gtk_text_buffer_insert_at_cursor (tb, msg, -1);
+  gtk_text_buffer_insert_at_cursor (tb, "\n", -1);
+  /* and scroll to the end of the buffer */
+  gtk_text_view_scroll_to_mark (tv, gtk_text_buffer_get_insert (tb),
+				0.0, FALSE, 0.0, 0.0);
 }
 
 static void
@@ -704,7 +766,8 @@ on_button_clicked (GtkButton *button,
 		   gpointer user_data)
 {
   GNCInteractor *data = user_data;
-  char *name;
+  GMainContext *context;
+  const char *name;
   g_assert(data);
   
   name = gtk_widget_get_name (GTK_WIDGET (button));
@@ -721,7 +784,8 @@ on_button_clicked (GtkButton *button,
 	   name);
   }
   /* Let the widgets be redrawn */
-  while (g_main_iteration (FALSE));
+  context = g_main_context_default();
+  while (g_main_context_iteration(context, FALSE));
 }
 
 GWEN_INHERIT(AB_BANKING, GNCInteractor)
@@ -738,35 +802,35 @@ gnc_hbci_add_callbacks(AB_BANKING *ab, GNCInteractor *data)
   /* Create the progress dialog window */
   xml = gnc_glade_xml_new ("hbci.glade", "HBCI_connection_dialog");
 
-  g_assert (dialog = glade_xml_get_widget (xml, "HBCI_connection_dialog"));
+  g_assert ((dialog = glade_xml_get_widget (xml, "HBCI_connection_dialog")) != NULL);
   data->dialog = dialog;
-  g_assert (data->job_entry = glade_xml_get_widget (xml, "job_entry"));
-  g_assert (data->action_entry = glade_xml_get_widget (xml, "action_entry"));
-  g_assert (data->action_progress = 
-	    glade_xml_get_widget (xml, "action_progress"));
-  g_assert (data->log_text = glade_xml_get_widget (xml, "log_text"));
-  g_assert (data->abort_button = glade_xml_get_widget (xml, "abort_button"));
+  g_assert ((data->job_entry = glade_xml_get_widget (xml, "job_entry")) != NULL);
+  g_assert ((data->action_entry = glade_xml_get_widget (xml, "action_entry")) != NULL);
+  g_assert ((data->action_progress = 
+	     glade_xml_get_widget (xml, "action_progress")) != NULL);
+  g_assert ((data->log_text = glade_xml_get_widget (xml, "log_text")) != NULL);
+  g_assert ((data->abort_button = glade_xml_get_widget (xml, "abort_button")) != NULL);
   gtk_widget_set_sensitive (GTK_WIDGET (data->abort_button), FALSE);
-  g_assert (data->close_button = glade_xml_get_widget (xml, "close_button"));
-  g_assert (data->close_checkbutton = 
-	    glade_xml_get_widget (xml, "close_checkbutton"));
+  g_assert ((data->close_button = glade_xml_get_widget (xml, "close_button")) != NULL);
+  g_assert ((data->close_checkbutton = 
+	     glade_xml_get_widget (xml, "close_checkbutton")) != NULL);
 
   /* grey out the progress bar -- its unused at the moment */
   gtk_widget_set_sensitive (data->action_progress, FALSE);
   gtk_toggle_button_set_active 
     (GTK_TOGGLE_BUTTON (data->close_checkbutton), 
-     gnc_lookup_boolean_option("__gui", "hbci_close_on_finish", TRUE));
+     gnc_gconf_get_bool(GCONF_SECTION, KEY_CLOSE_ON_FINISH, NULL));
 
-  gtk_signal_connect (GTK_OBJECT (data->abort_button), "clicked", 
-		      GTK_SIGNAL_FUNC (on_button_clicked), data);
-  gtk_signal_connect (GTK_OBJECT (data->close_button), "clicked", 
-		      GTK_SIGNAL_FUNC (on_button_clicked), data);
+  g_signal_connect (data->abort_button, "clicked", 
+		    G_CALLBACK (on_button_clicked), data);
+  g_signal_connect (data->close_button, "clicked", 
+		    G_CALLBACK (on_button_clicked), data);
 
   if (data->parent)
-    gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (data->parent));
-  /*gtk_widget_set_parent (GTK_WIDGET (dialog), data->parent);*/
+    gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (data->parent));
+  gnc_restore_window_size(GCONF_SECTION_CONNECTION, GTK_WINDOW (dialog));
 
-  gtk_object_ref (GTK_OBJECT (dialog));
+  g_object_ref (G_OBJECT (dialog));
   gtk_widget_hide_all (dialog);
 
   GWEN_INHERIT_SETDATA(AB_BANKING, GNCInteractor,
