@@ -13,8 +13,8 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 /** @addtogroup Import_Export
     @{ */
@@ -25,34 +25,29 @@
     @author Christian Stimming
 */
  
-#define _GNU_SOURCE
-
 #include "config.h"
 
-
-#include <glib.h>
-
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <stdlib.h> 
 #include <math.h>
+
+#include "gnc-gconf-utils.h"
 #include "import-backend.h"
 #include "import-utilities.h"
 #include "Account.h"
-#include "dialog-utils.h"
-#include "global-options.h"
 #include "Query.h"
-
-#include "gnc-engine-util.h"
-
+#include "gnc-engine.h"
 #include "gnc-ui-util.h"
 
-#define IMPORT_PAGE	"Online Banking & Importing" /* from app-utils/prefs.scm */
-#define BAYES_OPTION	"Use Bayesian Matching?"
+#define GCONF_SECTION "dialogs/import/generic_matcher"
+#define BAYES_OPTION  "use_bayes"
 
 /********************************************************************\
  *   Constants   *
 \********************************************************************/
 
-static short module = MOD_IMPORT;
+static QofLogModule log_module = GNC_MOD_IMPORT;
 
 /********************************************************************\
  *   Constants, should idealy be defined a user preference dialog    *
@@ -130,7 +125,7 @@ gboolean
 gnc_import_TransInfo_is_balanced (const GNCImportTransInfo *info)
 {
  g_assert (info);
- if(gnc_numeric_equal(xaccTransGetImbalance(gnc_import_TransInfo_get_trans(info)),gnc_numeric_zero()))
+ if(gnc_numeric_zero_p(xaccTransGetImbalance(gnc_import_TransInfo_get_trans(info))))
    {
      return TRUE;
    }
@@ -245,17 +240,17 @@ void gnc_import_TransInfo_delete (GNCImportTransInfo *info)
     /*If the transaction exists and is still open, it must be destroyed*/
     if(info->trans && xaccTransIsOpen(info->trans))
       {
-	xaccTransDestroy(info->trans);
-	xaccTransCommitEdit(info->trans);
+        xaccTransDestroy(info->trans);
+        xaccTransCommitEdit(info->trans);
       }
     if (info->match_tokens)
       {
-	GList *node;
+        GList *node;
 
-	for (node = info->match_tokens; node; node = node->next)
-	  g_free (node->data);
+        for (node = info->match_tokens; node; node = node->next)
+          g_free (node->data);
 
-	g_list_free (info->match_tokens);
+        g_list_free (info->match_tokens);
       }
     g_free(info);
   }
@@ -282,6 +277,7 @@ GdkPixmap* gen_probability_pixmap(gint score_original, GNCImportSettings *settin
   gchar * red_color_str = g_strdup_printf("r c red");
   gchar * black_color_str = g_strdup_printf("b c black");
   gchar * xpm[2+num_colors+height];
+  gint add_threshold, clear_threshold;
 
   g_assert(settings);
   g_assert(widget);
@@ -302,7 +298,9 @@ GdkPixmap* gen_probability_pixmap(gint score_original, GNCImportSettings *settin
   xpm[3]=yellow_color_str;
   xpm[4]=red_color_str; 
   xpm[5]=black_color_str;
-  
+  add_threshold = gnc_import_Settings_get_add_threshold(settings);
+  clear_threshold = gnc_import_Settings_get_clear_threshold(settings);
+
   for(i=0;i<height;i++)
     {
       xpm[num_colors+1+i]= g_new0(char,(width_each_bar*score)+width_first_bar+1);
@@ -325,11 +323,11 @@ GdkPixmap* gen_probability_pixmap(gint score_original, GNCImportSettings *settin
 		{
 		  strcat(xpm[num_colors+1+i],black_first_bar);
 		}
-	      else if (j<=gnc_import_Settings_get_add_threshold(settings))
+	      else if (j <= add_threshold)
 		{
 		  strcat(xpm[num_colors+1+i],red_bar);
 		}
-	      else if (j>=gnc_import_Settings_get_clear_threshold(settings))
+	      else if (j >= clear_threshold)
 		{
 		  strcat(xpm[num_colors+1+i],green_bar);
 		}
@@ -461,7 +459,7 @@ matchmap_find_destination (GncImportMatchMap *matchmap, GNCImportTransInfo *info
 	     (xaccSplitGetAccount
 	      (gnc_import_TransInfo_get_fsplit (info))));
 
-  useBayes = gnc_lookup_boolean_option(IMPORT_PAGE, BAYES_OPTION, TRUE);
+  useBayes = gnc_gconf_get_bool(GCONF_SECTION, BAYES_OPTION, NULL);
   if(useBayes)
     {
       /* get the tokens for this transaction* */
@@ -530,7 +528,7 @@ matchmap_store_destination (GncImportMatchMap *matchmap,
 		   (gnc_import_TransInfo_get_fsplit (trans_info))));
 
   /* see what matching system we are currently using */
-  useBayes = gnc_lookup_boolean_option(IMPORT_PAGE, BAYES_OPTION, TRUE);
+  useBayes = gnc_gconf_get_bool(GCONF_SECTION, BAYES_OPTION, NULL);
   if(useBayes)
     {
       /* tokenize this transaction */
@@ -586,6 +584,7 @@ static void split_find_match (GNCImportTransInfo * trans_info,
       double downloaded_split_amount, match_split_amount;
       time_t match_time, download_time;
       int datediff_day;
+      Transaction *new_trans = gnc_import_TransInfo_get_trans (trans_info);
     
       /* Matching heuristics */
     
@@ -596,9 +595,11 @@ static void split_find_match (GNCImportTransInfo * trans_info,
       /*DEBUG(" downloaded_split_amount=%f", downloaded_split_amount);*/
       match_split_amount = gnc_numeric_to_double(xaccSplitGetAmount(split));
       /*DEBUG(" match_split_amount=%f", match_split_amount);*/
-      if(gnc_numeric_equal(xaccSplitGetAmount
-			   (gnc_import_TransInfo_get_fsplit (trans_info)),
-			   xaccSplitGetAmount(split)))
+      if(downloaded_split_amount == match_split_amount)
+	/*if (gnc_numeric_equal(xaccSplitGetAmount
+	  (gnc_import_TransInfo_get_fsplit (trans_info)),
+	  xaccSplitGetAmount(split))) 
+	  -- gnc_numeric_equal is an expensive function call */
 	{
 	  prob = prob+3;
 	  /*DEBUG("heuristics:  probability + 3 (amount)");*/
@@ -623,8 +624,7 @@ static void split_find_match (GNCImportTransInfo * trans_info,
       
       /* Date heuristics */
       match_time = xaccTransGetDate (xaccSplitGetParent (split));
-      download_time = 
-	xaccTransGetDate (gnc_import_TransInfo_get_trans (trans_info));
+      download_time = xaccTransGetDate (new_trans);
       datediff_day = abs(match_time - download_time)/86400;
       /* Sorry, there are not really functions around at all that
 	 provide for less hacky calculation of days of date
@@ -655,10 +655,9 @@ static void split_find_match (GNCImportTransInfo * trans_info,
 	}
       
       /* Check number heuristics */  
-      if(strlen(xaccTransGetNum(gnc_import_TransInfo_get_trans (trans_info)))!=0)
+      if(strlen(xaccTransGetNum(new_trans))!=0)
 	{     
-	  if((strcmp(xaccTransGetNum
-		     (gnc_import_TransInfo_get_trans (trans_info)),
+	  if((strcmp(xaccTransGetNum (new_trans),
 		     xaccTransGetNum(xaccSplitGetParent(split)))
 	      ==0))
 	    {	
@@ -666,7 +665,7 @@ static void split_find_match (GNCImportTransInfo * trans_info,
 	      prob = prob+4;
 	      /*DEBUG("heuristics:  probability + 5 (Check number)");*/
 	    }
-	  else if(strlen(xaccTransGetNum(gnc_import_TransInfo_get_trans (trans_info))) > 0 &&
+	  else if(strlen(xaccTransGetNum(new_trans)) > 0 &&
 		  strlen(xaccTransGetNum(xaccSplitGetParent(split))) > 0)
 	    {
 	      /* If both number are not empty yet do not match, add a little extre penality */
@@ -700,10 +699,9 @@ static void split_find_match (GNCImportTransInfo * trans_info,
 	}
 
       /* Description heuristics */  
-      if(strlen(xaccTransGetDescription(gnc_import_TransInfo_get_trans (trans_info)))!=0)
+      if(strlen(xaccTransGetDescription(new_trans))!=0)
 	{
-	  if((strcmp(xaccTransGetDescription
-		     (gnc_import_TransInfo_get_trans (trans_info)),
+	  if((strcmp(xaccTransGetDescription (new_trans),
 		     xaccTransGetDescription(xaccSplitGetParent(split)))
 	      ==0))
 	    {	
@@ -711,11 +709,9 @@ static void split_find_match (GNCImportTransInfo * trans_info,
 	      prob = prob+2;
 	      /*DEBUG("heuristics:  probability + 2 (description)");*/
 	    }
-	  else if((strncmp(xaccTransGetDescription
-			   (gnc_import_TransInfo_get_trans (trans_info)),
-			   xaccTransGetDescription(xaccSplitGetParent(split)),
-			   strlen(xaccTransGetDescription
-				  (gnc_import_TransInfo_get_trans (trans_info)))/2)
+	  else if((strncmp(xaccTransGetDescription (new_trans),
+			   xaccTransGetDescription (xaccSplitGetParent(split)),
+			   strlen(xaccTransGetDescription (new_trans))/2)
 		   ==0))
 	    {
 	      /* Very primitive fuzzy match worth +1.  This matches the
@@ -787,11 +783,11 @@ void gnc_import_find_split_matches(GNCImportTransInfo *trans_info,
 
     xaccQuerySetBook (query, gnc_get_current_book());
     xaccQueryAddSingleAccountMatch (query, importaccount,			    
-				    QUERY_AND);
+				    QOF_QUERY_AND);
     xaccQueryAddDateMatchTT (query,
 			     TRUE, download_time - match_date_hardlimit*86400,
 			     TRUE, download_time + match_date_hardlimit*86400,
-			     QUERY_AND);
+			     QOF_QUERY_AND);
     list_element = xaccQueryGetSplits (query);
     /* Sigh. Doesnt help too much. We still create and run one query
        for each imported transaction. Maybe it would improve
@@ -892,27 +888,35 @@ gnc_import_process_trans_clist (GtkCList *clist,
 	  xaccTransCommitEdit 
 	    (gnc_import_TransInfo_get_trans (trans_info));
 	  break;
-	case GNCImport_CLEAR:
+	case GNCImport_CLEAR: {
+	  GNCImportMatchInfo *selected_match =
+	    gnc_import_TransInfo_get_selected_match (trans_info);
+
+	  /* If there is no selection, ignore this transaction. */
+	  if (!selected_match) {
+	    PWARN("No matching translaction to be cleared was chosen. Imported transaction will be ignored.");
+	    break;
+	  }
+
 	  /* Transaction gets not imported but the matching one gets 
 	     reconciled. */
-	  if(gnc_import_MatchInfo_get_split 
-	     (gnc_import_TransInfo_get_selected_match (trans_info)) ==NULL)
+	  if(gnc_import_MatchInfo_get_split (selected_match) ==NULL)
 	    {
-	      PERR("The split I am trying to reconcile is NULL, shouldn't happen!")
+                PERR("The split I am trying to reconcile is NULL, shouldn't happen!");
 	    }
 	  else
 	    {
 	      /* Reconcile the matching transaction */
 	      /*DEBUG("BeginEdit selected_match")*/
-	      xaccTransBeginEdit(trans_info->selected_match_info->trans);
+	      xaccTransBeginEdit(selected_match->trans);
 
 	      if (xaccSplitGetReconcile 
-		  (trans_info->selected_match_info->split) == NREC)
+		  (selected_match->split) == NREC)
 		xaccSplitSetReconcile
-		  (trans_info->selected_match_info->split, CREC);
+		  (selected_match->split, CREC);
 	      /* Set reconcile date to today */
 	      xaccSplitSetDateReconciledSecs
-		(trans_info->selected_match_info->split,time(NULL));
+		(selected_match->split,time(NULL));
 
 	      /* Copy the online id to the reconciled transaction, so
 		 the match will be remembered */ 
@@ -921,13 +925,13 @@ gnc_import_process_trans_clist (GtkCList *clist,
 		  (strlen (gnc_import_get_trans_online_id(trans_info->trans))
 		   > 0))
 		gnc_import_set_trans_online_id
-		  (trans_info->selected_match_info->trans, 
+		  (selected_match->trans, 
 		   gnc_import_get_trans_online_id(trans_info->trans));
 	      
 	      /* Done editing. */
 	      /*DEBUG("CommitEdit selected_match")*/
 	      xaccTransCommitEdit
-		(trans_info->selected_match_info->trans);
+		(selected_match->trans);
 	      
 	      /* Store the mapping to the other account in the MatchMap. */
 	      matchmap_store_destination (matchmap, trans_info, TRUE);
@@ -939,9 +943,10 @@ gnc_import_process_trans_clist (GtkCList *clist,
 	      /* Very important: Make sure the freed transaction is not freed again! */
 	      trans_info->trans = NULL;
 	    }
+	  }
 	  break;
 	case GNCImport_EDIT:
-	    PERR("EDIT action is UNSUPPORTED!")
+	    PERR("EDIT action is UNSUPPORTED!");
 	  break;
 	default:
 	  DEBUG("Invalid GNCImportAction for this imported transaction.");
@@ -969,10 +974,10 @@ gnc_import_process_trans_clist (GtkCList *clist,
 /********************************************************************\
  * check_trans_online_id() Callback function to be used by
  * gnc_import_exists_online_id.  Takes pointers to two transaction and
- * returns TRUE if their online_id kvp_frame do NOT match, or if both
+ * returns 0 if their online_id kvp_frame do NOT match, or if both
  * pointers point to the same transaction.
  * \********************************************************************/
-static gboolean check_trans_online_id(Transaction *trans1, void *user_data)
+static gint check_trans_online_id(Transaction *trans1, void *user_data)
 {
   Transaction *trans2 = user_data;
   const gchar *online_id1 = gnc_import_get_trans_online_id(trans1);
@@ -981,12 +986,12 @@ static gboolean check_trans_online_id(Transaction *trans1, void *user_data)
   if ((trans1 == trans2) || (online_id1 == NULL) || 
       (online_id2 == NULL) || (strcmp(online_id1, online_id2) != 0))
     {
-      return TRUE;
+      return 0;
     }
   else
     {
       /*printf("test_trans_online_id(): Duplicate found\n");*/
-      return FALSE;
+      return 1;
     }
 }
 
@@ -1008,7 +1013,7 @@ gboolean gnc_import_exists_online_id (Transaction *trans)
     {
       /* DEBUG("%s%d%s","Checking split ",i," for duplicates"); */
       dest_acct = xaccSplitGetAccount(source_split);
-      online_id_exists = !xaccAccountForEachTransaction(dest_acct,
+      online_id_exists = xaccAccountForEachTransaction(dest_acct,
 							check_trans_online_id,
 							trans);
     }
