@@ -13,8 +13,8 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
 \********************************************************************/
 /** @addtogroup Import_Export
     @{ */
@@ -27,7 +27,6 @@
 #ifndef TRANSACTION_MATCHER_H
 #define TRANSACTION_MATCHER_H
 
-#include <gtk/gtk.h>
 #include "Transaction.h"
 #include "import-match-map.h"
 #include "import-settings.h"
