@@ -6,10 +6,8 @@
  *********************************************************************/
 
 #include <stdio.h>
-#include <guile/gh.h>
+#include <libguile.h>
 #include <glib.h>
-#include <libguile/strports.h>
-#include <libguile/modules.h>
 
 #include "gnc-module.h"
 #include "gnc-module-api.h"
