@@ -8,24 +8,31 @@
 (define-module (gnucash report report-gnome))
 (use-modules (gnucash main)) ;; FIXME: delete after we finish modularizing.
 (use-modules (gnucash gnc-module))
+(use-modules (gnucash gnome-utils))
 
 (use-modules (ice-9 slib))
 (require 'printf)
 
+(use-modules (g-wrapped gw-core-utils))
 (use-modules (g-wrapped gw-report-gnome))
 
 (gnc:module-load "gnucash/gnome-utils" 0)
 (gnc:module-load "gnucash/report/report-system" 0)
+(gnc:module-load "gnucash/report/utility-reports" 0)
 
 (export gnc:report-edit-options)
 (export gnc:report-menu-setup)
 (export gnc:add-report-template-menu-items)
 
+;; returns a function that takes a list: (options, report),
+;; and returns a widget
 (define (gnc:report-options-editor report) 
   (if (equal? (gnc:report-type report) "Multicolumn View")
       gnc:column-view-edit-options
       gnc:default-options-editor))
 
+;; do not rely on the return value of this function - it has none.
+;; instead, this function's side-effect is to set the report's editor widget.
 (define (gnc:report-edit-options report) 
   (let* ((editor-widg (gnc:report-editor-widget report)))
     (if editor-widg
@@ -38,10 +45,9 @@
                        (gnc:report-options report)
                        report))
                 (gnc:report-set-editor-widget! report editor-widg))
-              (gnc:warning-dialog (_ "This report has no options.")))))))
+              (gnc:warning-dialog #f (_ "This report has no options.")))))))
 
 (define (gnc:add-report-template-menu-items)
-  (define menu-namer (gnc:new-menu-namer))
   (define *template-items* '())
 
   (define (add-template-menu-item name template)
@@ -51,12 +57,15 @@
               (menu-tip (gnc:report-template-menu-tip template))
               (item #f))
 
-          (if (not menu-path)
-              (set! menu-path '(""))
-              (set! menu-path
-                    (append menu-path '(""))))
+          ;;(if (not menu-path)
+              ;;(set! menu-path '(""))
+              ;;(set! menu-path
+              ;; (append menu-path '(""))))
 
-          (set! menu-path (append (list gnc:window-name-main gnc:menuname-reports) menu-path))
+          (if (not menu-path)
+              (set! menu-path '()))
+
+          (set! menu-path (append (list gnc:menuname-reports) menu-path))
 
           (if (not menu-tip)
               (set! menu-tip
@@ -64,13 +73,13 @@
 
           (set! item
                 (gnc:make-menu-item
-                 ((menu-namer 'add-name) name)
+                 name
                  menu-tip
                  menu-path
-                 (lambda ()
+                 (lambda (window)
                    (let ((report (gnc:make-report
                                   (gnc:report-template-name template))))
-                     (gnc:main-window-open-report report #f)))))
+                     (gnc:main-window-open-report report window)))))
           (gnc:add-extension item))))
 
   (define (add-template name template)
@@ -87,30 +96,17 @@
      (add-template-menu-item (car item) (cdr item)))
    (sort *template-items* sort-templates)))
 
-
 (define (gnc:report-menu-setup)
-  ;; since this menu gets added to every child window, we say it 
-  ;; comes after the "_Actions" menu. 
-  (define menu (gnc:make-menu gnc:menuname-reports
-			      (list gnc:window-name-main "_Actions")))
-  (define menu-namer (gnc:new-menu-namer))
   (define asset-liability-menu
-    (gnc:make-menu gnc:menuname-asset-liability
-                   (list gnc:window-name-main gnc:menuname-reports "")))
+    (gnc:make-menu gnc:menuname-asset-liability (list gnc:menuname-reports)))
   (define income-expense-menu
-    (gnc:make-menu gnc:menuname-income-expense
-                   (list gnc:window-name-main gnc:menuname-reports "")))
+    (gnc:make-menu gnc:menuname-income-expense (list gnc:menuname-reports)))
   (define utility-menu
-    (gnc:make-menu gnc:menuname-utility
-                   (list gnc:window-name-main gnc:menuname-reports "")))
+    (gnc:make-menu gnc:menuname-utility (list gnc:menuname-reports)))
   (define custom-menu
-    (gnc:make-menu gnc:menuname-custom
-                   (list gnc:window-name-main gnc:menuname-reports "")))
+    (gnc:make-menu gnc:menuname-custom (list gnc:menuname-reports)))
   (define tax-menu 
-    (gnc:make-menu gnc:menuname-taxes
-		   (list gnc:window-name-main gnc:menuname-reports "")))
-
-  (gnc:add-extension menu)
+    (gnc:make-menu gnc:menuname-taxes (list gnc:menuname-reports)))
 
   ;; (gnc:add-extension tax-menu)
   (gnc:add-extension income-expense-menu)
@@ -122,4 +118,14 @@
   (gnc:hook-run-danglers gnc:*report-hook*)
 
   ;; push reports (new items added on top of menu)
-  (gnc:add-report-template-menu-items))
+  (gnc:add-report-template-menu-items)
+
+  ;; the Welcome to GnuCash "extravaganza" report
+  (gnc:add-extension 
+   (gnc:make-menu-item 
+    (N_ "Welcome Sample Report")
+    (N_ "Welcome-to-GnuCash report screen")
+    (list gnc:menuname-reports gnc:menuname-utility "")
+    (lambda (window)
+      (gnc:main-window-open-report (gnc:make-welcome-report) window))))
+)
