@@ -16,27 +16,23 @@
 ;; along with this program; if not, contact:
 ;;
 ;; Free Software Foundation           Voice:  +1-617-542-5942
-;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
-;; Boston, MA  02111-1307,  USA       gnu@gnu.org
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (use-modules (gnucash main))
+(use-modules (g-wrapped gw-report-system)) 
 
 ;; This hash should contain all the reports available and will be used
 ;; to generate the reports menu whenever a new window opens and to
 ;; figure out what to do when a report needs to be generated.
 ;;
-;; The key is the string naming the report and the value is the 
-;; report definition structure.
+;; The key is the string naming the report (the report "type") and the
+;; value is the report definition structure.
 (define *gnc:_report-templates_* (make-hash-table 23))
 
-;; this is a hash of 'report ID' to instantiated report.  the 
-;; report id is generated at report-record creation time. 
-(define *gnc:_reports_* (make-hash-table 23))
-(define *gnc:_report-next-serial_* 0)
-
 ;; Define those strings here to make changes easier and avoid typos.
-(define gnc:menuname-reports (N_ "_Reports"))
+(define gnc:menuname-reports "Reports/StandardReports")
 (define gnc:menuname-asset-liability (N_ "_Assets & Liabilities"))
 (define gnc:menuname-income-expense (N_ "_Income & Expense"))
 (define gnc:menuname-taxes (N_ "_Taxes"))
@@ -47,6 +43,7 @@
 (define gnc:pagename-display (N_ "Display"))
 (define gnc:optname-reportname (N_ "Report name"))
 
+;; A <report-template> represents one of the available report types.
 (define <report-template>
   (make-record-type "<report-template>"
                     ;; The data items in a report record
@@ -55,6 +52,7 @@
                               renderer in-menu? menu-path menu-name
                               menu-tip export-types export-thunk)))
 
+;; if args is supplied, it is a list of field names and values
 (define (gnc:define-report . args)
   ;; For now the version is ignored, but in the future it'll let us
   ;; change behaviors without breaking older reports.
@@ -94,14 +92,25 @@
   (let ((report-rec (args-to-defn #f args)))
     (if (and report-rec
              (gnc:report-template-name report-rec))
-        (hash-set! *gnc:_report-templates_*
-                   (gnc:report-template-name report-rec) report-rec)
+	(let* ((name (gnc:report-template-name report-rec))
+	       (tmpl (hash-ref *gnc:_report-templates_* name)))
+	  (if (not tmpl)
+	      (hash-set! *gnc:_report-templates_*
+			 (gnc:report-template-name report-rec) report-rec)
+	      (begin
+		;; FIXME: We should pass the top-level window
+		;; instead of the #f to gnc:error-dialog, but I
+		;; have no idea where to get it from.
+		(gnc:error-dialog #f (string-append (_ "A custom report with this name already exists. Either rename the report to store it with a different name, or edit your saved-reports file and delete the section with the following name: ") name ))
+		)))
         (gnc:warn "gnc:define-report: bad report"))))
 
 (define gnc:report-template-version
   (record-accessor <report-template> 'version))
 (define gnc:report-template-name
   (record-accessor <report-template> 'name))
+(define gnc:report-template-set-name
+  (record-modifier <report-template> 'name))
 (define gnc:report-template-options-generator
   (record-accessor <report-template> 'options-generator))
 (define gnc:report-template-options-cleanup-cb
@@ -173,6 +182,7 @@
           (gnc:register-option options names)
           options))))
 
+;; A <report> represents an instantiation of a particular report type.
 (define <report>
   (make-record-type "<report>"
                     '(type id options dirty? needs-save? editor-widget ctext)))
@@ -221,12 +231,15 @@
 (define gnc:report-set-editor-widget!
   (record-modifier <report> 'editor-widget))
 
+;; ctext is for caching the rendered html
 (define gnc:report-ctext 
   (record-accessor <report> 'ctext))
 
 (define gnc:report-set-ctext!
   (record-modifier <report> 'ctext))
 
+;; gnc:make-report instantiates a report from a report-template.
+;; The actual report is stored away in a hash-table -- only the id is returned.
 (define (gnc:make-report template-name . rest)
   (let ((r ((record-constructor <report>) 
             template-name ;; type
@@ -238,9 +251,7 @@
             #f            ;; ctext
             ))
         (template (hash-ref *gnc:_report-templates_* template-name))
-        (id *gnc:_report-next-serial_*))
-    (gnc:report-set-id! r id)
-    (set! *gnc:_report-next-serial_* (+ 1 id))
+        )
     (let ((options 
            (if (not (null? rest))
                (car rest)
@@ -254,16 +265,17 @@
            (if cb
                (cb r))))
        options))
-    
-    (hash-set! *gnc:_reports_* (gnc:report-id r) r)
-    id))
 
+    (gnc:report-set-id! r (gnc:report-add r))
+    (gnc:report-id r))
+  )
+
+;; This is the function that is called when saved reports are evaluated.
 (define (gnc:restore-report id template-name options)
   (let ((r ((record-constructor <report>)
             template-name id options #t #t #f #f)))
-    (if (>= id *gnc:_report-next-serial_*)
-        (set! *gnc:_report-next-serial_* (+ id 1)))
-    (hash-set! *gnc:_reports_* id r)))
+    (gnc:report-add r))
+  )
 
 
 (define (gnc:make-report-options template-name)
@@ -272,6 +284,8 @@
         (gnc:report-template-new-options template)
         #f)))
 
+;; A convenience wrapper to get the report-template's export types from
+;; an instantiated report.
 (define (gnc:report-export-types report)
   (let ((template (hash-ref *gnc:_report-templates_* 
                             (gnc:report-type report))))
@@ -279,6 +293,8 @@
         (gnc:report-template-export-types template)
         #f)))
 
+;; A convenience wrapper to get the report-template's export thunk from
+;; an instantiated report.
 (define (gnc:report-export-thunk report)
   (let ((template (hash-ref *gnc:_report-templates_* 
                             (gnc:report-type report))))
@@ -295,9 +311,11 @@
         #f)))
 
 (define (gnc:report-name report) 
-  (gnc:option-value
-   (gnc:lookup-option (gnc:report-options report)
-                      gnc:pagename-general gnc:optname-reportname)))
+  (let* ((opt (gnc:report-options report)))
+    (if opt
+        (gnc:option-value
+         (gnc:lookup-option opt gnc:pagename-general gnc:optname-reportname))
+        #f)))
 
 (define (gnc:report-stylesheet report)
   (gnc:html-style-sheet-find 
@@ -324,26 +342,14 @@
     '() *gnc:_report-templates_*)
    string<?))
 
-(define (gnc:report-remove-by-id id)
-  (let ((r (hash-ref *gnc:_reports_* id)))
-    (for-each 
-     (lambda (child)
-       (gnc:report-remove-by-id child))
-     (gnc:report-children r))
-    (hash-remove! *gnc:_reports_* id)))
-
-(define (gnc:find-report id) 
-  (hash-ref *gnc:_reports_* id))
-
 (define (gnc:find-report-template report-type) 
   (hash-ref *gnc:_report-templates_* report-type))
 
 (define (gnc:report-generate-restore-forms report)
   ;; clean up the options if necessary.  this is only needed 
   ;; in special cases.  
-  (let* ((template 
-          (hash-ref  *gnc:_report-templates_* 
-                     (gnc:report-type report)))
+  (let* ((report-type (gnc:report-type report))
+         (template (hash-ref *gnc:_report-templates_* report-type))
          (thunk (gnc:report-template-options-cleanup-cb template)))
     (if thunk 
         (thunk report)))
@@ -385,6 +391,9 @@
     (gnc:report-name report)
     (gnc:report-type report))))
 
+(define gnc:current-saved-reports
+  (gnc:build-dotgnucash-path "saved-reports-2.0"))
+
 (define (gnc:report-save-to-savefile report)
   (let ((conf-file-name gnc:current-saved-reports))
     ;;(display conf-file-name)
@@ -392,30 +401,37 @@
 	     (open-file conf-file-name "a"))
     (force-output)))
 
+;; gets the renderer from the report template;
+;; gets the stylesheet from the report;
+;; renders the html doc and caches the resulting string;
+;; returns the html string.
 (define (gnc:report-render-html report headers?)
   (if (and (not (gnc:report-dirty? report))
            (gnc:report-ctext report))
       ;; if there's clean cached text, return it 
-      (begin 
-        (gnc:report-ctext report))
+      ;;(begin
+      (gnc:report-ctext report)
+      ;;  )
       
       ;; otherwise, rerun the report 
       (let ((template (hash-ref *gnc:_report-templates_* 
                                 (gnc:report-type report)))
 	    (doc #f))
         (set! doc (if template
-            (let* ((renderer (gnc:report-template-renderer template))
-                   (stylesheet (gnc:report-stylesheet report))
-                   (doc (renderer report))
-                   (html #f))
-              (gnc:html-document-set-style-sheet! doc stylesheet)
-              (set! html (gnc:html-document-render doc headers?))
-              (gnc:report-set-ctext! report html)
-              (gnc:report-set-dirty?! report #f)              
-              html)
-            #f))
-	doc)))
+                      (let* ((renderer (gnc:report-template-renderer template))
+                             (stylesheet (gnc:report-stylesheet report))
+                             (doc (renderer report))
+                             (html #f))
+                        (gnc:html-document-set-style-sheet! doc stylesheet)
+                        (set! html (gnc:html-document-render doc headers?))
+                        (gnc:report-set-ctext! report html) ;; cache the html
+                        (gnc:report-set-dirty?! report #f)  ;; mark it clean
+                        html)
+                      #f))
+	doc))) ;; YUK! inner doc is html-doc object; outer doc is a string.
 
+;; looks up the report by id and renders it with gnc:report-render-html
+;; marks the cursor busy during rendering; returns the html
 (define (gnc:report-run id)
   (let ((report (gnc:find-report id))
 	(start-time (gettimeofday))
@@ -434,6 +450,17 @@
     (gnc:unset-busy-cursor #f)
     html))
 
+
+;; "thunk" should take the report-type and the report template record
 (define (gnc:report-templates-for-each thunk)
   (hash-for-each (lambda (name template) (thunk name template))
                  *gnc:_report-templates_*))
+
+;; return the list of reports embedded in the specified report
+(define (gnc:report-embedded-list report)
+  (let* ((options (gnc:report-options report))
+	 (option (gnc:lookup-option options "__general" "report-list")))
+    (if option
+	(let ((opt-value (gnc:option-value option)))
+	  (map (lambda (x) (car x)) opt-value))
+	#f)))
