@@ -24,8 +24,8 @@
 ;; along with this program; if not, contact:
 ;;
 ;; Free Software Foundation           Voice:  +1-617-542-5942
-;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
-;; Boston, MA  02111-1307,  USA       gnu@gnu.org
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -56,6 +56,8 @@
 (define optname-sec-date-subtotal (N_ "Secondary Subtotal for Date Key"))
 (define optname-void-transactions (N_ "Void Transactions?"))
 (define optname-table-export (N_ "Table for Exporting"))
+(define optname-common-currency (N_ "Common Currency"))
+(define optname-currency (N_ "Report Currency"))
 (define def:grand-total-style "grand-total")
 (define def:normal-row-style "normal-row")
 (define def:alternate-row-style "alternate-row")
@@ -81,19 +83,29 @@
   (= (gnc:split-compare-other-account-codes a b) 0))
 
 (define (timepair-same-year tp-a tp-b)
-  (= (tm:year (gnc:timepair->date tp-a))
-     (tm:year (gnc:timepair->date tp-b))))
+  (= (gnc:timepair-get-year tp-a)
+     (gnc:timepair-get-year tp-b)))
+
+(define (timepair-same-quarter tp-a tp-b)
+  (and (timepair-same-year tp-a tp-b) 
+       (= (gnc:timepair-get-quarter tp-a)
+          (gnc:timepair-get-quarter tp-b))))
 
 (define (timepair-same-month tp-a tp-b)
   (and (timepair-same-year tp-a tp-b) 
-       (= (tm:mon (gnc:timepair->date tp-a))
-          (tm:mon (gnc:timepair->date tp-b)))))
+       (= (gnc:timepair-get-month tp-a)
+          (gnc:timepair-get-month tp-b))))
 
 (define (split-same-month-p a b)
   (let ((tp-a (gnc:transaction-get-date-posted (gnc:split-get-parent a)))
         (tp-b (gnc:transaction-get-date-posted (gnc:split-get-parent b))))
     (timepair-same-month tp-a tp-b)))
 
+(define (split-same-quarter-p a b)
+  (let ((tp-a (gnc:transaction-get-date-posted (gnc:split-get-parent a)))
+        (tp-b (gnc:transaction-get-date-posted (gnc:split-get-parent b))))
+    (timepair-same-quarter tp-a tp-b)))
+
 (define (split-same-year-p a b)
   (let ((tp-a (gnc:transaction-get-date-posted (gnc:split-get-parent a)))
         (tp-b (gnc:transaction-get-date-posted (gnc:split-get-parent b))))
@@ -158,15 +170,24 @@
                         table width subheading-style)))
 
 (define (render-month-subheading split table width subheading-style column-vector)
-  (let ((tm (gnc:timepair->date (gnc:transaction-get-date-posted
-                                 (gnc:split-get-parent split)))))
-    (add-subheading-row (strftime "%B %Y" tm)
-                        table width subheading-style)))
+  (add-subheading-row (gnc:date-get-month-year-string
+                      (gnc:timepair->date 
+                       (gnc:transaction-get-date-posted
+                        (gnc:split-get-parent split))))
+                     table width subheading-style))
+
+(define (render-quarter-subheading split table width subheading-style column-vector)
+  (add-subheading-row (gnc:date-get-quarter-year-string 
+                      (gnc:timepair->date 
+                       (gnc:transaction-get-date-posted
+                        (gnc:split-get-parent split))))
+                     table width subheading-style))
 
 (define (render-year-subheading split table width subheading-style column-vector)
-  (add-subheading-row (strftime "%Y" (gnc:timepair->date
-                                      (gnc:transaction-get-date-posted
-                                       (gnc:split-get-parent split))))
+  (add-subheading-row (gnc:date-get-year-string 
+                      (gnc:timepair->date 
+                       (gnc:transaction-get-date-posted
+                        (gnc:split-get-parent split))))
                       table width subheading-style))
 
 
@@ -227,10 +248,18 @@
   (let ((tm (gnc:timepair->date (gnc:transaction-get-date-posted
                                  (gnc:split-get-parent split)))))
     (add-subtotal-row table width 
-                      (total-string (strftime "%B %Y" tm))
+                      (total-string (gnc:date-get-month-year-string tm))
                       total-collector subtotal-style export?)))
 
 
+(define (render-quarter-subtotal
+         table width split total-collector subtotal-style column-vector export?)
+  (let ((tm (gnc:timepair->date (gnc:transaction-get-date-posted
+                                 (gnc:split-get-parent split)))))
+    (add-subtotal-row table width 
+                      (total-string (gnc:date-get-quarter-year-string tm))
+                     total-collector subtotal-style export?)))
+
 (define (render-year-subtotal
          table width split total-collector subtotal-style column-vector export?)
   (let ((tm (gnc:timepair->date (gnc:transaction-get-date-posted
@@ -374,8 +403,13 @@
         (addto! heading-list (_ "Balance")))
     (reverse heading-list)))
 
-(define (add-split-row table split column-vector
+(define (add-split-row table split column-vector options
                        row-style account-types-to-reverse transaction-row?)
+
+  (define (opt-val section name)
+    (gnc:option-value 
+     (gnc:lookup-option options section name)))
+
   (let* ((row-contents '())
 	 (dummy  (gnc:debug "split is originally" split))
          (parent (gnc:split-get-parent split))
@@ -385,14 +419,24 @@
          (currency (if account
                        (gnc:account-get-commodity account)
                        (gnc:default-currency)))
+	 (report-currency (if (opt-val gnc:pagename-general optname-common-currency)
+			       (opt-val gnc:pagename-general optname-currency)
+			       currency))
          (damount (if (gnc:split-voided? split)
 					 (gnc:split-void-former-amount split)
 					 (gnc:split-get-amount split)))
-         (split-value (gnc:make-gnc-monetary 
-                       currency 
-                       (if (member account-type account-types-to-reverse) 
-                           (gnc:numeric-neg damount)
-                           damount))))
+	 (trans-date (gnc:transaction-get-date-posted parent))
+	 (split-value (gnc:exchange-by-pricedb-nearest
+		       (gnc:make-gnc-monetary 
+			currency
+			(if (member account-type account-types-to-reverse) 
+			    (gnc:numeric-neg damount)
+			    damount))
+		       report-currency
+		       ;; Use midday as the transaction time so it matches a price
+		       ;; on the same day.  Otherwise it uses midnight which will
+		       ;; likely match a price on the previous day
+		       (gnc:timepair-canonical-day-time trans-date))))
     
     (if (used-date column-vector)
         (addto! row-contents
@@ -438,23 +482,23 @@
     (if (used-price column-vector)
         (addto! 
          row-contents 
-         (gnc:make-gnc-monetary currency
+         (gnc:make-gnc-monetary (gnc:transaction-get-currency parent)
                                 (gnc:split-get-share-price split))))
     (if (used-amount-single column-vector)
         (addto! row-contents
                 (gnc:make-html-table-cell/markup "number-cell"
-                                                 split-value)))
+                                                 (gnc:html-transaction-anchor parent split-value))))
     (if (used-amount-double-positive column-vector)
         (if (gnc:numeric-positive-p (gnc:gnc-monetary-amount split-value))
             (addto! row-contents
                     (gnc:make-html-table-cell/markup "number-cell"
-                                                     split-value))
+                                                     (gnc:html-transaction-anchor parent split-value)))
             (addto! row-contents " ")))
     (if (used-amount-double-negative column-vector)
         (if (gnc:numeric-negative-p (gnc:gnc-monetary-amount split-value))
             (addto! row-contents
                     (gnc:make-html-table-cell/markup
-                     "number-cell" (gnc:monetary-neg split-value)))
+                     "number-cell" (gnc:html-transaction-anchor parent (gnc:monetary-neg split-value))))
             (addto! row-contents " ")))
     (if (used-running-balance column-vector)
 	(begin
@@ -496,91 +540,95 @@
                   (N_ "Single")
                   (N_ "Display 1 line")))))
 
+  (gnc:register-trep-option
+   (gnc:make-complex-boolean-option
+    gnc:pagename-general optname-common-currency
+    "e" (N_ "Convert all transactions into a common currency") #f
+    #f
+    (lambda (x) (gnc:option-db-set-option-selectable-by-name
+		 gnc:*transaction-report-options*
+		 gnc:pagename-general
+		 optname-currency
+		 x))
+    ))
+
+  (gnc:options-add-currency!
+   gnc:*transaction-report-options* gnc:pagename-general optname-currency "f")
+
   (gnc:register-trep-option
    (gnc:make-simple-boolean-option
     gnc:pagename-general optname-table-export
-    "e" (N_ "Formats the table suitable for cut & paste exporting with extra cells") #f))  
+    "g" (N_ "Formats the table suitable for cut & paste exporting with extra cells") #f))  
   
   ;; Accounts options
   
   ;; account to do report on
   (gnc:register-trep-option
    (gnc:make-account-list-option
-        gnc:pagename-accounts (N_ "Report Accounts")
-        "a" (N_ "Report on these accounts")
+    gnc:pagename-accounts (N_ "Report Accounts")
+    "a" (N_ "Report on these accounts")
+    ;; select, by default, all accounts...
+    (lambda ()
+      (gnc:filter-accountlist-type 
+       '(bank cash credit asset liability stock mutual-fund currency
+	      payable receivable equity income expense)
+       (gnc:group-get-subaccounts (gnc:get-current-group))))
+    #f #t))
+
+  (gnc:register-trep-option
+   (gnc:make-account-list-option
+    gnc:pagename-accounts (N_ "Filter Accounts")
+    "b" (N_ "Filter on these accounts")
     (lambda ()
       ;; FIXME : gnc:get-current-accounts disappeared.
       (let ((current-accounts '())
-            (num-accounts (gnc:group-get-num-accounts
-                           (gnc:get-current-group)))
-            (first-account (gnc:group-get-account
-                            (gnc:get-current-group) 0)))
-        (cond ((not (null? current-accounts))
-               (list (car current-accounts)))
-              ((> num-accounts 0) (list first-account))
-              (else ()))))
+	    (num-accounts (gnc:group-get-num-accounts
+			   (gnc:get-current-group)))
+	    (first-account (gnc:group-get-account
+			    (gnc:get-current-group) 0)))
+	(cond ((not (null? current-accounts))
+	       (list (car current-accounts)))
+	      ((> num-accounts 0) (list first-account))
+	      (else ()))))
     #f #t))
-  
-      (gnc:register-trep-option
-       (gnc:make-account-list-option
-        gnc:pagename-accounts (N_ "Filter Accounts")
-        "b" (N_ "Filter on these accounts")
-        (lambda ()
-          ;; FIXME : gnc:get-current-accounts disappeared.
-          (let ((current-accounts '())
-                (num-accounts (gnc:group-get-num-accounts
-                               (gnc:get-current-group)))
-                (first-account (gnc:group-get-account
-                                (gnc:get-current-group) 0)))
-            (cond ((not (null? current-accounts))
-                   (list (car current-accounts)))
-                  ((> num-accounts 0) (list first-account))
-                  (else ()))))
-        #f #t))
 
-      (gnc:register-trep-option
-       (gnc:make-multichoice-option
-        gnc:pagename-accounts (N_ "Filter Type")
-        "c" (N_ "Filter account")
-        'none
-        (list (vector 'none
-                      (N_ "None")
-                      (N_ "Do not do any filtering"))
-              (vector 'include
-                      (N_ "Include Transactions to/from Filter Accounts")
-                      (N_ "Include transactions to/from filter accounts only"))
-              (vector 'exclude
-                      (N_ "Exclude Transactions to/from Filter Accounts")
-                      (N_ "Exclude transactions to/from all filter accounts"))
-	      )))
+  (gnc:register-trep-option
+   (gnc:make-multichoice-option
+    gnc:pagename-accounts (N_ "Filter Type")
+    "c" (N_ "Filter account")
+    'none
+    (list (vector 'none
+		  (N_ "None")
+		  (N_ "Do not do any filtering"))
+	  (vector 'include
+		  (N_ "Include Transactions to/from Filter Accounts")
+		  (N_ "Include transactions to/from filter accounts only"))
+	  (vector 'exclude
+		  (N_ "Exclude Transactions to/from Filter Accounts")
+		  (N_ "Exclude transactions to/from all filter accounts"))
+	  )))
 
+  ;;
 
-      ;;
+  (gnc:register-trep-option
+   (gnc:make-multichoice-option
+    gnc:pagename-accounts optname-void-transactions
+    "d" (N_ "How to handle void transactions")
+    'non-void-only
+    (list (vector
+	   'non-void-only
+	   (N_ "Non-void only")
+	   (N_ "Show only non-voided transactions"))
+	  (vector
+	   'void-only
+	   (N_ "Void only")
+	   (N_ "Show only voided transactions"))
+	  (vector 
+	   'both
+	   (N_ "Both")
+	   (N_ "Show both (and include void transactions in totals)")))))
 
-      (gnc:register-trep-option
-       (gnc:make-multichoice-option
-	gnc:pagename-accounts optname-void-transactions
-	"d" (N_ "How to handle void transactions")
-	'non-void-only
-	(list (vector
-	       'non-void-only
-	       (N_ "Non-void only")
-	       (N_ "Show only non-voided transactions"))
-	      (vector
-	       'void-only
-	       (N_ "Void only")
-	       (N_ "Show only voided transactions"))
-	      (vector 
-	       'both
-	       (N_ "Both")
-		(N_ "Show both (and include void transactions in totals)")))))
-	     
-	
-	'void-only
-	'all
   ;; Sorting options
-  
-
       
   (let ((options gnc:*transaction-report-options*)
 
@@ -636,7 +684,7 @@
                (vector 'memo
                        (N_ "Memo")
                        (N_ "Sort by memo"))))
-        ;;description))
+
         (ascending-choice-list 
          (list
           (vector 'ascend
@@ -645,11 +693,13 @@
           (vector 'descend
                   (N_ "Descending")
                   (N_ "largest to smallest, latest to earliest"))))
+
         (subtotal-choice-list
          (list
           (vector 'none (N_ "None") (N_ "None"))
           ;;(vector 'weekly (N_ "Weekly") (N_ "Weekly"))
           (vector 'monthly (N_ "Monthly") (N_ "Monthly"))
+          (vector 'quarterly (N_ "Quarterly") (N_ "Quarterly"))
           (vector 'yearly (N_ "Yearly") (N_ "Yearly")))))
     
     ;; primary sorting criterion
@@ -872,7 +922,7 @@ Credit Card, and Income accounts")))))
               ((equal? current split)
                (other-rows-driver split parent table used-columns (+ i 1)))
               (else (begin
-                      (add-split-row table current used-columns
+                      (add-split-row table current used-columns options
                                      row-style account-types-to-reverse #f)
                       (other-rows-driver split parent table used-columns
                                          (+ i 1)))))))
@@ -908,9 +958,9 @@ Credit Card, and Income accounts")))))
            (list
             (gnc:make-html-table-cell/size
              1 width (gnc:make-html-text (gnc:html-markup-hr)))))
-
-          (render-grand-total table width total-collector export?))
-
+	  (if (gnc:option-value (gnc:lookup-option options "Display" "Totals"))
+	      (render-grand-total table width total-collector export?)))
+	
         (let* ((current (car splits))
                (current-row-style (if multi-rows? def:normal-row-style
                                       (if odd-row? def:normal-row-style 
@@ -921,7 +971,8 @@ Credit Card, and Income accounts")))))
                (split-value (add-split-row 
                              table 
                              current 
-                             used-columns 
+                             used-columns
+			     options
                              current-row-style
                              account-types-to-reverse
                              #t)))
@@ -1110,6 +1161,8 @@ Credit Card, and Income accounts")))))
      (cons 'none (vector #f #f #f))
      (cons 'monthly (vector split-same-month-p render-month-subheading 
                             render-month-subtotal))
+     (cons 'quarterly (vector split-same-quarter-p render-quarter-subheading 
+                            render-quarter-subtotal))
      (cons 'yearly (vector split-same-year-p render-year-subheading
                            render-year-subtotal))))
 
@@ -1240,9 +1293,9 @@ Credit Card, and Income accounts")))))
                                          #t)
 
 	  (case void-status
-	   (('non-void-only) 
+	   ((non-void-only) 
 	    (gnc:query-set-match-non-voids-only! query (gnc:get-current-book)))
-	   (('void-only)
+	   ((void-only)
 	    (gnc:query-set-match-voids-only! query (gnc:get-current-book)))
 	   (else #f))
 
@@ -1315,7 +1368,8 @@ Credit Card, and Income accounts")))))
                   (_ "No matching transactions found"))
                  (gnc:html-markup-p
                   (_ "No transactions were found that \
-match the given time interval and account selection.")))
+match the time interval and account selection specified \
+in the Options panel.")))
                 (gnc:html-document-add-object! document p))))
 
         ;; error condition: no accounts specified
