@@ -9,8 +9,9 @@
 
 (use-modules (gnucash main)) ;; FIXME: delete after we finish modularizing.
 (use-modules (gnucash report hello-world))
-(use-modules (gnucash report iframe-url))
 (use-modules (gnucash report view-column))
 (use-modules (gnucash report welcome-to-gnucash))
 
-(export gnc:make-welcome-report)
+(use-modules (gnucash report test-graphing))
+
+(re-export gnc:make-welcome-report)
